/***************************************************************************************************/
/***************************************************************************************************/
//! \file 2D_bsp.h
//! \brief dfini les classes pour implmanter une BSP 2D avec des murs comme dividers
//!    
//!	Auteur : Adrien Auclair
//!
//! En cours de codage, seules les fonctions membres hrites ont t crites \n
//!	On preferera utiliser HV_bsp.h qui dcrit les murs horizontaux et verticaux et \n
//!	des pieces rectangulaires
//!
/***************************************************************************************************/
/***************************************************************************************************/
#ifndef D_BSP_H
#define D_BSP_H

/*------------------------------------  Prototypes  ---------------------------------------------*/
class cWall;              
class cArea_Describer;


/*------------------------------------   Includes   ---------------------------------------------*/
#include <math.h>

#include "geo_utils.h"
#include "bsp.h"
#include "ddutil.h"




/*-------------------------------------   Classes   ---------------------------------------------*/

/*************************************************************************************************/
//! un wall est un divider pour diviser le plan en deux sous-espaces
/*! cette classe gere tous les murs (ie. pas forcement verticaux ou horizontaux)  \n    
**************************************************************************************************/
class cWall
{
	friend std::ostream &operator<<(std::ostream &os, const cWall &ex);// { return os << ex.p0 << "-->" << ex.p1; };
public:
	/// constructeur par defaut, construit un mur entre (0,0) et (0,0)
	cWall();

	/// copy constructeur
	cWall(cWall& wall);
	
	///constucteur, le mur va de A  Bdgd
	cWall( cPoint2D & A, cPoint2D & B);

	// trace la ligne du mur dans le back buffer de display
	void Display( CDisplay * display );

	/// retourne la position du mur 'w' par rapport  ce mur
	BSP_POSITION_DIVIDER Get_Position_Divider( cWall & w );

	/// retourne la position du mur 'w' par rapport  ce mur et le split si necessaire
	/** si la position est 'DIVIDER_CROSS' alors \n
	 *  'w_front' contiendra la partie de 'w' devant ce mur \n
	 *  'w_back' contiendra la partie de 'w' derriere ce mur \n
	*/	
	BSP_POSITION_DIVIDER Get_Position_Divider_And_Split( cWall & w , cWall & w_front  , cWall & w_back );
	
	/// retourne la longueur du mur
	double Get_Length();
	
	cPoint2D p0;    ///< point de depart du mur
	cPoint2D p1;	///< point d'arrive du mur

private:	
	/// retourne la position du poit 'p' par rapport  ce mur
	virtual BSP_POSITION_POINT Get_Position_Point( cPoint2D& p) 
	{ 
		std::cout << "BSP_POSITION_POINT cWall::Get_Position_Point( cPoint2D& p) not implemented yet !! \n";
		return POINT_ON;
	}
};

/*************************************************************************************************/
//! classe dcrivant un sous-espaces non divis dans l'arbre BSP
/*! c'est  dire une region de l'espace
**************************************************************************************************/
class cArea_Describer
{
public:
	inline cArea_Describer() { Id=-1;}
	inline cArea_Describer(int id) { Id=id;}
	inline Get_Id() { return Id; }
private:
	int Id;
};


#endif