/*------------------------------------   Includes   ---------------------------------------------*/
#include "HV_bsp.h"

using namespace std;

	

/*--------------------------------------------------------------------------------------------------*/
// met le segment p0<-->p_inter dans w_front
// met le segment p_inter<-->p1 dans w_back
static void Build_Two_Segments( cPoint2D & p0 , cPoint2D & p1 ,cPoint2D& p_inter ,cHV_Wall& w_front , cHV_Wall& w_back)
{
	w_front = cHV_Wall(p0 , p_inter);
	w_back  = cHV_Wall( cPoint2D(p_inter.Get_X(),p_inter.Get_Y()) , p1);
}

/*--------------------------------------------------------------------------------------------------*/
std::ostream &operator<<(std::ostream &os, const cHV_Wall &ex) 
{ 
	return os << ex.p0 << "-->" << ex.p1; 
};


/****************************************************************************************************/
// class cHV_Wall
/****************************************************************************************************/
/*--------------------------------------------------------------------------------------------------*/
cHV_Wall::cHV_Wall():cWall() 
{		
	Set_HV();		
	Set_Order();
}

/*--------------------------------------------------------------------------------------------------*/
cHV_Wall::cHV_Wall(cWall& wall):cWall(wall) 
{
	Set_HV();
	Set_Order();
}



/*--------------------------------------------------------------------------------------------------*/
cHV_Wall::cHV_Wall( cPoint2D &A, cPoint2D &B):cWall(A,B) 
{
	Set_HV();
	Set_Order();
	if( Get_Length()<0.1 ) 
	{
		std::cout << "WARNING : cHV_Wall( cPoint2D &A, cPoint2D &B):cWall(A,B) \n";
		std::cout << "le cHV_Wall a une longueur tres petite ... \n";
		std::cout << (*this) << "\n";
	}
}

/*--------------------------------------------------------------------------------------------------*/
void cHV_Wall::Set_HV()
{
	if( p0.Get_X() == p1.Get_X() )
		bHorizontal = false;
	else if( p0.Get_Y() == p1.Get_Y() )
		bHorizontal = true;
	else
	{
		cout << "ERROR : void cHV_Wall::Set_HV()";
		cout << (*this) << " is not an valid HV_wall !!! \n";
	}
}

/*--------------------------------------------------------------------------------------------------*/
void cHV_Wall::Get_Wall_Seg(cHV_Seg & wall)
{
	wall.Set(p0 , p1);
}


/*--------------------------------------------------------------------------------------------------*/
BSP_POSITION_POINT cHV_Wall::Get_Position_Point( cPoint2D & A )
{
	if( bHorizontal)
	{
		if( A.Get_Y() < this->p0.Get_Y() )
			return POINT_FRONT;
		else if( A.Get_Y() > this->p0.Get_Y() )
			return POINT_BACK;
		else
			return POINT_ON;
	}
	else
	{
		if( A.Get_X() < this->p0.Get_X() )
			return POINT_FRONT;
		else if( A.Get_X() > this->p0.Get_X() )
			return POINT_BACK;
		else
			return POINT_ON;
	}
}


/*--------------------------------------------------------------------------------------------------*/
void cHV_Wall::Set_Order()
{
	int t;
	if( bHorizontal)
	{
		if( p0.Get_X() > p1.Get_X() )
		{			
			t = p0.Get_X();
			p0.Set(p1.Get_X() , p1.Get_Y() );
			p1.Set( t , p1.Get_Y() );			
		}
	}
	else		
	{
		if( p0.Get_Y() > p1.Get_Y() )
		{			
			t = p0.Get_Y();
			p0.Set(p1.Get_X() , p1.Get_Y() );
			p1.Set( p1.Get_X() , t);			
		}
	}
}




/*--------------------------------------------------------------------------------------------------*/
void cHV_Wall::Get_Intersection( cHV_Wall& w ,cPoint2D&  p_inter)
{
	if( bHorizontal)
	{
		if( w.bHorizontal)
		{
			cout << "ERROR : void Get_Intersection( cHV_Wall& w ,cPoint2D&  ) \n";
			cout << " les deux droites sont horizontales !! \n";
			cout << "d1 =" << (*this) << "\n";
			cout << "d2 =" << w << "\n";
		}
		else
		{
			p_inter.Set( w.p0.Get_X(),this->p0.Get_Y() );
		}
	}
	else
	{
		if( w.bHorizontal)
		{
			p_inter.Set( this->p0.Get_X(),w.p0.Get_Y() );
		}
		else
		{
			cout << "ERROR : void Get_Intersection( cHV_Wall& w ,cPoint2D&  ) \n";
			cout << " les deux droites sont verticales !! \n";
			cout << "d1 =" << (*this) << "\n";
			cout << "d2 =" << w << "\n";
		}
	}
}

/*--------------------------------------------------------------------------------------------------*/
BSP_POSITION_DIVIDER cHV_Wall::Get_Position_Divider_And_Split( cHV_Wall & w , cHV_Wall & w_front  , cHV_Wall & w_back ) 
{
	BSP_POSITION_POINT pos0 = Get_Position_Point( w.p0) ;
	BSP_POSITION_POINT pos1 = Get_Position_Point( w.p1) ;		
	cPoint2D p_inter;	

	switch( pos0)
	{
	case POINT_FRONT:
		switch(pos1)
		{
		case POINT_FRONT:
			return DIVIDER_FRONT;
		case POINT_BACK:
			Get_Intersection( w , p_inter);			
			Build_Two_Segments( w.p0 , w.p1 , p_inter , w_front , w_back);
			return DIVIDER_CROSS;
		case POINT_ON:
			return DIVIDER_FRONT;
		default:
			cout << "ERROR : BSP_POSITION_DIVIDER cHV_Wall::Get_Position_Divider_And_Split( cHV_Wall & w , cHV_Wall & w_front  , cHV_Wall & w_back ) \n"; 			
			return DIVIDER_ON;  // pour eviter le warning "path without return"
		}
		break;
	case POINT_BACK:
		switch(pos1)
		{
		case POINT_FRONT:
			Get_Intersection( w , p_inter);	
			Build_Two_Segments( w.p1 , w.p0 , p_inter , w_front , w_back);							
			return DIVIDER_CROSS;
		case POINT_BACK:
			return DIVIDER_BACK;
		case POINT_ON:
			return DIVIDER_BACK;
		default:
			cout << "ERROR : BSP_POSITION_DIVIDER cHV_Wall::Get_Position_Divider_And_Split( cHV_Wall & w , cHV_Wall & w_front  , cHV_Wall & w_back ) \n"; 			
			return DIVIDER_ON;  // pour eviter le warning "path without return"
		}
		break;
	case POINT_ON:
		switch(pos1)
		{
		case POINT_FRONT:
			return DIVIDER_ON;
		case POINT_BACK:
			return DIVIDER_BACK;
		case POINT_ON:			
			cout << "ERROR : BSP_POSITION_DIVIDER cHV_Wall::Get_Position_Divider_And_Split( cHV_Wall & w , cHV_Wall & w_front  , cHV_Wall & w_back ) \n"; 
			cout << "les murs parallelles ne sont pas encore grs ! \n";
			cout << "cWall :" << w << " et " << (*this) << " \n";
			return DIVIDER_ON;  // pour eviter le warning "path without return"
		default:
			cout << "ERROR : BSP_POSITION_DIVIDER cHV_Wall::Get_Position_Divider_And_Split( cHV_Wall & w , cHV_Wall & w_front  , cHV_Wall & w_back ) \n"; 			
			return DIVIDER_ON;  // pour eviter le warning "path without return"
		}
		break;
	default:
		cout << "ERROR : BSP_POSITION_DIVIDER cHV_Wall::Get_Position_Divider_And_Split( cHV_Wall & w , cHV_Wall & w_front  , cHV_Wall & w_back ) \n"; 		
		return DIVIDER_ON;  // pour eviter le warning "path without return"
	}
}