/***************************************************************************************************/
/***************************************************************************************************/
//! \file HV_bsp.h
//! \brief Utilise la class generique de bsp de bsp.h avec des murs horizontaux ou verticaux
//!    
//!	Auteur : Adrien Auclair
//!
//! pour faire un jeu simple  debuger, au vu des delais, on a choisi de ne mettre que des murs
//! horizontaux ou verticaux, plus simples  debuger, et  gomtrie exacte 
//! ( pas d'arrondi ds les calculs)
/***************************************************************************************************/
/***************************************************************************************************/
#ifndef HV_BSP_H
#define HV_BSP_H

/*------------------------------------  Prototypes  ---------------------------------------------*/
class cHV_Wall;

/*------------------------------------   Includes   ---------------------------------------------*/


#include "2D_bsp.h"
#include "bsp.h"


/*------------------------------------     Types    ---------------------------------------------*/

/// instanciation du trait cBSP_Node avec 'cHV_Wall' comme divider
typedef cBSP_Node< cHV_Wall> cHV_Node;

//! instanciation du trait cBSp_Tree pour contenir des 'cPoint2D', 
//! avec pour noeud des 'cHV_Node'
//! avec pour divider des 'cHV_Wall'
typedef cBSP_Tree< cPoint2D , cHV_Node , cHV_Wall > cHV_BSP_Tree;


/*-------------------------------------   Classes   ---------------------------------------------*/


/*************************************************************************************************/
//! un cHVWall est un mur bas sur un segment horizontal ou vertical
/*************************************************************************************************/
class cHV_Wall : public cWall
{
	/// envoie vers un ostream l'affichage du segment
	friend std::ostream &operator<<(std::ostream &os, const cHV_Wall &ex);// { return os << ex.p0 << "-->" << ex.p1; };		
public:		
	/// default contructeur ( mur(0,0)->(0,0) )
	cHV_Wall();                                             
	/// copy constructeur	
	cHV_Wall(cWall& wall);                                  
	/// construit un mur de A  B
	cHV_Wall( cPoint2D &A, cPoint2D &B);                    

	/// retourne la position d'un point par rapport  ce divider	
	BSP_POSITION_POINT Get_Position_Point( cPoint2D & A );  
	/// copie dans 'wall' le segment de ce murs
	void Get_Wall_Seg(cHV_Seg & wall);                      

	/// retourne true si horizontal, false si vertical
	inline bool Is_Horizontal()                             
	{ 
		return bHorizontal; 
	}		

	/// retourne la position du cHVWall 'w' par rapport  ce divider
	/** dans le cas ou la position est DIVIDER_CROSS, alors le divider 'w' est coup en deux \n
	*  'w_front' recoit la partie de 'w' qui devant \n
	*  'w_back' recoit la partie de 'w' qui derriere \n
	* les segments 'w_front' et 'w_back' ont alors pour intersection un unique point qui appartient a ce divider */
	BSP_POSITION_DIVIDER Get_Position_Divider_And_Split( cHV_Wall & w , cHV_Wall & w_front  , cHV_Wall & w_back );	

private:
	bool bHorizontal;                                       ///< true si horizontal, false si vertical

	/// met a jour la donne membre 'bHorizontal' selon les points p0 et p1 de ce mur
	/** imprime un message d'erreur si le mur n'est ni horizontal ni vertical*/
	void Set_HV();

	/// met les points p0 et p1 de ce mur dans l'ordre, c'est a dire que :
	/** pour un segment horizontal, p0 est a une abcisse infrieure  celle de p1 \n
	  * pour un segment vertical,   p0 est a une ordonne infrieure  celle de p1 */ 
	void Set_Order();	

	/// met dans 'p_inter' le point d'intersections entre les deux droites
	/** d1-> droite porte par ce mur \n
	* d2-> droite porte par 'w' \n
	* imprime un message d'erreur si les deux segments sont parallelles \n
	* TOTO : retourner false si pas d'intersection plutot qu'un message d'erreur */
	void Get_Intersection( cHV_Wall& w ,cPoint2D&  p_inter);	
};




#endif