/*------------------------------------   Includes   ---------------------------------------------*/
#include "a_star.h"


/*--------------------------------------------------------------------------------------------------*/
bool cList_A_Star_Nodes::Contain( cA_Star_Node* node)
{
	std::list<cA_Star_Node*>::iterator itr;
	for(itr = this->begin() ; itr != this->end() ; itr++)
		if( (*itr) == node) return true;

	return false;
}



/*--------------------------------------------------------------------------------------------------*/
void cA_Star_Priority_List::Add(cA_Star_Node* new_node)
{		
	if( empty() )
	{
		push_front(new_node);			
		return;
	}

	std::list<cA_Star_Node*>::iterator itr;  
	for( itr = this->begin() ; itr != this->end() ; itr++)
	{
		
		if( new_node->Get_Cost() < (*itr)->Get_Cost()  )
		{				
			insert(itr , new_node);				
			return;
		}
	}

	push_back(new_node);
	return;
}

/*--------------------------------------------------------------------------------------------------*/
bool cA_Star_Priority_List::Contain( cA_Star_Node* node)
{
	std::list<cA_Star_Node*>::iterator itr;
	for(itr = this->begin() ; itr != this->end() ; itr++)
		if( (*itr) == node)
			return true;
	return false;
}

/*--------------------------------------------------------------------------------------------------*/
void cA_Star_Search::Construct_Path( cA_Star_Node* destination , cList_A_Star_Nodes & chemin )
{
	assert( destination != NULL );
	while( destination->Path_Parent != NULL)
	{
		chemin.push_front( destination );
		destination = destination->Path_Parent;
	}
}

/*--------------------------------------------------------------------------------------------------*/
bool cA_Star_Search::Find_Path(  cA_Star_Node* start , cA_Star_Node* goal , cList_A_Star_Nodes & chemin  )
{
	assert( start != NULL );
	assert( goal  != NULL );

	cA_Star_Priority_List            Open_List;
	cA_Star_Priority_List::iterator  open_itr;
	cList_A_Star_Nodes                      Closed_List;
	cList_A_Star_Nodes::iterator            closed_itr;
	cList_A_Star_Nodes::iterator            voisins_itr;

	cA_Star_Node *                   current_node = NULL;
	cList_A_Star_Nodes                      Voisins_List;
	bool                             bIs_Open;
	bool                             bIs_Closed;	
	float                            Cost_From_Start;
	int                              n_voisins;

	start->Cost_From_Start        = 0;
	start->Estimated_Cost_To_Goal = start->Get_Estimated_Cost(goal);
	start->Path_Parent            = NULL;

	Open_List.Add(start);
		
	while( ! Open_List.empty() )
	{
		current_node = Open_List.front();  // on prends le noeud au meilleur cout ( open_list est trie !)
		Open_List.pop_front();

		if( current_node == goal)     // arrive ?
		{
			Construct_Path(goal , chemin);
			return true;
		}

		Closed_List.push_front( current_node);
		current_node->Get_List_Neighbors(Voisins_List);
		n_voisins = Voisins_List.size();	
		
		// on regarde les voisins de ce noeud pour les mettre a jour
		for( voisins_itr = Voisins_List.begin() ; voisins_itr != Voisins_List.end() ; voisins_itr++)
		{
			bIs_Open        = Open_List.Contain(*voisins_itr);
			bIs_Closed      = Closed_List.Contain(*voisins_itr);
			Cost_From_Start = current_node->Cost_From_Start + current_node->Get_Cost(*voisins_itr);				

			// si le voisin n'a pas ete travers, ou si l'on vient a ce voisin par un chemin plus court
			if( ( !bIs_Open && !bIs_Closed) || Cost_From_Start < (*voisins_itr)->Cost_From_Start )
			{
				(*voisins_itr)->Path_Parent            = current_node;
				(*voisins_itr)->Cost_From_Start        = Cost_From_Start;
				(*voisins_itr)->Estimated_Cost_To_Goal = (*voisins_itr)->Get_Estimated_Cost(goal);

				if( bIs_Closed)
					Closed_List.remove((*voisins_itr));

				if (!bIs_Open )
					Open_List.Add((*voisins_itr));
			}
		}			
	}
	return false;
};