/***************************************************************************************************/
/***************************************************************************************************/
//! \file bonus.h
//! \brief Dfini les objets interactifs du jeu, non mobiles
//!    
//!	Auteur : Adrien Auclair \n
//! \n
//! les objets interactifs sont circulaires ( pour les detection )
//! 
/***************************************************************************************************/
/***************************************************************************************************/
#ifndef BONUS_H
#define BONUS_H

/*------------------------------------  Prototypes  ---------------------------------------------*/
class cInteractive_Item;
class cBomb;
class cTeleporteur;
class cHologramme;

/*------------------------------------    Define    ---------------------------------------------*/
typedef enum { TELEPORTEUR , BOMB , HOLOGRAMME , CADEAU , RED_TRAP , BLUE_TRAP, GREEN_TRAP } Interactive_Item_Type;
typedef enum { CADEAU_POINTS , CADEAU_TELEPORTEUR , CADEAU_BOMB , CADEAU_HOLOGRAMME , CADEAU_COEUR , CADEAU_TIME } Cadeau_Type;

#define N_BONUS 7
#define RAYON_BOMB 10
#define RAYON_TELEPORTEUR 10
#define RAYON_HOLOGRAMME 10
#define RAYON_CADEAU 10

#define RAYON_RED_TRAP 10
#define RAYON_BLUE_TRAP 10
#define RAYON_GREEN_TRAP 10



/*------------------------------------   Includes   ---------------------------------------------*/
#include "display.h"




/*-------------------------------------   Classes   ---------------------------------------------*/


/*************************************************************************************************/
//! classe gnrique des objets immobiles interactifs
/**************************************************************************************************/
class cInteractive_Item
{
public:
	/// constructeur
	/**
	* px,py : position de l'objet \n
	* def   : pointeur vers une classe d'affichage
	* r     : rayon de l'objet
	* t     : type d'objet
	*/
	cInteractive_Item(int px, int py , cSprite_Display_Def *def, int r,Interactive_Item_Type t);

	/// affiche l'objet ds le backbuffer
	void Display() ;	

	/// retourne le rayon de l'objet
	inline int Get_Rayon() { return Rayon;};
	
	/// position de l'objet
	int Px,Py;

	Interactive_Item_Type Type;          ///< type de cet objet

	/// permet de remettre  son tat initial un cInteractive_Item
	virtual void Reset() {};
private:
	cSprite_Display_Def * pDisplay_Def;  ///< classe d'affichage
	int Rayon;                           ///< rayon pour les collisions
	
};



/*************************************************************************************************/
//! classe des bombes
/**************************************************************************************************/
class cBomb : public cInteractive_Item
{
public:	
	cBomb(int x, int y);

	/// remet le Time_Stamp de creation  l'instant present
	void Reset();

	/// date  laquelle a ete pose cette bombe
	float Time_Stamp;
};



/*************************************************************************************************/
//! classe des teleporteurs
/**************************************************************************************************/
class cTeleporteur: public cInteractive_Item
{
public:
	cTeleporteur(int x, int y);
};


/*************************************************************************************************/
//! classe des hologrammes
/**************************************************************************************************/
class cHologramme: public cInteractive_Item
{
public:
	cHologramme(int x, int y);
};

/*************************************************************************************************/
//! classe des cadeaux
/**************************************************************************************************/
class cCadeau : public cInteractive_Item
{
public:
	/// constructeur :
	/**
	* x,y : position \n
	* type : type de bonus qu'on cache ds ce cadeau
	*/
	cCadeau(int x, int y, Cadeau_Type type);
	
	/// retourne le type de bonus de ce cadeau
	inline Cadeau_Type Get_Inside() { return Inside; };
private:
	Cadeau_Type Inside;         ///< type de bonus cach ds le cadeau
};

/*************************************************************************************************/
//! classe des pieges  boules rouges
/**************************************************************************************************/
class cRed_Trap : public cInteractive_Item
{
public:
	cRed_Trap(int x, int y);
};


/*************************************************************************************************/
//! classe des pieges a boules bleues
/**************************************************************************************************/
class cBlue_Trap: public cInteractive_Item
{
public:
	cBlue_Trap(int x, int y);
};


/*************************************************************************************************/
//! classe des pieges  boules vertes
/**************************************************************************************************/
class cGreen_Trap: public cInteractive_Item
{
public:
	cGreen_Trap(int x, int y);
};



#endif