/*------------------------------------   Includes   ---------------------------------------------*/
#include "bresenham.h"
#include "main.h"






/****************************************************************************************************/
// class cIncremental_Bresenham
/****************************************************************************************************/
void cIncremental_Bresenham::Reset( cPoint2D & start, cPoint2D & goal )
{
	A   = start;
	B   = goal;

	/*----------------------- initialize the Bresenham algorithm -----------------*/
	sdx = goal.X - start.X;
	sdy = goal.Y - start.Y;
	dx  = goal.X - start.X;
	dy  = goal.Y - start.Y;

	Px  = start.X;
	Py  = start.Y;
	
	if(dx<0)
	{
		x_inc = -1;
		dx=-dx;	
	}
	else
		x_inc=1;

	if(dy<0)	
	{
		y_inc = -1;
		dy=-dy;
	}
	else
		y_inc = 1;

	// Pre-calcul du double de dx et dy
	dx2=dx*2;
	dy2=dy*2;

	if(dx>dy)	// La ligne est X-dominante
	{
		// Erreur de dpart
		error=dy2-dx;
	}
	else
		error=dx2-dy;
}


/*--------------------------------------------------------------------------------------------------*/
bool cIncremental_Bresenham::Step(int &length , cPoint2D & position )
{	
	bool arrived = false;
	int  lx      = 0;
	int  ly      = 0;	

	while( lx*lx+ly*ly < length)
	{
		if(dx>dy)	// La ligne est X-dominante
		{				
			// On dcale et on corrige l'erreur en cas de dbordement
			if(error>=0)
			{
				error-=dx2;
				// Ligne suivante
				Py+=y_inc;
				ly+=y_inc;
				
			}
			// Ajustement de l'erreur
			error+=dy2;
			// Pixel suivant
			Px+=x_inc;
			lx+=x_inc;			
		}	
		else	// La ligne est Y-dominante ou "qui-penche"
		{
			// On dcale et on corrige l'erreur en cas de dbordement
			if(error>=0)
			{
				error-=dy2;
				// Ligne suivante
				Px+=x_inc;
				lx+=x_inc;
			}
			// Ajustement de l'erreur
			error+=dx2;
			// Pixel suivant
			Py+=y_inc;
			ly+=y_inc;
		}
	}
	
	if( sdx >=0 && sdy>= 0 && Px >= B.X && Py >= B.Y )
		arrived=true;
	if( sdx >=0 && sdy<0   && Px >= B.X && Py <= B.Y )
		arrived=true;
	if( sdx <0 && sdy>= 0  && Px <= B.X && Py >= B.Y )
		arrived=true;
	if( sdx <0 && sdy< 0   && Px <= B.X && Py <= B.Y )
		arrived=true;
	
	position.X = Px;
	position.Y = Py;
	return arrived;
}


/*--------------------------------------------------------------------------------------------------*/
bool cIncremental_Bresenham::Step(cPoint2D & position )
{	
	bool arrived = false;
	int  lx      = 0;
	int  ly      = 0;	

	
	if(dx>dy)	// La ligne est X-dominante
	{			
		// On dcale et on corrige l'erreur en cas de dbordement
		if(error>=0)
		{
			error-=dx2;
			// Ligne suivante
			Py+=y_inc;
			ly+=y_inc;
			
		}
		// Ajustement de l'erreur
		error+=dy2;
		// Pixel suivant
		Px+=x_inc;
		lx+=x_inc;		
	}	
	else	// La ligne est Y-dominante ou "qui-penche"
	{
		// On dcale et on corrige l'erreur en cas de dbordement
		if(error>=0)
		{
			error-=dy2;
			// Ligne suivante
			Px+=x_inc;
			lx+=x_inc;
		}
		// Ajustement de l'erreur
		error+=dx2;
		// Pixel suivant
		Py+=y_inc;
		ly+=y_inc;
	}

	/*
	if( Px == B.X && Py == B.Y)	
		arrived=true;
		*/

	if( sdx >=0 && sdy>= 0 && Px >= B.X && Py >= B.Y )
		arrived=true;
	if( sdx >=0 && sdy<0   && Px >= B.X && Py <= B.Y )
		arrived=true;
	if( sdx <0 && sdy>= 0  && Px <= B.X && Py >= B.Y )
		arrived=true;
	if( sdx <0 && sdy< 0   && Px <= B.X && Py <= B.Y )
		arrived=true;
	
	position.X = Px;
	position.Y = Py;
	return arrived;
}

