/***************************************************************************************************/
/***************************************************************************************************/
//! \file bresenham.h
//! \brief Dfini l'algorithm de bresenham pour tracer une ligne discrete continue
//!    
//!	Auteur : Adrien Auclair
//!
/***************************************************************************************************/
/***************************************************************************************************/


#ifndef BRESENHAM_H
#define BRESENHAM_H

/*------------------------------------  Prototypes  ---------------------------------------------*/
class cIncremental_Bresenham;


/*------------------------------------   Includes   ---------------------------------------------*/
#include "geo_utils.h"


/*-------------------------------------   Classes   ---------------------------------------------*/



/*************************************************************************************************/
//! encapsule l'algo de bresenham
/*! permet d'initialiser l'algo et ensuite de faire des pas en appellant une fonction membre
**************************************************************************************************/
class cIncremental_Bresenham
{	
public:
	/// constructeur par defaut
	/** initiale les valeurs de l'algorithme de bresenham pour aller de (0,0)  (0,0)
	*/
	cIncremental_Bresenham() 
	{ 
		A=cPoint2D(0,0); 
		B=cPoint2D(0,0);
	};

	/// initialise les variables interne pour faire le parcours de start vers goal
	void Reset( cPoint2D & start, cPoint2D & goal );

	/// parcours la ligne du point 'start'  'goal' en appelant  chaque point 'Call_Back(x,y)'
	/** la fonction 'Call_Back' prends [x,y] comme parametres entier \n
	*   elle renvoie un booleen \n
	* si elle renvoie true, on continue le parcours \n
	* sinon, on stoppe le parcours */
	void Parcourir_Et_Executer( cPoint2D & start, cPoint2D & goal , bool (*Call_Back)(int x , int y) );

	/// effectue un pas dans l'algo de bresenham
	/** position contiendra en retour la position courante
	*/
	bool Step(cPoint2D & position );

	/// effectue un pas, de longueur 'length'
	/** si on atteint le point final avant de parcourir 'length' \n
	* en retour 'length' vaut la distance qu'on aurait pu parcourir en plus	\n
	* 'position' contient la position apres le pas	
	*/
	bool Step(int &length , cPoint2D & position );
	
	
private:

	cPoint2D A;                                   ///< point de depart
	cPoint2D B;                                   ///< point d'arrive
	
	int sdx,sdy,dx,dy,dx2,dy2,error,x_inc,y_inc;  ///< variables internes for the Bresenham algorithm
	int Px,Py;                                    ///< position courante de l'algorithme
};


#endif