#pragma warning(disable: 4786)

/*------------------------------------   Includes   ---------------------------------------------*/
#include <fstream>
#include <string>
#include <iostream>

#include "bsp_loader.h"


using namespace std;


typedef enum { READING_AREA , READING_WALL , READING_UNDEF } READING_STATE;


/*------------------------------------  Prototypes  ---------------------------------------------*/
bool Get_Pair(int &x, int &y , ifstream& in);




/*-----------------------------------------------------------------------------------------*/
// lit le texte "[x,y]" dans le stream 'in' et met a jour 'x' et 'y' avec les valeurs lues
// return true s'il n'y a pas eu d'erreur, false sinon
bool Get_Pair(int &x, int &y , ifstream& in)
{
	char c;
	
	in >> c;
	if( c != '[' )
		return false;
	in >> x;
	in >> c;
	if( c != ',' )
		return false;
	in >> y;
	in >> c;
	if( c != ']' )
		return false;

	return true;
}



/****************************************************************************************************/
// class cBSP_Loader
/****************************************************************************************************/
/*-----------------------------------------------------------------------------------------*/
bool cBSP_Loader::Read_Area(ifstream& in,  cGame_World * world)
{	
	int x,y;
	if( !Get_Pair( x, y , in))
		return false;

	cGame_World::Ecran = cPositive_Rectangle( cPoint2D(0,0) , cPoint2D(x,y));
	return true;
}


/*-----------------------------------------------------------------------------------------*/
bool cBSP_Loader::Read_Wall(ifstream& in ,  cGame_World * world)
{
	char c;	
	int x0,y0,x1,y1;
	
	if( !Get_Pair( x0, y0 , in))
		return false;

	// on lit la fleche entre les deux points
	in >> c;
	if( c!='-' )
		return false;
	in >> c;
	if( c!='>' )
		return false;

	if( !Get_Pair( x1, y1 , in))
		return false;	

	world->My_BSP.Add_Divider( 	cHV_Wall(cPoint2D(x0,y0),cPoint2D(x1,y1)  )	);


	return true;
}



/*-----------------------------------------------------------------------------------------*/
bool cBSP_Loader::Load(char *filename ,  cGame_World * world)
{
	ifstream in;
	int n_line=0;
	char c;
	char line[100];
	string str_line;
	READING_STATE state = READING_UNDEF;
	std::string err_message;

	in.open( filename );
	if( ! in.is_open() )
	{
		cout << "error while opening file : " <<  filename << "\n";
		err_message += "fichier ";
		err_message +=filename;
		err_message += " introuvable ";
		//Display_Fatal_Error( err_message.c_str() );	
		cout << "error loading \n";
		

		return false;
	}

	while( ! in.eof() )
	{
		while( in.peek() == '\n' )
		{			
			in.ignore(1);
			n_line++;
		}
		if( in.eof() )
			return true;

		c= in.peek();
		if( c == '<' )   // le caractere '<' en debut de ligne indique une balise, on change d'etat
		{
			in.getline(line,100);
			n_line++;
			str_line = string(line);
			if( str_line == "<AREA>")
				state = READING_AREA;
			else if( str_line == "<WALLS>")
				state = READING_WALL;
			else 
			{
				state = READING_UNDEF;
				return false;
			}
		}
		
		switch( state)  // on appelle la fonction adequate selon l'etat de lecture
		{
		case READING_AREA:			
			if( !Read_Area(in,world))
			{
				Display_Error( n_line , filename);
				return false;
			}			
			break;
		case READING_WALL:			
			if( ! Read_Wall(in,world) )
			{
				Display_Error( n_line , filename);
				return false;
			}			
			break;		
		default:
			in.getline(line,100);			
			return false;			
		}
	}

	return true;
}


/*-----------------------------------------------------------------------------------------*/
void cBSP_Loader::Display_Error(int n_line , char * file)
{
	// modifier selon le type d'affichage d'erreur que l'on desire
	// sortie text/box message...
	cout << "ERROR while reading file : " << file << " at line " << n_line << "\n";
}