/***************************************************************************************************/
/***************************************************************************************************/
//! \file bsp_loader.h
//! \brief Decrit la class 'cBsp_Loader' utilise pour lire un ensemble de murs depuis un fichier
//!    
//!	Auteur : Adrien Auclair
/***************************************************************************************************/
/***************************************************************************************************/
#ifndef BSP_LOADER_H
#define BSP_LOADER_H


/*------------------------------------  Prototypes  ---------------------------------------------*/
class cBSP_Loader;


/*------------------------------------   Includes   ---------------------------------------------*/
#include <fstream>

#include "game_bsp.h"


/*-------------------------------------   Classes   ---------------------------------------------*/


/*************************************************************************************************/
//! class encapsulant l'algorithme pour charger un BSP
/*************************************************************************************************/
class cBSP_Loader
{
public:	
	/// charge le niveau du fichier donn dans le cGame_World pass en parametre
	/** retourne true si il n'y a pas eu d'erreur de lecture, false sinon */
	bool Load(char * filename , cGame_World * world);	

private:
	/// lit la taille du jeu a partir du stream 'in' et l'indique au 'game'
	/** retourne true si il n'y a pas eu d'erreur de lecture, false sinon */
	bool Read_Area(std::ifstream & in,  cGame_World * game);

	/// lit un mur a partir du stream 'in', et l'ajoute dans le 'game'
	/** retourne true si il n'y a pas eu d'erreur de lecture, false sinon */
	bool Read_Wall(std::ifstream & in,  cGame_World * game);

	/// affiche un message comme quoi une erreur s'est produite a la ligne 'n_line' du fichier 'file'
	void Display_Error(int n_line , char * file);
};


#endif