#include "draw_line.h"
#include "dxutil.h"


/*--------------------------------------------------------------------------------------------------*/
void Draw_Line(CSurface * p_surface , cPoint2D & start,cPoint2D & goal,unsigned char color )
{

	int sdx,sdy,dx,dy,dx2,dy2,error,x_inc,y_inc;  // var for the Bresenham algorithm
	int Px,Py;
	bool arrived = false;

	if( p_surface == NULL )  return;

	/*----------------------- initialize the Bresenham algorithm -----------------*/
	sdx     = goal.X - start.X;
	sdy     = goal.Y - start.Y;
	dx      = goal.X - start.X;
	dy      = goal.Y - start.Y;

	Px = start.X;
	Py = start.Y;
	
	if(dx<0)
	{
		x_inc = -1;
		dx=-dx;	
	}
	else
		x_inc=1;

	if(dy<0)	
	{
		y_inc = -1;
		dy=-dy;
	}
	else
		y_inc = 1;

	// Pre-calcul du double de dx et dy
	dx2=dx*2;
	dy2=dy*2;

	if(dx>dy)	// La ligne est X-dominante
	{
		// Erreur de dpart
		error=dy2-dx;
	}
	else
		error=dx2-dy;


    DDSURFACEDESC2 ddsd;
    HRESULT        hr;
    
    // Now lock the surface so we can read back the converted color
    ddsd.dwSize = sizeof(ddsd);
    hr = p_surface->GetDDrawSurface()->Lock( NULL, &ddsd, DDLOCK_WAIT, NULL );
    if( hr != DD_OK)
    {   
		//Display_Fatal_Error(hWnd , "Draw_Line(CSurface * p_surface , cPoint2D & start,cPoint2D & goal,unsigned char color )",hr);
		_DbgOut("display.cpp",137,hr,"void Draw_Line(CSurface * p_surface , cPoint2D & start,cPoint2D & goal)" );
		return;        
    }

	/*-------------  algo de trac -----------*/
	while( !arrived )
	{
		if(dx>dy)	// La ligne est X-dominante
		{	
			
			// On dcale et on corrige l'erreur en cas de dbordement
			if(error>=0)
			{
				error-=dx2;
				// Ligne suivante
				Py+=y_inc;
				
				
			}
			// Ajustement de l'erreur
			error+=dy2;
			// Pixel suivant
			Px+=x_inc;
			
			
		}	
		else	// La ligne est Y-dominante ou "qui-penche"
		{
			// On dcale et on corrige l'erreur en cas de dbordement
			if(error>=0)
			{
				error-=dy2;
				// Ligne suivante
				Px+=x_inc;
				
			}
			// Ajustement de l'erreur
			error+=dx2;
			// Pixel suivant
			Py+=y_inc;
			
		}

		((unsigned char *)ddsd.lpSurface)[Px + Py * ddsd.lPitch] = color;



		if( sdx >=0 && sdy>= 0 && Px >= goal.X && Py >= goal.Y )
			arrived=true;
		if( sdx >=0 && sdy<0   && Px >= goal.X && Py <= goal.Y )
			arrived=true;
		if( sdx <0 && sdy>= 0  && Px <= goal.X && Py >= goal.Y )
			arrived=true;
		if( sdx <0 && sdy< 0   && Px <= goal.X && Py <= goal.Y )
			arrived=true;
	}
	
	p_surface->GetDDrawSurface()->Unlock(NULL);

}


/*--------------------------------------------------------------------------------------------------*/
void Draw_Line(CDisplay * g_pDisplay, cPoint2D & start,cPoint2D & goal, unsigned char color )
{

	int sdx,sdy,dx,dy,dx2,dy2,error,x_inc,y_inc;  // var for the Bresenham algorithm
	int Px,Py;
	bool arrived = false;
	

	/*----------------------- initialize the Bresenham algorithm -----------------*/
	sdx     = goal.X - start.X;
	sdy     = goal.Y - start.Y;
	dx      = goal.X - start.X;
	dy      = goal.Y - start.Y;

	Px = start.X;
	Py = start.Y;
	
	if(dx<0)
	{
		x_inc = -1;
		dx=-dx;	
	}
	else
		x_inc=1;

	if(dy<0)	
	{
		y_inc = -1;
		dy=-dy;
	}
	else
		y_inc = 1;

	// Pre-calcul du double de dx et dy
	dx2=dx*2;
	dy2=dy*2;

	if(dx>dy)	// La ligne est X-dominante
	{
		// Erreur de dpart
		error=dy2-dx;
	}
	else
		error=dx2-dy;


    DDSURFACEDESC2 ddsd;
    HRESULT        hr;
    
    // Now lock the surface so we can read back the converted color
    ddsd.dwSize = sizeof(ddsd);
    hr = g_pDisplay->GetBackBuffer()->Lock( NULL, &ddsd, DDLOCK_WAIT, NULL );
    if( hr != DD_OK)
    {    
		//Display_Fatal_Error(hWnd , "Draw_Line(cPoint2D & start,cPoint2D & goal, unsigned char color )",hr);
		_DbgOut("display.cpp",249,hr,"void Draw_Line( cPoint2D & start,cPoint2D & goal)" );
		return;        
    }

	/*-------------  algo de trac -----------*/
	while( !arrived )
	{
		if(dx>dy)	// La ligne est X-dominante
		{	
			
			// On dcale et on corrige l'erreur en cas de dbordement
			if(error>=0)
			{
				error-=dx2;
				// Ligne suivante
				Py+=y_inc;
				
				
			}
			// Ajustement de l'erreur
			error+=dy2;
			// Pixel suivant
			Px+=x_inc;
			
			
		}	
		else	// La ligne est Y-dominante ou "qui-penche"
		{
			// On dcale et on corrige l'erreur en cas de dbordement
			if(error>=0)
			{
				error-=dy2;
				// Ligne suivante
				Px+=x_inc;
				
			}
			// Ajustement de l'erreur
			error+=dx2;
			// Pixel suivant
			Py+=y_inc;
			
		}

		((unsigned char *)ddsd.lpSurface)[Px + Py * ddsd.lPitch] = color;



		if( sdx >=0 && sdy>= 0 && Px >= goal.X && Py >= goal.Y )
			arrived=true;
		if( sdx >=0 && sdy<0   && Px >= goal.X && Py <= goal.Y )
			arrived=true;
		if( sdx <0 && sdy>= 0  && Px <= goal.X && Py >= goal.Y )
			arrived=true;
		if( sdx <0 && sdy< 0   && Px <= goal.X && Py <= goal.Y )
			arrived=true;
	}
	
	g_pDisplay->GetBackBuffer()->Unlock(NULL);

}
