#pragma warning(disable: 4786)

/*------------------------------------   Includes   ---------------------------------------------*/
#include "ennemies.h"
#include "game_manager.h"


/*------------------------------------    Define    ---------------------------------------------*/
#define SQUARE_VIEW_RANGE_RED_BALL  20000
#define SQUARE_VIEW_RANGE_BLUE_BALL  40000

#define POINTS_WHEN_BOMB_RED_BALL  500
#define POINTS_WHEN_BOMB_BLUE_BALL  200
#define POINTS_WHEN_BOMB_GREEN_BALL  50

#define POINTS_WHEN_TRAP_RED_BALL  1000
#define POINTS_WHEN_TRAP_BLUE_BALL  400
#define POINTS_WHEN_TRAP_GREEN_BALL 100



/****************************************************************************************************/
// class cEnnemy
/****************************************************************************************************/
cEnnemy::cEnnemy(int x, int y,cSprite_Display_Def* def, int speed, int rayon):cUnit(x,y,def,speed,rayon) 
{
	Initial_Position.X = x;
	Initial_Position.Y = y;
	Points_When_Killed = 0;
}

/*--------------------------------------------------------------------------------------------------*/
void cEnnemy::React_To_Bomb(cBomb * bomb)
{		
	Reset();
	Game_Manager.Remove_Bonus( bomb );	
	Game_Manager.Score += Points_When_Killed*(Game_Manager.Id_Level+1)*(Game_Manager.Id_Level+1);
}

/*--------------------------------------------------------------------------------------------------*/
void cEnnemy::Reset()
{
	Px = Initial_Position.X;
	Py = Initial_Position.Y;
	bOn_The_Road = false;
	Current_Path.clear();
}

/*--------------------------------------------------------------------------------------------------*/
void cEnnemy::React_To_Hologramme(cHologramme * hologramme)
{	
	Game_Manager.Remove_Bonus(hologramme);	
}

/*--------------------------------------------------------------------------------------------------*/
void cEnnemy::React_To_Teleporteur(cTeleporteur * teleporteur)
{	
	Game_Manager.Remove_Bonus(teleporteur);	
	assert( Game_Manager.bTeleporteur_In_Game );
	Game_Manager.bTeleporteur_In_Game = false;
}

/*--------------------------------------------------------------------------------------------------*/
void cEnnemy::React_To_Cadeau(cCadeau * cadeau)
{	
	Game_Manager.Remove_Bonus(cadeau);	
}

/*--------------------------------------------------------------------------------------------------*/
bool cEnnemy::Get_Destination(cPoint2D& destination)
{
	unsigned int dist;
	unsigned int new_dist;
	std::list<cInteractive_Item*>::iterator  bonus_itr;

	// initialement, destination est le joueur
	destination.Set( Game_Manager.Player->Px , Game_Manager.Player->Py );

	dist = Get_Square_Distance( destination , cPoint2D(Px,Py));
	// on cherche un hologramme plus pres, si c le cas, la destination est cet hologramme
	for( bonus_itr = Game_Manager.Bonus_List.begin() ; bonus_itr != Game_Manager.Bonus_List.end() ; bonus_itr++)
	{
		if( (*bonus_itr)->Type == HOLOGRAMME)			
		{
			new_dist = Get_Square_Distance( cPoint2D((*bonus_itr)->Px,(*bonus_itr)->Py) 
				                          , cPoint2D(Px,   Py) );
			if( new_dist < dist)
			{
				dist = new_dist;
				destination = cPoint2D((*bonus_itr)->Px,(*bonus_itr)->Py );
			}
		}				
	}

	return true;
}

/*--------------------------------------------------------------------------------------------------*/
void cEnnemy::Update_Path()
{
	cPoint2D destination;	
	std::vector<cPoint2D> chemin;	
	std::vector<cPoint2D>::iterator chemin_itr;

	// si on ne trouve pas de destination, 
	if( ! Get_Destination(destination) )
	{	
		// si on suivait un chemin, on le continue
		if( ! Current_Path.empty())
		{
			
			return;
		}
		else
		{
			bOn_The_Road = false; // sinon on s'arrete, TODO : trouver un chemin aleatoire plutots
			Current_Path.clear();
			return;
		}
	}
	

	// si le chemin n'etait pas vide, 
	// si la cible n'a pas boug, on n'update pas
	// sinon, on ne fait pas un algo de bresenham, mais plein de petit algo de bresenham, ce qui equivaut
	// a un trac bete de ligne
	if( ! Current_Path.empty())  
		if( Current_Path.back() == destination )
			return;		

	chemin.clear();
	Game_Manager.Game_World->Find_Path(  cPoint2D(Px,Py) ,
						    destination , chemin);
	Current_Path.clear();
	bOn_The_Road = false;
		
	for( chemin_itr=chemin.begin(); chemin_itr!=chemin.end() ; chemin_itr++)
		Current_Path.push_back( (*chemin_itr) );
}



/****************************************************************************************************/
// class cRed_Ball
/****************************************************************************************************/
cRed_Ball::cRed_Ball(int x, int y):cEnnemy(x,y,Game_Manager.pDisplay_Red_Ball,SPEED_RED_BALL,RAYON_RED_BALL) 
{
	Points_When_Killed = POINTS_WHEN_BOMB_RED_BALL;
}

/*--------------------------------------------------------------------------------------------------*/
void cRed_Ball::React_To_Red_Trap(cRed_Trap * trap)
{
	
	Game_Manager.Kill_Ennemy( this );
	Game_Manager.Score += POINTS_WHEN_TRAP_RED_BALL*(Game_Manager.Id_Level+1)*(Game_Manager.Id_Level+1);
}

/*--------------------------------------------------------------------------------------------------*/
bool cRed_Ball::Get_Destination(cPoint2D& destination)
{
	unsigned int dist;
	unsigned int new_dist;
	std::list<cInteractive_Item*>::iterator  bonus_itr;

	// initialement, destination est le joueur
	destination.Set( Game_Manager.Player->Px , Game_Manager.Player->Py );

	dist = Get_Square_Distance( destination , cPoint2D(Px,Py));
	// on cherche un hologramme plus pres, si c le cas, la destination est cet hologramme
	for( bonus_itr = Game_Manager.Bonus_List.begin() ; bonus_itr != Game_Manager.Bonus_List.end() ; bonus_itr++)
	{
		if( (*bonus_itr)->Type == HOLOGRAMME)			
		{
			new_dist = Get_Square_Distance( cPoint2D((*bonus_itr)->Px,(*bonus_itr)->Py) 
				                          , cPoint2D(Px,   Py) );
			if( new_dist < dist)
			{
				dist = new_dist;
				destination = cPoint2D((*bonus_itr)->Px,(*bonus_itr)->Py );
			}
		}				
	}

	// si le point est assez pres, on
	if( dist <= SQUARE_VIEW_RANGE_RED_BALL)
		return true;
	else
		return false;

}



/****************************************************************************************************/
// class cBlue_Ball
/****************************************************************************************************/
cBlue_Ball::cBlue_Ball(int x, int y):cEnnemy(x,y,Game_Manager.pDisplay_Blue_Ball,SPEED_BLUE_BALL,RAYON_BLUE_BALL) 
{
	Points_When_Killed = POINTS_WHEN_BOMB_BLUE_BALL;
}

/*--------------------------------------------------------------------------------------------------*/
void cBlue_Ball::React_To_Blue_Trap(cBlue_Trap * trap)
{
	
	Game_Manager.Kill_Ennemy( this );
	Game_Manager.Score += POINTS_WHEN_TRAP_BLUE_BALL*(Game_Manager.Id_Level+1)*(Game_Manager.Id_Level+1);
}

/*--------------------------------------------------------------------------------------------------*/
bool cBlue_Ball::Get_Destination(cPoint2D& destination)
{
	unsigned int dist;
	unsigned int new_dist;
	std::list<cInteractive_Item*>::iterator  bonus_itr;

	// initialement, destination est le joueur
	destination.Set( Game_Manager.Player->Px , Game_Manager.Player->Py );

	dist = Get_Square_Distance( destination , cPoint2D(Px,Py));
	// on cherche un hologramme plus pres, si c le cas, la destination est cet hologramme
	for( bonus_itr = Game_Manager.Bonus_List.begin() ; bonus_itr != Game_Manager.Bonus_List.end() ; bonus_itr++)
	{
		if( (*bonus_itr)->Type == HOLOGRAMME)			
		{
			new_dist = Get_Square_Distance( cPoint2D((*bonus_itr)->Px,(*bonus_itr)->Py) 
				                          , cPoint2D(Px,   Py) );
			if( new_dist < dist)
			{
				dist = new_dist;
				destination = cPoint2D((*bonus_itr)->Px,(*bonus_itr)->Py );
			}
		}				
	}

	// si le point est assez pres, on
	if( dist <= SQUARE_VIEW_RANGE_BLUE_BALL)
		return true;
	else
		return false;

}

/****************************************************************************************************/
// class cGreen_Ball
/****************************************************************************************************/
cGreen_Ball::cGreen_Ball(int x, int y):cEnnemy(x,y,Game_Manager.pDisplay_Green_Ball,SPEED_GREEN_BALL,RAYON_GREEN_BALL) 
{
	Points_When_Killed = POINTS_WHEN_BOMB_GREEN_BALL;
}

/*--------------------------------------------------------------------------------------------------*/
void cGreen_Ball::React_To_Green_Trap(cGreen_Trap * trap)
{	
	Game_Manager.Kill_Ennemy( this );
	Game_Manager.Score += POINTS_WHEN_TRAP_GREEN_BALL*(Game_Manager.Id_Level+1)*(Game_Manager.Id_Level+1);
}

/*--------------------------------------------------------------------------------------------------*/
bool cGreen_Ball::Get_Destination(cPoint2D& destination)
{	
	destination.Set( Game_Manager.Player->Px , Game_Manager.Player->Py );
	return true;
}



