/***************************************************************************************************/
/***************************************************************************************************/
//! \file ennemies.h
//! \brief Dfini les classes des ennemies
//!    
//!	Auteur : Adrien Auclair
//!
//! 
//! les ennemies heritent de cUnit, ils sont regroups ds une classe gnriques
/***************************************************************************************************/
/***************************************************************************************************/
#ifndef ENNEMIES_H
#define ENNEMIES_H

/*------------------------------------  Prototypes  ---------------------------------------------*/
class cEnnemy;
class cRed_Ball;
class cBlue_Ball;
class cGreen_Ball;

/*------------------------------------    Define    ---------------------------------------------*/
#define SPEED_RED_BALL 5
#define SPEED_BLUE_BALL 3
#define SPEED_GREEN_BALL 1

#define RAYON_RED_BALL 10
#define RAYON_BLUE_BALL 10
#define RAYON_GREEN_BALL 10

typedef enum { RED_BALL , BLUE_BALL , GREEN_BALL } Ennemy_Type;

/*------------------------------------   Includes   ---------------------------------------------*/
#include "Unit.h"


/*-------------------------------------   Classes   ---------------------------------------------*/


/*************************************************************************************************/
//! classe gnrique des ennemis
/*************************************************************************************************/
class cEnnemy:public cUnit
{
public:
	/// constructeur
	/**
	* x,y : position de l'ennemi \n
	* def : pointeur sur la classe d'affichage \n	
	*/
	cEnnemy(int x, int y,cSprite_Display_Def* def, int speed, int rayon);	

	/// met ds 'destination' le point vers lequel l'unit doit se rendre
	/**
	* retourne true si un point a ete trouv, false sinon \n
	* le comportement par defaut est de chercher le point le plus pres entre le joueur et les hologrammes \n
	*/	
	virtual bool Get_Destination(cPoint2D& destination);
	
	///  met a jour le chemin a suivre
	/**
	* appelle la fonction virtuelle Get_Destination pour trouver o aller \n
	*  et met a jour le chemin pour aller  ce point \n
	*/
	void Update_Path();
	
	/// remet cet unit immobile  sa position initiale
	void Reset();
	
	/// reaction  une bomb
	void React_To_Bomb(cBomb * bomb);	
	/// reaction  un teleporteur
	void React_To_Teleporteur(cTeleporteur * teleporteur);
	/// reaction  un hologramme
	void React_To_Hologramme(cHologramme * hologramme);
	/// reaction  une cadeau
	void React_To_Cadeau(cCadeau * cadeau);
	
	/// position  laquelle cette unit est au debut du niveau
	cPoint2D Initial_Position;

	int Points_When_Killed;    ///< nb de points gagns qd on tue cette unit
};


/*************************************************************************************************/
//! classe des boules rouges
/*! les boules rouges ont le comportement suivant : 
//!  -> rapides 
//!  -> ont un view-range limit
//!  -> se laisse pieger par les hologrammes
//!
/*************************************************************************************************/
class cRed_Ball : public cEnnemy
{
public:
	/// constructeur
	/**
	* x,y : position
	*/
	cRed_Ball(int x, int y);
	
	/// cherche un point ou aller
	bool Get_Destination(cPoint2D& destination);
private:
	/// reagit  un piege rouge
	void React_To_Red_Trap(cRed_Trap * red_trap);	
};

/*************************************************************************************************/
//! classe des boules bleues
/*! les boules rouges ont le comportement suivant : 
//!  -> vitesse moyenne
//!  -> ont un view-range assez grand
//!  -> se laisse pieger par les hologrammes
/*************************************************************************************************/
class cBlue_Ball : public cEnnemy
{
public:
	/// constructeur
	/**
	* x,y : position
	*/
	cBlue_Ball(int x, int y);

	/// cherche un point ou aller
	bool Get_Destination(cPoint2D& destination);
private:	
	/// reagit  un piege bleu
	void React_To_Blue_Trap(cBlue_Trap * blue_trap);
};

/*************************************************************************************************/
//! classe des boules vertes
/*! les boules rouges ont le comportement suivant : 
//!  -> lentes
//!  -> ont un view-range illimit
//!  -> savent faire la difference entre un hologramme et le joueur !
/*************************************************************************************************/
class cGreen_Ball : public cEnnemy
{
public:
	/// constructeur
	/**
	* x,y : position
	*/
	cGreen_Ball(int x, int y);

	/// cherche un point ou aller
	bool Get_Destination(cPoint2D& destination);
private:	
	/// reagit  un piege vert
	void React_To_Green_Trap(cGreen_Trap * green_trap);
};


#endif