#pragma warning(disable: 4786)

/*------------------------------------   Includes   ---------------------------------------------*/
#include "bsp.h"
#include "game_bsp.h"
#include "bresenham.h"



using namespace std;


class cPositive_Rectangle cGame_World::Ecran = cPositive_Rectangle( cPoint2D(0,0) , cPoint2D(100,100));

/****************************************************************************************************/
// class cSquare_Area
/****************************************************************************************************/
cSquare_Area::~cSquare_Area()
{		
	std::vector<cHV_Seg*>::iterator walls_itr;
	for(walls_itr = List_Walls.begin() ; walls_itr != List_Walls.end() ; walls_itr++)
	{
		delete( (*walls_itr) );
	}	
	List_Walls.clear();
	List_Portals.clear();

}

/*--------------------------------------------------------------------------------------------------*/
void cSquare_Area::Print()
{
	assert( Front == NULL && Back == NULL);

	std::vector<cHV_Seg*>::iterator walls_itr;
	std::vector<cPortal*>::iterator portals_itr;	

	cout << "[Leaf:" << Id << "]" ;
	cout << "]-> Area " << Bounds << "\n";
	cout << "		n_walls ="<<List_Walls.size() << "\n";
	cout << "		n_portals ="<<List_Portals.size() << "\n";
	cout << "       walls : \n";
	for(walls_itr = List_Walls.begin() ; walls_itr != List_Walls.end() ; walls_itr++)
		cout << "			-> " <<  *(*walls_itr) << "\n";
	cout << "       portals : \n";	
	for(portals_itr = List_Portals.begin() ; portals_itr != List_Portals.end() ; portals_itr++)
		cout << "			-> " <<  *(*portals_itr) << "\n";
}

/*--------------------------------------------------------------------------------------------------*/
void cSquare_Area::split( cHV_Wall & wall, cSquare_Area ** A1, cSquare_Area** A2)
{
	cPoint2D pA;
	cPoint2D pB;

	if( wall.Is_Horizontal() )
	{		
		pA    = this->Bounds.p0;
		pB.Set(this->Bounds.p1.X , wall.p0.Get_Y() );
		(*A1) = new cSquare_Area(  cPositive_Rectangle( pA, pB)   ); 

		pA.Set(this->Bounds.p0.X , wall.p0.Get_Y() );
		pB    = this->Bounds.p1;
		(*A2) = new cSquare_Area(cPositive_Rectangle( pA, pB));
	}
	else
	{
		pA    = this->Bounds.p0;
		pB.Set( wall.p0.Get_X() ,  this->Bounds.p1.Y );
		(*A1) = new cSquare_Area(cPositive_Rectangle( pA , pB)); 
		
		pA.Set( wall.p0.Get_X() ,  this->Bounds.p0.Y);
		pB    = this->Bounds.p1;
		(*A2) = new cSquare_Area(cPositive_Rectangle( pA, pB));
	}
}


/*--------------------------------------------------------------------------------------------------*/
void cSquare_Area::Add_Wall(cHV_Seg & w)
{
	// on verifie que le wall appartient bien a la zone
	assert( Bounds.In(w.Get_p0() ));
	assert( Bounds.In(w.Get_p1() ));

	cHV_Seg * wall = new cHV_Seg(w.Get_p0() , w.Get_p1() );
	List_Walls.push_back(wall);

}


/*--------------------------------------------------------------------------------------------------*/
void cSquare_Area::Manage_Paralell_Portals()
{

	cPoint2D pA;
	cPoint2D pB;
	cPoint2D p1;
	cPoint2D p2;
	cPoint2D pnew;
	cHV_Seg dist;
	cSeg door;
	cPortal * new_portal = NULL;


	std::vector<cPortal*>::iterator portals_itr1;
	std::vector<cPortal*>::iterator portals_itr2;	

	for( portals_itr1 = List_Portals.begin(); 	portals_itr1 != List_Portals.end(); portals_itr1++)	
		for( portals_itr2 = List_Portals.begin(); 	portals_itr2 != List_Portals.end(); portals_itr2++)
		{
			if( (*portals_itr1) < (*portals_itr2) )				
				if(  (*portals_itr1)->Segment.On_Same_Line((*portals_itr2)->Segment) ) // portails parallell
				{
					(*portals_itr1)->Segment.Get_Middle(pA);
					(*portals_itr2)->Segment.Get_Middle(pB);

					dist.Set( pA,pB);
					dist.Get_Middle(p1);
					this->Get_Centre(p2);
					if( dist.Is_Horizontal() )
					{
						if( p2.Get_Y() < p1.Get_Y() )
							pnew.Set(p1.X , p1.Y-40);
						else
							pnew.Set(p1.X , p1.Y+40);
					}
					else
					{
						if( p2.Get_X() < p1.Get_X() )
							pnew.Set(p1.X-40 , p1.Y);
						else
							pnew.Set(p1.X +40, p1.Y);
					}

					new_portal = new cParalell_Portal(pnew , -1 , this);				
					//N_Portals++;
					(*portals_itr1)->Add_Voisin( new_portal);
					(*portals_itr2)->Add_Voisin(  new_portal );		
				}
		}
}


/*--------------------------------------------------------------------------------------------------*/
void cSquare_Area::Add_Portal(cPortal * portal)
{
	assert( portal != NULL);	
	assert( Bounds.In(portal->Segment.Get_p0()) ); // on verifie que ce portal appartient bien a la zone
	assert( Bounds.In(portal->Segment.Get_p1()) );

	bool paralell=false;
	cPoint2D p1;
	cPoint2D p2;
	cPortal * new_portal = NULL;

	// on parcours la liste des portals de cette piece
	// et on signale a chacun ce nouveau voisin 'portal'
	// on signale a 'portal' qu'il est voisin de chacun
	std::vector<cPortal*>::iterator portals_itr;	
	for( portals_itr = List_Portals.begin(); 	portals_itr != List_Portals.end(); portals_itr++)
	{		
		// ce sont deux portails  non parallelles !!
		if( ! portal->Segment.On_Same_Line((*portals_itr)->Segment) )
		{
			(*portals_itr)->Add_Voisin( portal);
			portal->Add_Voisin(  (*portals_itr) );
		}		
	}
	
	// on ajoute a la liste de portals de cette piece le nouveau portal 'portal'
	List_Portals.push_back(portal);
}


/*--------------------------------------------------------------------------------------------------*/
void cSquare_Area::Add_Paralells_Portals(cPortal * portal1 , cPortal * portal2)
{
	assert( portal1 != NULL);
	assert( portal2 != NULL);
	assert( Bounds.In(portal1->Segment.Get_p0()) );
	assert( Bounds.In(portal1->Segment.Get_p1()) );
	assert( Bounds.In(portal2->Segment.Get_p0()) );
	assert( Bounds.In(portal2->Segment.Get_p1()) );

	// on signale a chaque portal de la piece qu'il a pour nouveau voisin 'portal1' et 'portal2'
	std::vector<cPortal*>::iterator portals_itr;	
	for( portals_itr = List_Portals.begin(); 	portals_itr != List_Portals.end(); portals_itr++)
	{		
		(*portals_itr)->Add_Voisin( portal1);
		(*portals_itr)->Add_Voisin( portal2);		
	}
	
	// on ajoute 'portal1' et 'portal2' a la liste des portals de la piece
	List_Portals.push_back(portal1);
	List_Portals.push_back(portal2);
}




/****************************************************************************************************/
// class cGame_World
/****************************************************************************************************/
/*--------------------------------------------------------------------------------------------------*/
cGame_World::cGame_World() 
{ 
	N_Leaves=0; 
	bDecorated_With_Areas=false;
	N_Portals=0;		
	
}	

/*--------------------------------------------------------------------------------------------------*/
cGame_World::~cGame_World() 
{ 
	cPortal * pToDelete=NULL;
	
	N_Leaves=0; 
	bDecorated_With_Areas=false;
	N_Portals=0;		
	//delete(Level_Loader);

	// on n'a pas a detruire les feuilles, c le destructeur du bsp_tree qui s'en charge
	// il faut detruire les portals
	std::vector<cPortal*>::iterator Portals_Itr;	
	for(Portals_Itr = List_Portals.begin() ; Portals_Itr != List_Portals.end() ; Portals_Itr++)	
	{
		pToDelete = (*Portals_Itr);
		delete(pToDelete);
	}

	List_Portals.clear();
	List_Areas.clear();

		
}	


/*--------------------------------------------------------------------------------------------------*/
void cGame_World::Decorate_With_Areas()
{
	cHV_Node * root = My_BSP.Get_Root();		
	cSquare_Area * big_one = new cSquare_Area( cGame_World::Ecran );	
	bDecorated_With_Areas  = true;
	
	if( root == NULL)
	{
		root = big_one;	
		N_Leaves++;
		root->Set_Id(-N_Leaves); // les feuilles ont les id negatives
	}
	else	
		Decorate_With_Areas(root , big_one);	
}



/*--------------------------------------------------------------------------------------------------*/
void cGame_World::Decorate_With_Areas(cHV_Node * node ,  cSquare_Area * area )
{
	cSquare_Area * A_Front = NULL;
	cSquare_Area * A_Back  = NULL;

	area->split( node->Divider, &A_Front, &A_Back);
	if( node->Front == NULL)
	{
		node->Front = A_Front;
		N_Leaves++;
		node->Front->Set_Id(-N_Leaves);
		List_Areas.push_back(A_Front);
	}
	else
	{
		Decorate_With_Areas( node->Front , A_Front);
		if(area != NULL) 
		{
			delete(area);
			area = NULL;
		}
	}

	if( node->Back == NULL)
	{
		node->Back = A_Back;
		N_Leaves++;
		node->Back->Set_Id(-N_Leaves);
		List_Areas.push_back(A_Back);
	}
	else
	{
		Decorate_With_Areas( node->Back , A_Back);
		if( area != NULL) 	
		{
			delete(area);
			area = NULL;
		}
	}
}

	

/*--------------------------------------------------------------------------------------------------*/
void cGame_World::Print_Leaves()
{
	std::vector<cSquare_Area*>::iterator Areas_Itr;

	cout << "####### Liste des zones : ########## \n";
	for(Areas_Itr = List_Areas.begin() ; Areas_Itr != List_Areas.end() ; Areas_Itr++)
		(*Areas_Itr)->Print();
	cout << "#################################### \n";
}


/*--------------------------------------------------------------------------------------------------*/
void cGame_World::Decorate_With_Walls_And_Portals()
{
	cPoint2D pA,pB;	
	cHV_Seg cote;

	if( ! bDecorated_With_Areas )
	{
		cout << " ERROR : void cGame_World::Decorate_With_Walls_And_Portals() \n";
		cout << " l'arbre doit d'abord etre dcor avec les cSquarePortals !!! \n";		
		return;
	}

	std::vector<cSquare_Area*>::iterator Areas_Itr;	
	for(Areas_Itr = List_Areas.begin() ; Areas_Itr != List_Areas.end() ; Areas_Itr++)
	{
		/*----- cote nord -------------*/				
		(*Areas_Itr)->Bounds.Get_Nord_Seg(cote);		
		Add_Wall_And_Portals( (*Areas_Itr) , cote);		

		/*------cote est ------------*/		
		(*Areas_Itr)->Bounds.Get_Est_Seg(cote);		
		Add_Wall_And_Portals( (*Areas_Itr) , cote);				

		/*------- cote sud-----------*/
		(*Areas_Itr)->Bounds.Get_Sud_Seg(cote);		
		Add_Wall_And_Portals( (*Areas_Itr) , cote);

		/*-------- cote ouest ----------*/
		(*Areas_Itr)->Bounds.Get_Ouest_Seg(cote);		
		Add_Wall_And_Portals( (*Areas_Itr) , cote);
	}	
}



/*--------------------------------------------------------------------------------------------------*/
void cGame_World::Manage_Paralell_Portals(void)
{
	std::vector<cSquare_Area*>::iterator Areas_Itr;	

	for(Areas_Itr = List_Areas.begin() ; Areas_Itr != List_Areas.end() ; Areas_Itr++)
	{
		(*Areas_Itr)->Manage_Paralell_Portals();
	}
}




/*--------------------------------------------------------------------------------------------------*/
void cGame_World::Creates_And_Add_Portals(cSquare_Area * area , cHV_Seg& seg ,cHV_Seg& wall)
{
	cPortal * portal1 = NULL;
	cPortal * portal2 = NULL;
	//bool portal_1     = false;
	//bool portal_2     = false;	
	cPortal * portal  = NULL;
	cHV_Seg seg_portal;
	cPoint2D p1;
	cPoint2D p2;
	

	if( wall.Get_p0() != seg.Get_p0() ) // portal 1			
	{	
		seg_portal.Set( seg.Get_p0() ,wall.Get_p0() ); // portal entre le debut du cote et le debut du mur				
		portal1 = Declare_Portal( area , seg_portal );  // creation ou mise a jour de ce portal				
		//portal_1 = true;
	}			

	if( wall.Get_p1() != seg.Get_p1() ) // portal2
	{				
		seg_portal.Set( wall.Get_p1() , seg.Get_p1() );  // portal entre la fin du mur et la fin du seg				
		portal2 = Declare_Portal( area , seg_portal ); // creation ou mise a jour de ce portal				
		//portal_2 = true;
	}
	
}

/*--------------------------------------------------------------------------------*/
// on va inserer les murs et portals du cot 'seg' de 'area' au jeu
// on a cinq cas possible :
//  1) seg entier est un portal
//  2) seg entier est un mur de 'area'
//  3) seg est : "MUR---PORTAL"
//  4) seg est : "MUR---PORTAL---MUR"
//  5) seg est : "PORTAL---MUR"
/*--------------------------------------------------------------------------------*/
void cGame_World::Add_Wall_And_Portals(cSquare_Area * area , cHV_Seg& seg)
{
	assert( area != NULL);	
	
	cPoint2D   p_test;	
	cHV_Seg    wall;
	cHV_Seg    wall_in_the_room;	
	cHV_Seg    seg_portal;
	cPortal *  portal  = NULL;
	cHV_Node * node    = NULL;	

	// on teste un point de seg pour savoir si ce point est sur un divider, on prends le point milieu
	seg.Get_Middle(p_test);
	node = My_BSP.Where_Is( p_test );	
	
	//si ce n'est pas une feuille, c qu'il s'agit bien d'un mur
	if( node->Front == NULL && node->Back==NULL) return;
	
	node->Divider.Get_Wall_Seg(wall);  // wall contiendra le segment du mur de ce divider

	// si wall ne vient pas jusqu'a la piece, alors seg est un portal entier
	// au passage, 'inter' contient la portion de wall qui borde 'area'
	if( !seg.Get_Intersection(wall,wall_in_the_room) ) 
	{
		seg_portal.Set( seg.Get_p0() , seg.Get_p1() ); // portal entre le debut du cote et le debut du mur
		portal = Declare_Portal( area , seg_portal );  // creation ou mise a jour de ce portal
		//area->Add_Portal(portal);
		return;
	}
	else 
	{
		// ce mur peut-etre de taille 1 s'il est ds un coin !!
		// si c'est le cas, on evite de le rajouter !
		// sinon on ajoute la portion de mur effective a cette piece
		if( wall_in_the_room.Get_p0() != wall_in_the_room.Get_p1() )
			area->Add_Wall(wall_in_the_room);

		Creates_And_Add_Portals(area,seg,wall_in_the_room);		
	}
}



/*--------------------------------------------------------------------------------------------------*/
cPortal * cGame_World::Declare_Portal( cSquare_Area * area , cHV_Seg& seg)
{
	assert( area != NULL);

	cPortal * portal = NULL;
	cPortal * current_portal = NULL;
	cHV_Seg inter;
	cHV_Seg comp0;
	cHV_Seg comp1;	
	int type_inter;
	bool do_inter;
	

	// on parcours tous les portals pour chercher si le seg qu'on veut ajouter n'est pas deja un portal
	// ceci car il est possible que la cSquare_Area de l'autre cot de ce portal est dja ajout
	// ce portal  la liste 'List_Portals'
	std::vector<cPortal*>::iterator Portals_Itr;	
	for(Portals_Itr = List_Portals.begin() ; Portals_Itr != List_Portals.end() ; Portals_Itr++)
	{
		if( seg == (*Portals_Itr)->Segment )  // le portail voisin est identique, on l'identifie au nouveau
		{
			// on verifie que ce portal a un cot de non connu
			assert( (*Portals_Itr)->Area1 != NULL ||  (*Portals_Itr)->Area2 != NULL );

			// ce portal existe deja
			// on lui signale qu'il est voisin de la piece 'area'
			(*Portals_Itr)->Add_Area(area);			
			area->Add_Portal((*Portals_Itr) );
			return (*Portals_Itr);
		}		
		else
		{
			do_inter = (*Portals_Itr)->Segment.Get_Intersection(seg,inter);	
			if( do_inter && (inter.Get_p0() != inter.Get_p1() ))   // les deux portails s'intersecte, non reduit en 1 point
			{
				current_portal = (*Portals_Itr);

				if( inter == current_portal->Segment)   // si l'inter est pas egale a l'ancien portail
				{
					//(*Portals_Itr)->Add_Area(area);     //on ne fait que lui ajouter l'area 
				}
				else
				{
					Split_Portal( current_portal , inter );    // sinon, il faut le diviser					
					assert(current_portal->Segment == inter );					
				}

				// on rajoute les portail du au segment
				type_inter = seg.Get_Complementaires(inter , comp0 , comp1);				

				switch(type_inter)
				{
				case 0:
					assert(0);
				case 1 :  
					current_portal->Add_Area(area);
					area->Add_Portal(current_portal );
					break;
				case 2:					
					Declare_Portal( area , inter);
					Declare_Portal( area , comp0);
					Declare_Portal( area , comp1);
					break;
				case 3:					
					Declare_Portal( area , inter);
					Declare_Portal( area , comp0);					
					break;
				case 4:					
					Declare_Portal( area , inter);					
					Declare_Portal( area , comp1);
					break;
				default:
					assert(0);
				}
				return NULL;
			}	
		}
	}
	
	// ce portal n'existe pas, il faut le creer, l'ajouter  la liste 'List_Portals'
	portal = new cPortal(seg , area , N_Portals);
	List_Portals.push_back(portal);
	area->Add_Portal( portal );
	N_Portals++;
	return portal;
}


/*--------------------------------------------------------------------------------------------------*/
void cGame_World::Split_Portal(cPortal * p , cHV_Seg& seg)
{
	cHV_Seg comp1;
	cHV_Seg comp2;
	
	assert( p->Segment.Contient(seg ) );
	
	cPortal * portal = NULL;
	int inter_type;

	assert( p->Segment.Contient(seg) );

	inter_type = p->Segment.Get_Complementaires(seg , comp1 , comp2);
	

	switch( inter_type)
	{
	case 0:
		assert(0);
		break;
	case 1:
		assert(0);
		break;
	case 2:
		p->Segment.Set(seg.Get_p0() , seg.Get_p1() );	
		p->Segment.Get_Middle(p->Position );
		//p->Add_Area(area);

		portal = new cPortal(comp1 , p->Area1 ,N_Portals );
		//portal->Add_Area(area);
		List_Portals.push_back(portal);
		N_Portals++;
		//area->Add_Portal( portal );
		p->Area1->Add_Portal( portal );

		portal = new cPortal(comp2 , p->Area1 ,N_Portals );
		//portal->Add_Area(area);
		List_Portals.push_back(portal);
		N_Portals++;
		//area->Add_Portal( portal );
		p->Area1->Add_Portal( portal );
		break;
	case 3:				
		p->Segment.Set(seg.Get_p0() , seg.Get_p1() );
		p->Segment.Get_Middle(p->Position );
		portal = new cPortal(comp1 , p->Area1 ,N_Portals );
		//portal->Add_Area(area);
		List_Portals.push_back(portal);
		N_Portals++;
		//area->Add_Portal( portal );
		p->Area1->Add_Portal( portal );
		break;
	case 4:		
		p->Segment.Set(seg.Get_p0() , seg.Get_p1() );	
		p->Segment.Get_Middle(p->Position );
		portal = new cPortal(comp2 , p->Area1 ,N_Portals );
		//portal->Add_Area(area);
		List_Portals.push_back(portal);  // modifie la liste des portals !!!
		N_Portals++;
		//area->Add_Portal( portal );
		p->Area1->Add_Portal( portal );
		break;
	default:
		assert(0);
		break;	
	} 


}



/*--------------------------------------------------------------------------------------------------*/
bool cGame_World::Find_Path( cPoint2D& pA , cPoint2D& pB, std::vector<cPoint2D> & Path)
{
	cSquare_Area * area_start = NULL;
	cSquare_Area * area_goal  = NULL;
	cA_Star_Node * start_node = NULL;
	cA_Star_Node * goal_node  = NULL;
	cPortal      * portal     = NULL;
	bool chemin_trouve        = false;

	cList_A_Star_Nodes voisinage;
	cList_A_Star_Nodes::iterator portals_itr;
	cList_A_Star_Nodes nodes_path;
	cList_A_Star_Nodes::iterator nodes_itr;
	cPoint2D step;

	Path.clear();
	
	// on obtient forcement une feuille, 
	// si le point est sur un divider, on obtient une feuille adjacente
	area_start = (cSquare_Area*) My_BSP.In_Which_Leaf_Is( pA );
	area_goal =  (cSquare_Area*) My_BSP.In_Which_Leaf_Is( pB );

	if(  area_start == area_goal)
	{
		step.X= pB.Get_X();
		step.Y= pB.Get_Y();
		Path.push_back( step );
		return true;
	}

	start_node = new cA_Star_Point( area_start , pA );
	goal_node  = new cA_Star_Point( area_goal , pB );

	// ajout du noeud correspondant au point d'arrive
	goal_node->Get_List_Neighbors(voisinage);
	for( portals_itr = voisinage.begin(); portals_itr != voisinage.end() ; portals_itr++)
	{
		portal = (cPortal*) (*portals_itr);		
		portal->Add_Voisin( goal_node);		
	}

	// recherche du chemin
	chemin_trouve = Path_Finder.Find_Path( start_node , goal_node , nodes_path);
	if( chemin_trouve) // on transforme la liste de noeuds en chemin
	{
		for( nodes_itr = nodes_path.begin(); nodes_itr != nodes_path.end() ; nodes_itr++)
		{
			step.X= (*nodes_itr)->Position.Get_X();
			step.Y= (*nodes_itr)->Position.Get_Y();
			Path.push_back( step );
		}
	}
	
	// suppression du noeud correspondant au point d'arrive
	goal_node->Get_List_Neighbors(voisinage);
	for( portals_itr = voisinage.begin(); portals_itr != voisinage.end() ; portals_itr++)
	{
		portal = (cPortal*) (*portals_itr);		
		portal->Remove_Voisin( goal_node);		
	}
	
	delete(start_node);
	delete(goal_node);

	return chemin_trouve;
}



/*--------------------------------------------------------------------------------------------------*/
bool cGame_World::Do_Ball_Intersect_Walls( int x, int y, int r)
{
	cSquare_Area *leaf = NULL;
	cPoint2D p(x,y);
	
	leaf = (cSquare_Area*) My_BSP.In_Which_Leaf_Is( p );

	std::vector<cHV_Seg*>::iterator walls_itr;	
	for(walls_itr = leaf->List_Walls.begin() ; walls_itr != leaf->List_Walls.end() ; walls_itr++)
	{		
		if( (*walls_itr)->Get_Square_Distance_From_Point(p ) < r*r)		
			return true;
	}
		

	return false;
}


/*--------------------------------------------------------------------------------------------------*/
bool cGame_World::Do_Segment_Intersect_Walls(cSeg& seg, cPoint2D& p_result)
{
	return Do_Segment_Intersect_Walls(seg,p_result,seg.Get_Square_Length() );
}



/*--------------------------------------------------------------------------------------------------*/
bool cGame_World::Do_Segment_Intersect_Walls(cSeg& seg, cPoint2D& p_result, int square_test_range)
{
	cPoint2D p;
	cPoint2D p_bis;
	cPoint2D p0;
	cPoint2D p1;
	cHV_Seg hv_seg;
	cIncremental_Bresenham bresenham;
	bool arrived              = false;
	cHV_Node * node           = NULL;
	int current_square_length = 0;
	int n_test                = 0;

	p0.X =  seg.Get_p0().Get_X();
	p0.Y =  seg.Get_p0().Get_Y();

	p1.X =  seg.Get_p1().Get_X();
	p1.Y =  seg.Get_p1().Get_Y();

	// si les deux points sont ds la meme area, ils se voient forcement
	// car les area sont convexees
	if( (cSquare_Area*) My_BSP.In_Which_Leaf_Is( seg.Get_p0() ) == (cSquare_Area*) My_BSP.In_Which_Leaf_Is( seg.Get_p1() ) )
		return false;

	// si le segment est de longeur nulle, il n'intesecte rien
	if( seg.Get_p0() == seg.Get_p1() )
		return false;	
	
	bresenham.Reset(p0 , p1 );
	p = seg.Get_p0();
	while(!arrived && current_square_length<square_test_range)	
	{
		p_bis.X = p.Get_X();
		p_bis.Y = p.Get_Y();
		arrived = bresenham.Step(p_bis);
		p = p_bis;
		current_square_length = (p.Get_X() - p0.X)*(p.Get_X() - p0.X) + (p.Get_Y() - p0.Y)*(p.Get_Y() - p0.Y);

		n_test++;
		node = My_BSP.Where_Is( p );
		if( node->Front != NULL && node->Back != NULL)  // on est sur un noeud qui divise le world, et pas sur une feuille
		{			
			hv_seg.Set( node->Divider.p0 , node->Divider.p1); 
			if( hv_seg.Contient_Point(p) )
			{				
				p_result = p;
				return true;
			}
		}
	}
	
	return false;   // on a pas trouv d'intersections
}



/*--------------------------------------------------------------------------------------------------*/
void cGame_World::Decorate()
{
	Decorate_With_Areas();
	Decorate_With_Walls_And_Portals();
	Manage_Paralell_Portals();
}