/*------------------------------------   Includes   ---------------------------------------------*/
#pragma warning(disable: 4786)

#include <iostream>

#include "bsp.h"
#include "game_graph.h"

using namespace std;


/****************************************************************************************************/
// class cPortal
/****************************************************************************************************/
/*--------------------------------------------------------------------------------------------------*/
ostream & operator<<(ostream &os, const cPortal &ex) 
{ 
	os << "[" << ex.Get_Id()  << "] between [";
	if( ex.Area1 != NULL )
		os << ex.Area1->Get_Id();
	else
		os << "NULL";

	cout << "] and [";

	if( ex.Area2 != NULL )
		os << ex.Area2->Get_Id();
	else
		os << "NULL";
		
	cout << "] ";

	return os << ex.Segment;

};

/*--------------------------------------------------------------------------------------------------*/
cPortal::cPortal(int id)
{	
		Area1=NULL;
		Area2=NULL;			
		Id=id;			
}

/*--------------------------------------------------------------------------------------------------*/
cPortal::cPortal(cHV_Seg & seg , cSquare_Area * area , int id)
{	
		assert( area != NULL);
		Area1=area;
		Area2=NULL;
		Segment=seg;
		Id=id; 
		seg.Get_Middle(Position);			
}


/*--------------------------------------------------------------------------------------------------*/
float cPortal::Get_Cost( cA_Star_Node * destination)           
{ 
	assert( destination != NULL);
	assert( Voisinage.Contain(destination 	));

	return Get_Distance(Position , destination->Position);
};

/*--------------------------------------------------------------------------------------------------*/
float cPortal::Get_Estimated_Cost( cA_Star_Node * destination) 
{ 
	// on prendre pour cout estime la longeeur de la ligne droite
	assert( destination != NULL);
	return Get_Distance(Position , destination->Position);
};
	

/*--------------------------------------------------------------------------------------------------*/
void cPortal::Add_Area( cSquare_Area * new_area) 
{ 
	assert( Area1 != NULL || Area2!=NULL);
	assert( new_area != NULL);

	if(Area1==NULL) 
	{
		assert( Area1 != new_area);
		Area1 = new_area;			
	}
	else 
	{
		assert( Area2 != new_area);
		Area2 = new_area;			
	}
}


/*--------------------------------------------------------------------------------------------------*/
// met ds voisins la liste des noeuds voisins 
void  cPortal::Get_List_Neighbors( cList_A_Star_Nodes & voisins )
{
	assert( Area1 != NULL);
	assert( Area2 != NULL);
		
	cList_A_Star_Nodes::iterator nodes_itr;
	voisins.clear();
	for( nodes_itr = Voisinage.begin() ; nodes_itr != Voisinage.end() ; nodes_itr++)
			voisins.push_front( (*nodes_itr) );
	
};

/*--------------------------------------------------------------------------------------------------*/
bool cPortal::Is_Similar( cPortal & portal)                   
{
	return (( Area1 == portal.Area1 && Area2 == portal.Area2 ) ||
		    ( Area1 == portal.Area2 && Area2 == portal.Area1 ) );
}

/*--------------------------------------------------------------------------------------------------*/
void cPortal::Add_Voisin( cA_Star_Node * node)
{
	assert( node != NULL);
	Voisinage.push_front( node);		
}

/*--------------------------------------------------------------------------------------------------*/
void cPortal::Remove_Voisin( cA_Star_Node * node)
{		
	Voisinage.remove(node);
}

/****************************************************************************************************/
// class cA_Star_Point
/****************************************************************************************************/
/*--------------------------------------------------------------------------------------------------*/
cA_Star_Point::cA_Star_Point( cSquare_Area * area , cPoint2D& p) 
{ 
	Area=area; 
	Position=p; 
};

/*--------------------------------------------------------------------------------------------------*/
void  cA_Star_Point::Get_List_Neighbors( cList_A_Star_Nodes & voisinage )
{
	std::vector<cPortal*>::iterator portals_itr;
	assert( Area != NULL);
	for( portals_itr = Area->List_Portals.begin() ; portals_itr != Area->List_Portals.end() ; portals_itr++)
			voisinage.push_front( (*portals_itr) );
}

/*--------------------------------------------------------------------------------------------------*/
float cA_Star_Point::Get_Cost( cA_Star_Node * destination) 
{ 
	assert( destination != NULL);		 		
	return Get_Distance(Position , destination->Position);
};

/*--------------------------------------------------------------------------------------------------*/
float cA_Star_Point::Get_Estimated_Cost( cA_Star_Node * destination)
{ 
	// on prendre pour cout estime la distance directe
	assert( destination != NULL);
	return Get_Distance(Position , destination->Position);
};