#pragma warning(disable: 4786)
/*------------------------------------   Includes   ---------------------------------------------*/
#include <fstream>
#include <string>
#include <iostream>
#include <assert.h>

#include "bsp.h"
#include "level.h"
#include "game_manager.h"
#include "main.h"
#include "errrors_display.h"

using namespace std;

/*------------------------------------    Define    ---------------------------------------------*/
#define DELAY_BETWEEN_ADDED_ITEMS 0.5

#define PX_GAME_OVER_MESSAGE  TX_SCREEN/2-276/2
#define PY_GAME_OVER_MESSAGE  TY_SCREEN/2-128/2

#define PX_VICTORY_MESSAGE  TX_SCREEN/2-276/2
#define PY_VICTORY_MESSAGE  TY_SCREEN/2-128/2

#define PX_PRESS_SPACE_MESSAGE  TX_SCREEN/2-276/2
#define PY_PRESS_SPACE_MESSAGE  TY_SCREEN/2-128/2

#define CADEAUX_FREQUENCY  0.8    // delais entre l'addition des cadeaux !

#define PX_COEURS  50
#define PY_COEURS  550
#define ECART_X_COEURS 40

#define X_DISPLAY_LEFT_TIME 200
#define Y_DISPLAY_LEFT_TIME 510

#define X_DISPLAY_SCORE 200
#define Y_DISPLAY_SCORE 550

#define PERCENT_CADEAUX_POINTS 80
#define PERCENT_CADEAUX_TELEPORTEUR 6
#define PERCENT_CADEAUX_HOLOGRAMME 6
#define PERCENT_CADEAUX_BOMB 6
#define PERCENT_CADEAUX_COEUR 2


#define KEYDOWN(buf, key) (buf[key] & 0x80)



static char * Level_Files[N_LEVELS] = { "../levels/level1.txt","../levels/level2.txt","../levels/level3.txt"};




/****************************************************************************************************/
// class cGame_Manager
/****************************************************************************************************/
cGame_Manager::cGame_Manager()
{	
	bool bDisplay_Square_Areas   = false;  
	bool bDisplay_Ennemies_Path  = false;  
	bool bDisplay_Path_Graph     = false;
	bool bGame_Paused            = false;  
	bool bInvincible             = false;

	Game_World                   = NULL;
	Level_Loader                 = new cLevel_Loader();
	Initiate_Unit_Defs();
	World_Debug_Displayer        = new cGame_World_Debug_Displayer;	
	Player                       = new cPlayer(20,20);  // l'objet joueur est cre une seule fois ici
	Nb_Seconds_For_Level         = 200;	
	Game_State                   = INITIALIZING;			
	DXUtil_Timer( TIMER_RESET );
	Time_Of_Last_Bonus           = DXUtil_Timer( TIMER_GETAPPTIME );
	Time_Since_Level_Start       = 0.0;
	Time_Last_Generated_Cadeau   = DXUtil_Timer( TIMER_GETAPPTIME );
	bTeleporteur_In_Game         = false;
	Score                        = 0;
}


/*--------------------------------------------------------------------------------------------------*/
cGame_Manager::~cGame_Manager()
{
	delete( Level_Loader);
	delete( World_Debug_Displayer);
	delete( Player);
	
	std::list<cEnnemy*>::iterator            enn_itr;
	std::list<cInteractive_Item*>::iterator  bonus_itr;	

	// efface les ennemies
	for( enn_itr = Ennemies_List.begin() ; enn_itr != Ennemies_List.end() ; enn_itr++)
		delete( (*enn_itr) );

	// efface les interactive item
	for( bonus_itr = Bonus_List.begin() ; bonus_itr != Bonus_List.end() ; bonus_itr++)
		delete( (*bonus_itr) );

	SAFE_DELETE(pDisplay_White_Ball );
	SAFE_DELETE(pDisplay_Red_Ball    );
	SAFE_DELETE(pDisplay_Blue_Ball   );
	SAFE_DELETE(pDisplay_Green_Ball  );
	SAFE_DELETE(pDisplay_Bomb        ); 
	SAFE_DELETE(pDisplay_Teleporteur );
	SAFE_DELETE(pDisplay_Hologramme  );
	SAFE_DELETE(pDisplay_Cadeau      );
	SAFE_DELETE(pDisplay_Red_Trap    );
	SAFE_DELETE(pDisplay_Blue_Trap   );
	SAFE_DELETE(pDisplay_Green_Trap  );
	SAFE_DELETE(pDisplay_Coeur  );	

	SAFE_DELETE(pDisplay_Game_Over);
	SAFE_DELETE(pDisplay_Victory );
	SAFE_DELETE(pDisplay_Press_Space) ;

		
}

/*--------------------------------------------------------------------------------------------------*/
void cGame_Manager::Clear()
{
	// on efface le Game_World contenant la descritpion du inveau
	SAFE_DELETE(  Game_World );		
	
	std::list<cEnnemy*>::iterator            enn_itr;
	std::list<cInteractive_Item*>::iterator  bonus_itr;	

	// efface les ennemies
	for( enn_itr = Ennemies_List.begin() ; enn_itr != Ennemies_List.end() ; enn_itr++)
		delete( (*enn_itr) );

	// efface les interactive item
	for( bonus_itr = Bonus_List.begin() ; bonus_itr != Bonus_List.end() ; bonus_itr++)
		delete( (*bonus_itr) );

	Ennemies_List.clear();
	Bonus_List.clear();
	bTeleporteur_In_Game       = false;

}

/*--------------------------------------------------------------------------------------------------*/
void cGame_Manager::Reset()
{
	DXUtil_Timer( TIMER_RESET );

	Time_Of_Last_Bonus         = DXUtil_Timer( TIMER_GETAPPTIME );
	Time_Since_Level_Start     = 0.0;
	Time_Last_Generated_Cadeau = DXUtil_Timer( TIMER_GETAPPTIME );

	std::list<cInteractive_Item*>::iterator  bonus_itr;
	for( bonus_itr = Bonus_List.begin() ; bonus_itr != Bonus_List.end() ; bonus_itr++)
	{		
		(*bonus_itr)->Reset();
	}
}





/*--------------------------------------------------------------------------------------------------*/
bool cGame_Manager::Load_Level( int i)
{
	Clear();
	bool valid;
	Game_World = new cGame_World();
	valid = Level_Loader->Load(Level_Files[i] ,this ); 
	if( ! valid)		
		return false;

	Game_World->Decorate();	
	Reset();

	// le niveau est pret a etre lanc !!
	Game_State = READY_TO_START;	 
	return true;	
}




/*-----------------------------------------------------------------------------------------------*/
void cGame_Manager::Start()
{
	std::string err_message;
	
	Game_State                    = RUNNING;
	DXUtil_Timer( TIMER_RESET );
	Time_Of_Last_Bonus            = DXUtil_Timer( TIMER_GETAPPTIME );

	std::list<cEnnemy*>::iterator enn_itr;
	for( enn_itr = Ennemies_List.begin() ; enn_itr != Ennemies_List.end() ; enn_itr++)
		(*enn_itr)->Time_Stamp_Last_Move = DXUtil_Timer( TIMER_GETAPPTIME )-FREQUENCY_MOVE;

	Player->Time_Stamp_Last_Move = DXUtil_Timer( TIMER_GETAPPTIME )-FREQUENCY_MOVE;

}

/*-----------------------------------------------------------------------------------------------*/
void cGame_Manager::Try_Start()
{
	if( Game_State == READY_TO_START )
				Start() ;
}

/*-----------------------------------------------------------------------------------------------*/
bool cGame_Manager::Add_Bonus( Interactive_Item_Type bonus , int x, int y)
{
	switch(bonus)
	{
	case BOMB:
		if( Player->Nb_Bombs ==0 )
			return false;
		Bonus_List.push_front(new cBomb(x,y) );
		Player->Nb_Bombs--;
		return true;
	case TELEPORTEUR:
		if( Player->Nb_Teleporteurs ==0 )
			return false;
		Bonus_List.push_front(new cTeleporteur(x,y) );
		Player->Nb_Teleporteurs--;
		return true;
	case HOLOGRAMME:
		if( Player->Nb_Hologrammes ==0 )
			return false;
		Bonus_List.push_front(new cHologramme(x,y) );
		Player->Nb_Hologrammes --;
		return true;
	case RED_TRAP:
		Bonus_List.push_front(new cRed_Trap(x,y) );		
		return true;
	case BLUE_TRAP:
		Bonus_List.push_front(new cBlue_Trap(x,y) );		
		return true;
	case GREEN_TRAP:
		Bonus_List.push_front(new cGreen_Trap(x,y) );		
		return true;
	default:
		return false;
	}	
}

/*-----------------------------------------------------------------------------------------------*/
bool cGame_Manager::Add_Ennemy( Ennemy_Type bonus , int x, int y)
{
	switch(bonus)
	{
	case RED_BALL:		
		Ennemies_List.push_front(new cRed_Ball(x,y) );		
		return true;
	case BLUE_BALL:
		Ennemies_List.push_front(new cBlue_Ball(x,y) );		
		return true;		
	case GREEN_BALL:
		Ennemies_List.push_front(new cGreen_Ball(x,y) );		
		return true;		
	default:
		return false;
	}	
}


/*-----------------------------------------------------------------------------------------------*/
// display le mur d'un noeud
void Call_Back_Display_Wall(cHV_Node * node)
{	
	node->Divider.Display(g_pDisplay);
}


/*-----------------------------------------------------------------------------------------------*/
void cGame_Manager::Display_Score()
{	
	char c_str[20];	
	std::string str_score;
	DDSURFACEDESC2 ddsd;
	HRESULT        hr;	
	
	// on cherche la chaine de caractere du score
	_itoa(Score,c_str,10);
	str_score.erase();
	str_score.append( c_str );		

	str_score.copy(c_str,20,'\0');
	c_str[str_score.size()]='\0';
	
	// on efface le score precedent
	ddsd.dwSize = sizeof(ddsd);
    hr = g_pScore_Text->GetDDrawSurface()->Lock( NULL, &ddsd, DDLOCK_WAIT, NULL );
    if( hr != DD_OK)
    { 
		Display_Fatal_Error(hWnd , "cGame_Manager::Display_Score()",hr);
		_DbgOut("game_manager.cpp",119,hr,"void cGame_Manager::Display_Score()" );
		PostQuitMessage(0);
		return;        
    }
	else
	{
		for( int i=0;i<ddsd.dwHeight;i++)		
			memset( (unsigned char*)ddsd.lpSurface+i*ddsd.lPitch ,  0 , ddsd.dwWidth );		
		g_pScore_Text->GetDDrawSurface()->Unlock(NULL);
	}

	// on blit le nouveau score
	g_pScore_Text->DrawText(NULL,c_str,0,0, RGB(0,0,0),RGB(0,255,0) );
	g_pDisplay->Blt( X_DISPLAY_SCORE , Y_DISPLAY_SCORE , g_pScore_Text->GetDDrawSurface() , NULL);
}


/*-----------------------------------------------------------------------------------------------*/
void cGame_Manager::Display_Time()
{
	std::string str_time;
	str_time = 10;
	char c_str[20];	
	int n_mins;
	int n_secs;
	DDSURFACEDESC2 ddsd;
	HRESULT        hr;

	// on cherche la chaine de caractere du temps
	n_secs = Nb_Seconds_For_Level - (int)Time_Since_Level_Start;
	if( n_secs < 0) n_secs =0;

	n_mins = n_secs/60;
	n_secs -= n_mins*60;
	
	_itoa(n_mins,c_str,10);

	str_time.erase();
	str_time.append( c_str );
	c_str[0]=':';
	c_str[1]='\0';
	str_time.append( c_str);
	
	if( n_secs < 10)
	{
		c_str[0]='0';
		c_str[1]='\0';
		str_time.append( c_str);
	}

	_itoa(n_secs,c_str,10);
	str_time.append( c_str);
	
	
	c_str[0]='\0';	
	str_time.append( c_str);
	str_time.copy(c_str,20,'\0');
	c_str[str_time.size()]='\0';

	// on efface le temps precendent
	ddsd.dwSize = sizeof(ddsd);
    hr = g_pLeft_Time_Text->GetDDrawSurface()->Lock( NULL, &ddsd, DDLOCK_WAIT, NULL );
    if( hr != DD_OK)
    { 
		Display_Fatal_Error(hWnd , "cGame_Manager::Display()",hr);
		_DbgOut("game_manager.cpp",119,hr,"void cGame_Manager::Display()" );
		PostQuitMessage(0);
		return;        
    }
	else
	{
		for( int i=0;i<ddsd.dwHeight;i++)		
			memset( (unsigned char*)ddsd.lpSurface+i*ddsd.lPitch ,  0 , ddsd.dwWidth );		
		g_pLeft_Time_Text->GetDDrawSurface()->Unlock(NULL);
	}

	// on blit le nouveau temps
	g_pLeft_Time_Text->DrawText(NULL,c_str,0,0, RGB(0,0,0),RGB(0,255,0) );
	g_pDisplay->Blt( X_DISPLAY_LEFT_TIME , Y_DISPLAY_LEFT_TIME , g_pLeft_Time_Text->GetDDrawSurface() , NULL);
}


/*-----------------------------------------------------------------------------------------------*/
void cGame_Manager::Display_Debug_X()
{
	std::list<cEnnemy*>::iterator            enn_itr;

	if(bDisplay_Square_Areas)	
		World_Debug_Displayer->Display_Areas( *g_pDisplay , *Game_World );		

	if(bDisplay_Path_Graph)	
		World_Debug_Displayer->Display_Graph( *g_pDisplay , *Game_World );

	if( bDisplay_Ennemies_Path )
		for( enn_itr = Ennemies_List.begin() ; enn_itr != Ennemies_List.end() ; enn_itr++)
			(*enn_itr)->Display_Path();
}

/*-----------------------------------------------------------------------------------------------*/
void cGame_Manager::Display_Inventaire_Player()
{
	int i;

	for( i=0; i <Player->Nb_Bombs;i++)
		pDisplay_Bomb->Display(X_DISPLAY_BOMBS + i*TX_BOMB , Y_DISPLAY_BOMBS );

	for( i=0; i <Player->Nb_Teleporteurs;i++)
		pDisplay_Teleporteur->Display(X_DISPLAY_TELEPORTEURS + i*TX_TELEPORTEUR , Y_DISPLAY_TELEPORTEURS );

	for( i=0; i <Player->Nb_Hologrammes;i++)
		pDisplay_Hologramme->Display(X_DISPLAY_HOLOGRAMMES + i*TX_HOLOGRAMME , Y_DISPLAY_HOLOGRAMMES );

	for( i=0; i < Player->Nb_Lives;i++)
		pDisplay_Coeur->Display(PX_COEURS+i*ECART_X_COEURS , PY_COEURS);


}
/*-----------------------------------------------------------------------------------------------*/
void cGame_Manager::Display()
{	   

	std::list<cEnnemy*>::iterator            enn_itr;
	std::list<cInteractive_Item*>::iterator  bonus_itr;	
	
	g_pDisplay->Clear(0);
	g_pDisplay->Blt( 0 , 0 , g_pFond->GetDDrawSurface() , NULL);

	Display_Time();
	Display_Score();
	Display_Debug_X();
	Display_Inventaire_Player();		
	
	Game_World->My_BSP.Apply_To_Each_Node(Call_Back_Display_Wall);	

	if( Player != NULL)
		Player->Display();
	 
	for( enn_itr = Ennemies_List.begin() ; enn_itr != Ennemies_List.end() ; enn_itr++)
		(*enn_itr)->Display();

	for( bonus_itr = Bonus_List.begin() ; bonus_itr != Bonus_List.end() ; bonus_itr++)
		(*bonus_itr)->Display();	
}



/*-----------------------------------------------------------------------------------------------*/
HRESULT cGame_Manager::Read_Keyboard_And_React()
{
    HRESULT hr;    
    BYTE    diks[256];   // DirectInput keyboard state buffer 

    if( NULL == g_pKeyboard ) 
        return S_OK;
    
    // Get the input's device state, and put the state in dims
    ZeroMemory( diks, sizeof(diks) );
    hr = g_pKeyboard->GetDeviceState( sizeof(diks), diks );
    if( FAILED(hr) ) 
    {
        // DirectInput may be telling us that the input stream has been
        // interrupted.  We aren't tracking any state between polls, so
        // we don't have any special reset that needs to be done.
        // We just re-acquire and try again.
        
        // If input is lost then acquire and keep trying 
        hr = g_pKeyboard->Acquire();
        while( hr == DIERR_INPUTLOST ) 
            hr = g_pKeyboard->Acquire();        

        // hr may be DIERR_OTHERAPPHASPRIO or other errors.  This
        // may occur when the app is minimized or in the process of 
        // switching, so just try again later 
        return S_OK; 
    }
    
	assert( Player != NULL);

	
	
	int move_x=0;
	int move_y=0;

	// on gere les touches de directions
    if( KEYDOWN(diks , DIK_RIGHT) )	
		move_x += Player->Get_Speed();		
	if( KEYDOWN(diks , DIK_LEFT) )
		move_x -= Player->Get_Speed();				
	if( KEYDOWN(diks , DIK_DOWN) )
		move_y += Player->Get_Speed();		
	if( KEYDOWN(diks , DIK_UP) )
		move_y-= Player->Get_Speed();

	if( move_x || move_y)
		Player->Move(move_x,move_y);


	// touche 'B'
	if( KEYDOWN(diks , DIK_B) )	
	{
		if( DXUtil_Timer( TIMER_GETAPPTIME ) - Time_Of_Last_Bonus >DELAY_BETWEEN_ADDED_ITEMS)
		{				
			Add_Bonus( BOMB , Player->Px  , Player->Py );
			Time_Of_Last_Bonus = DXUtil_Timer( TIMER_GETAPPTIME );
		}
	}

	if( KEYDOWN(diks , DIK_T) )	
	{
		if( DXUtil_Timer( TIMER_GETAPPTIME ) - Time_Of_Last_Bonus >DELAY_BETWEEN_ADDED_ITEMS)
		{
			if( bTeleporteur_In_Game)
			{
				Player->Use_Teleporteur();
				Time_Of_Last_Bonus = DXUtil_Timer( TIMER_GETAPPTIME );
			}
			else
			{
				if( Add_Bonus( TELEPORTEUR , Player->Px  , Player->Py )) 
				{
					Time_Of_Last_Bonus = DXUtil_Timer( TIMER_GETAPPTIME );
					bTeleporteur_In_Game = true;
				}
			}
		}
	}

	if( KEYDOWN(diks , DIK_H) )	
	{
		if( DXUtil_Timer( TIMER_GETAPPTIME ) - Time_Of_Last_Bonus >DELAY_BETWEEN_ADDED_ITEMS)
		{				
			if( Add_Bonus( HOLOGRAMME , Player->Px  , Player->Py ))
				Time_Of_Last_Bonus = DXUtil_Timer( TIMER_GETAPPTIME );
		}
	}
		
    
    return S_OK;
}


/*----------------------------------------------------------------------------------------------*/
void cGame_Manager::Try_Kill_Player()
{
	std::list<cEnnemy*>::iterator enn_itr;

	Player->Nb_Lives--;
	if( Player->Nb_Lives <= 0)
		Game_State = GAME_OVER;
	else
	{
		Game_State = READY_TO_START;
		for( enn_itr = Ennemies_List.begin() ; enn_itr != Ennemies_List.end() ; enn_itr++)
		{
			(*enn_itr)->Reset();
		}				
		Nb_Seconds_For_Level -= Time_Since_Level_Start;
		Reset();
	}
}


/*--------------------------------------------------------------------------------------------------*/
void cGame_Manager::Update_Ennemies()
{	
	cPoint2D                       destination;	
	std::list<cEnnemy*>::iterator  enn_itr;

	for( enn_itr = Ennemies_List.begin() ; enn_itr != Ennemies_List.end() ; enn_itr++)	
		(*enn_itr)->Update_Path();
	
	for( enn_itr = Ennemies_List.begin() ; enn_itr != Ennemies_List.end() ; enn_itr++)	
		(*enn_itr)->Follow_Path();
}


/*--------------------------------------------------------------------------------------------------*/
void cGame_Manager::Test_Collisions()
{
	int                             rayon;
	unsigned int                    dist;

	std::list<cEnnemy*>::iterator              enn_itr;
	std::list<cInteractive_Item*>::iterator    bonus_itr;
	
	/*-----------------------------------*/
	// interactions player<->ennemis
	if( !bInvincible)
	{
		for( enn_itr = Ennemies_List.begin() ; enn_itr != Ennemies_List.end() ; enn_itr++)
		{
			dist = Get_Square_Distance( cPoint2D((*enn_itr)->Px, (*enn_itr)->Py) , 
										cPoint2D(Player->Px, Player->Py) );
			rayon = (*enn_itr)->Get_Rayon() + Player->Get_Rayon();
			if(  dist < rayon*rayon )
			{
				Try_Kill_Player();
				return;
			} 
		}
	}

	/*-----------------------------------*/
	// interactions bonus<->joueur			
	for( bonus_itr = Bonus_List.begin() ; bonus_itr != Bonus_List.end() ; bonus_itr++)
	{
		// on teste ce bonus % au joueur
		dist = Get_Square_Distance( cPoint2D((*bonus_itr)->Px, (*bonus_itr)->Py) , 
								    cPoint2D(Player->Px, Player->Py) );

		rayon = (*bonus_itr)->Get_Rayon() + Player->Get_Rayon();
		if(  dist < rayon*rayon )
		{
			Player->React_To_Bonus( (*bonus_itr) );			
		}
	}

	/*-----------------------------------*/
	// interactions bonus ennemies
	for( bonus_itr = Bonus_List.begin() ; bonus_itr != Bonus_List.end() ; bonus_itr++)
	{
		for( enn_itr = Ennemies_List.begin() ; enn_itr != Ennemies_List.end() ; enn_itr++)
		{
			dist = Get_Square_Distance( cPoint2D((*bonus_itr)->Px, (*bonus_itr)->Py) , 
									    cPoint2D((*enn_itr)->Px, (*enn_itr)->Py) );

			rayon = (*bonus_itr)->Get_Rayon() + (*bonus_itr)->Get_Rayon();
			if(  dist < rayon*rayon )
			{
				(*enn_itr)->React_To_Bonus( (*bonus_itr) );			
			}
		}
	}

	Erase_Ennemies_Garbage_Collector();
	Erase_Bonus_Garbage_Collector();
}

/*--------------------------------------------------------------------------------------------------*/
void cGame_Manager::Update_Game()
{
	Time_Since_Level_Start = DXUtil_Timer( TIMER_GETAPPTIME );	
	
	if( Time_Since_Level_Start >= Nb_Seconds_For_Level )  // temps coul
	{			
		Game_State = GAME_OVER;
		OutputDebugString( "GAME_OVER !!! \n" );
		return;
	}
	
	Update_Ennemies();
	Test_Collisions();
	Generate_Cadeaux();
	
	if( Ennemies_List.empty() )
	{
		Game_Manager.Game_State = LEVEL_FINISHED;
		return;
	}
}


/*--------------------------------------------------------------------------------------------------*/
void cGame_Manager::Kill_Ennemy( cEnnemy * ennemy )
{
	assert( ennemy != NULL);	
	Ennemies_Garbage_Collector.push_back( ennemy );
}

/*--------------------------------------------------------------------------------------------------*/
void cGame_Manager::Remove_Bonus( cInteractive_Item * bonus )
{
	assert( bonus != NULL);
	Bonus_Garbage_Collector.push_back( bonus );
}


/*--------------------------------------------------------------------------------------------------*/
void cGame_Manager::Erase_Ennemies_Garbage_Collector(  )
{	
	std::vector<cEnnemy*>::iterator enn_itr;	
	// on enleve de la liste des ennemis ceux du garbage collector
	for( enn_itr = Ennemies_Garbage_Collector.begin();enn_itr != Ennemies_Garbage_Collector.end();enn_itr++)
	{				
		Ennemies_List.remove((*enn_itr));
		delete( (*enn_itr));
	}
	//on vide le garbage
	Ennemies_Garbage_Collector.clear();
}

/*--------------------------------------------------------------------------------------------------*/
void cGame_Manager::Erase_Bonus_Garbage_Collector(  )
{	
	std::vector<cInteractive_Item*>::iterator bonus_itr;
	
	// on enleve de la liste des bonus ceux du garbage collector
	for( bonus_itr = Bonus_Garbage_Collector.begin();bonus_itr != Bonus_Garbage_Collector.end();bonus_itr++)
	{		
		Bonus_List.remove((*bonus_itr));
		delete( (*bonus_itr));
	}
	//on vide le garbage
	Bonus_Garbage_Collector.clear();

}

/*--------------------------------------------------------------------------------------------------*/
void cGame_Manager::Main_Loop()
{

  // on switch sur l'etat du jeu et agit en consequence
  switch( Game_Manager.Game_State )  
  {
  case RUNNING:               // on joue, boucle principale
	  if( ! bGame_Paused)
	  {
		  Game_Manager.Read_Keyboard_And_React(); // on lit les actions
		  Game_Manager.Update_Game();             // on update le jeu
	  }
	  Game_Manager.Display();	                  // on affiche
	  g_pDisplay->Present();
	  break;
  case INITIALIZING:	  
	  Game_Manager.Load_Level(0 );	  // on charge le premier niveau
	  break;
  case READY_TO_START:                // on affiche le message 'press space'
	  Game_Manager.Display();	  
	  pDisplay_Press_Space->Display(PX_PRESS_SPACE_MESSAGE,PY_PRESS_SPACE_MESSAGE);
	  g_pDisplay->Present();
	  break;
  case VICTORY:                       // on affiche le message 'victory'
	  Game_Manager.Display();
	  pDisplay_Victory->Display(PX_VICTORY_MESSAGE,PY_VICTORY_MESSAGE);
	  g_pDisplay->Present();
	  break;
  case LEVEL_FINISHED:              // on essaie de passer au niveau suivant
	  Game_Manager.Id_Level++;
	  if( Game_Manager.Id_Level == N_LEVELS)	  // si finit, state=victory
		  Game_Manager.Game_State = VICTORY;	  
	  else
	  {
		  if( !  Game_Manager.Load_Level( Game_Manager.Id_Level) )  // sinon, on charge le niveau suivant
		  {
			  Display_Fatal_Error(hWnd,"erreur  la lecture du niveau");
			  PostQuitMessage(0);
		  }
	  }
	  break;  
  case GAME_OVER :	                // on affiche le message display
	  Game_Manager.Display();	  	  
	  pDisplay_Game_Over->Display(PX_GAME_OVER_MESSAGE,PY_GAME_OVER_MESSAGE);
	  g_pDisplay->Present();
	  break;
  default:	          // erreur
	  Display_Fatal_Error(hWnd, "cGame_Manager::Main_Loop() : unknown state \n");
	  PostQuitMessage(0);
	  break;
  }

}

/*--------------------------------------------------------------------------------------------------*/
bool cGame_Manager::Is_Free_Place( int x, int y , int r ) 
{
	cPoint2D pos(x,y);
	unsigned int dist;
	unsigned int r_2;

	// on teste intesection avec les murs
	if( Game_World->Do_Ball_Intersect_Walls(x , y , r ) )
	{		
		return false;
	}

	// on teste intersection avec les ennemis
	std::list<cEnnemy*>::iterator enn_itr;
	for( enn_itr = Ennemies_List.begin() ; enn_itr != Ennemies_List.end() ; enn_itr++)
	{		
		dist = Get_Square_Distance( pos , cPoint2D((*enn_itr)->Px,(*enn_itr)->Py));
		r_2 =  (r + (*enn_itr)->Get_Rayon())*(r + (*enn_itr)->Get_Rayon());
		
		if( dist* dist < r_2*r_2)					
			return false;		
	}

	// on teste intersection avec les intercative items
	std::list<cInteractive_Item*>::iterator  bonus_itr;
	for( bonus_itr = Bonus_List.begin() ; bonus_itr != Bonus_List.end() ; bonus_itr++)
	{
		dist = Get_Square_Distance( pos , cPoint2D((*bonus_itr)->Px,(*bonus_itr)->Py));
		r_2 =  ( r + (*bonus_itr)->Get_Rayon())*( r + (*bonus_itr)->Get_Rayon());
		
		if( dist*dist < r_2*r_2 )					
			return false;				
	}

	// on teste les intersection avec le joueur
	dist = Get_Square_Distance( pos , cPoint2D(Player->Px,Player->Py));
	r_2 = ( r + Player->Get_Rayon())*( r + Player->Get_Rayon());	
	if( dist*dist < r_2 *r_2)			
		return false;		
	

	return true;
}



/*--------------------------------------------------------------------------------------------------*/
void cGame_Manager::Generate_Cadeaux()
{
	// on verifie que les probas sont bien distribues
	assert( PERCENT_CADEAUX_POINTS + PERCENT_CADEAUX_TELEPORTEUR  + 
		    PERCENT_CADEAUX_HOLOGRAMME + PERCENT_CADEAUX_BOMB + PERCENT_CADEAUX_COEUR ==100);

	int x, y;
	int ind_cado;
	cInteractive_Item * cado=NULL;

	// si il est temps de rajouter un nouveau cado
	if(  DXUtil_Timer( TIMER_GETAPPTIME ) - Time_Last_Generated_Cadeau > CADEAUX_FREQUENCY )
	{
		//on cherche une place pour mettre le cado
		do
		{
			x = Rnd_Gen.Get_Int( RAYON_CADEAU +1, cGame_World::Ecran.p1.X - RAYON_CADEAU-1);
			y = Rnd_Gen.Get_Int( RAYON_CADEAU +1, cGame_World::Ecran.p1.Y - RAYON_CADEAU-1);			
		}while( !Is_Free_Place( x, y , RAYON_CADEAU ) );		

		// on genere un cado au hasard, selon les proba dfinies 
		ind_cado = Rnd_Gen.Get_Int(0,99);
		if( ind_cado < PERCENT_CADEAUX_POINTS)
		{
			cado = new cCadeau(x,y,CADEAU_POINTS);
			Bonus_List.push_front(cado);
		}
		else if( ind_cado < PERCENT_CADEAUX_POINTS+PERCENT_CADEAUX_TELEPORTEUR)
		{	
			cado = new cCadeau(x,y,CADEAU_TELEPORTEUR);
			Bonus_List.push_front(cado);
		}
		else if( ind_cado < PERCENT_CADEAUX_POINTS+PERCENT_CADEAUX_TELEPORTEUR+PERCENT_CADEAUX_HOLOGRAMME)
		{
			cado = new cCadeau(x,y,CADEAU_HOLOGRAMME);
			Bonus_List.push_front(cado);
		}
		else if( ind_cado < PERCENT_CADEAUX_POINTS+PERCENT_CADEAUX_TELEPORTEUR+PERCENT_CADEAUX_HOLOGRAMME+PERCENT_CADEAUX_BOMB)
		{
			cado = new cCadeau(x,y,CADEAU_BOMB);
			Bonus_List.push_front(cado);
		}
		else if( ind_cado < PERCENT_CADEAUX_POINTS+PERCENT_CADEAUX_TELEPORTEUR+PERCENT_CADEAUX_HOLOGRAMME+PERCENT_CADEAUX_BOMB+PERCENT_CADEAUX_COEUR)
		{
			cado = new cCadeau(x,y,CADEAU_COEUR);
			Bonus_List.push_front(cado);
		}
		else
		{
			Fatal_Error << "nb aleatoire out of range : " << ind_cado << "\n";
			PostQuitMessage(0);			
		}

		Time_Last_Generated_Cadeau = DXUtil_Timer( TIMER_GETAPPTIME );
	}
}


/*--------------------------------------------------------------------------------------------------*/
void cGame_Manager::Initiate_Unit_Defs()
{
	// on met dans chaque objet de classe d'affichage son bon RECT
	RECT rc_bitmap;
	rc_bitmap.bottom = 21;
	rc_bitmap.left   = 1;
	rc_bitmap.right  = 21;
	rc_bitmap.top    = 1;
	pDisplay_White_Ball  = new cSprite_Display_Def(  rc_bitmap);
	
	rc_bitmap.bottom = 21;
	rc_bitmap.left   = 22;
	rc_bitmap.right  = 42;
	rc_bitmap.top    = 1;
	pDisplay_Red_Ball    = new cSprite_Display_Def( rc_bitmap);
	
	rc_bitmap.bottom = 21;
	rc_bitmap.left   = 43;
	rc_bitmap.right  = 63;
	rc_bitmap.top    = 1;
	pDisplay_Blue_Ball   = new cSprite_Display_Def( rc_bitmap);

	rc_bitmap.bottom = 21;
	rc_bitmap.left   = 64;
	rc_bitmap.right  = 84;
	rc_bitmap.top    = 1;
	pDisplay_Green_Ball   = new cSprite_Display_Def( rc_bitmap);	

	rc_bitmap.bottom = 63;
	rc_bitmap.left   = 43;
	rc_bitmap.right  = 63;
	rc_bitmap.top    = 43;
	pDisplay_Bomb        = new cSprite_Display_Def(  rc_bitmap);

	rc_bitmap.bottom = 63;
	rc_bitmap.left   = 1;
	rc_bitmap.right  = 21;
	rc_bitmap.top    = 43;	
	pDisplay_Teleporteur = new cSprite_Display_Def( rc_bitmap);	
	
	rc_bitmap.bottom = 63;
	rc_bitmap.left   = 22;
	rc_bitmap.right  = 42;
	rc_bitmap.top    = 43;
	pDisplay_Hologramme  = new cSprite_Display_Def( rc_bitmap);

	rc_bitmap.bottom = 63;
	rc_bitmap.left   = 64;
	rc_bitmap.right  = 84;
	rc_bitmap.top    = 43;
	pDisplay_Cadeau      = new cSprite_Display_Def( rc_bitmap);

	rc_bitmap.bottom = 42;
	rc_bitmap.left   = 22;
	rc_bitmap.right  = 42;
	rc_bitmap.top    = 22;
	pDisplay_Red_Trap = new cSprite_Display_Def( rc_bitmap);	
	
	rc_bitmap.bottom = 42;
	rc_bitmap.left   = 43;
	rc_bitmap.right  = 63;
	rc_bitmap.top    = 22;
	pDisplay_Blue_Trap   = new cSprite_Display_Def( rc_bitmap);

	rc_bitmap.bottom = 42;
	rc_bitmap.left   = 64;
	rc_bitmap.right  = 84;
	rc_bitmap.top    = 22;
	pDisplay_Green_Trap  = new cSprite_Display_Def( rc_bitmap);

	rc_bitmap.bottom = 42;
	rc_bitmap.left   = 1;
	rc_bitmap.right  = 21;
	rc_bitmap.top    = 22;
	pDisplay_Coeur  = new cSprite_Display_Def( rc_bitmap);




	rc_bitmap.bottom = 64;
	rc_bitmap.left   = 0;
	rc_bitmap.right  = 276;
	rc_bitmap.top    = 0;
	pDisplay_Game_Over   = new cBitmap_Display_Def( rc_bitmap);

	rc_bitmap.bottom = 127;
	rc_bitmap.left   = 0;
	rc_bitmap.right  = 276;
	rc_bitmap.top    = 63;
	pDisplay_Victory   = new cBitmap_Display_Def( rc_bitmap);

	rc_bitmap.bottom = 190;
	rc_bitmap.left   = 0;
	rc_bitmap.right  = 276;
	rc_bitmap.top    = 126;
	pDisplay_Press_Space = new cBitmap_Display_Def( rc_bitmap);
	
}

