/*------------------------------------   Includes   ---------------------------------------------*/
#include "geo_utils.h"

#include <math.h>
#include <assert.h>

using namespace std;

/*--------------------------------------------------------------------------------------------------*/
float Get_Distance( cPoint2D& p1 , cPoint2D& p2 )
{
	return (float)sqrt( ( p1.X-p2.X ) * ( p1.X-p2.X )
				       +( p1.Y-p2.Y ) * ( p1.Y-p2.Y ) );

}

/*--------------------------------------------------------------------------------------------------*/
unsigned int Get_Square_Distance( cPoint2D& p1 , cPoint2D& p2 )
{
	return ( ( p1.X-p2.X ) * ( p1.X-p2.X ) + ( p1.Y-p2.Y ) * ( p1.Y-p2.Y ) );
}



/****************************************************************************************************/
// class cRectangle
/****************************************************************************************************/
void cRectangle::Get_Centre(cPoint2D & p)
{
	p.Set( (p0.X + p1.X)/2 , (p0.Y + p1.Y)/2 );
}





/****************************************************************************************************/
// class cPositive_Rectangle
/****************************************************************************************************/
cPositive_Rectangle::cPositive_Rectangle(cPoint2D &pA , cPoint2D &pB):cRectangle()
{ 
	int xmin,xmax,ymin,ymax;			
	if( pA.X <= pB.X)
	{
		xmin=pA.X;
		xmax=pB.X;
	}
	else
	{
		xmin=pB.X;
		xmax=pA.X;
	}

	if( pA.Y <= pB.Y)
	{
		ymin=pA.Y;
		ymax=pB.Y;
	}
	else
	{
		ymin=pB.Y;
		ymax=pA.Y;
	}

	p0.X = xmin;
	p0.Y = ymin; 
	p1.X = xmax;
	p1.Y = ymax; 
}

/*--------------------------------------------------------------------------------------------------*/
void cPositive_Rectangle::Get_Nord_Seg( cHV_Seg & seg )
{
	cPoint2D pA,pB;
	pA = p0;
	pB.Set( p1.X ,	p0.Y );
	seg.Set( pA , pB);	
}

/*--------------------------------------------------------------------------------------------------*/
void cPositive_Rectangle::Get_Ouest_Seg( cHV_Seg & seg )
{
	cPoint2D pA,pB;
	pA = p0;
	pB.Set( p0.X ,	p1.Y );
	seg.Set( pA , pB);

}

/*--------------------------------------------------------------------------------------------------*/
void cPositive_Rectangle::Get_Est_Seg( cHV_Seg & seg )
{
	cPoint2D pA,pB;
	pA.Set( p1.X ,	p0.Y );
	pB =p1;
	seg.Set( pA , pB);
}

/*--------------------------------------------------------------------------------------------------*/
void cPositive_Rectangle::Get_Sud_Seg( cHV_Seg & seg )
{
	cPoint2D pA,pB;
	pA.Set( p0.X ,	p1.Y );
	pB = p1;		
	seg.Set( pA , pB);
}



/*--------------------------------------------------------------------------------------------------*/
bool cPositive_Rectangle::In( cPoint2D &pA )
{
	return ( pA.Get_X()>=p0.X && pA.Get_X() <=p1.X && pA.Get_Y()>=p0.Y && pA.Get_Y() <=p1.Y );
};




/****************************************************************************************************/
// class cSeg
/****************************************************************************************************/
float cSeg::Get_Square_Distance_From_Point( cPoint2D & C )
{
	float AB[2];  // represente le segment
	float AC[2];
	float AC_sur_AB;
	float AE_2;  // E : projete de C sur AB, dist carre
	float AB_2;  // E : projete de C sur AB, dist carre
	float AC_2;  // E : projete de C sur AB, dist carre
	float BC_2;  // E : projete de C sur AB, dist carre

	AB[0] = p1.Get_X() - p0.Get_X();
	AB[1] = p1.Get_Y() - p0.Get_Y();
	AC[0] = C.Get_X() - p0.Get_X();
	AC[1] = C.Get_Y() - p0.Get_Y();

	AC_sur_AB =  AB[0] * AC[0] + AB[1] * AC[1];
	
	
	
	if( AC_sur_AB <0)     // le point le plus pres de C sur le segment est le point A
	{
		
		return AC[0]*AC[0]+AC[1]*AC[1];   // AC
	}
	else                 // le projet de de C est a droite de A
	{   
		AB_2 = AB[0]*AB[0]+AB[1]*AB[1];		
		AE_2 = AC_sur_AB * AC_sur_AB / AB_2;

		
		if( AE_2 < AB_2)          // le projete est sur le segment, on retourne CE
		{
			
			AC_2 = AC[0]*AC[0]+AC[1]*AC[1];
			return AC_2 - AE_2;
		}
		else                     // le projete est a droite de B, on retourne BC
		{
			
			BC_2 = ( C.Get_X() - p1.Get_X())*( C.Get_X() - p1.Get_X() )    // BC
                 + ( C.Get_Y() - p1.Get_Y())*( C.Get_Y() - p1.Get_Y() ) ;
			return BC_2;
		}
	}


}

/*--------------------------------------------------------------------------------------------------*/
void cSeg::Get_Middle(cPoint2D& p)       
{ 
		p.Set( (p0.Get_X() +  p1.Get_X())/2  , (p0.Get_Y() +  p1.Get_Y())/2 ) ;
}

/*--------------------------------------------------------------------------------------------------*/
bool cSeg::operator==(cSeg& seg2)                           
{ 
	return ( (p0==seg2.Get_p0() && p1==seg2.Get_p1()) || 
		     (p0==seg2.Get_p1() && p1==seg2.Get_p0()) );
}

/*--------------------------------------------------------------------------------------------------*/
unsigned int cSeg::Get_Square_Length() 
{ 
	return (p0.Get_X()-p1.Get_X()) * (p0.Get_X()-p1.Get_X())
		+  (p0.Get_Y()-p1.Get_Y()) * (p0.Get_Y()-p1.Get_Y()) ;
}







/****************************************************************************************************/
// class cHV_Seg
/****************************************************************************************************/
cHV_Seg::cHV_Seg(cPoint2D &pA , cPoint2D &pB):cSeg(pA , pB)
{
	int xmin,xmax,ymin,ymax;

	if( pA.Get_X() <= pB.Get_X() )
	{
		xmin = pA.Get_X();
		xmax = pB.Get_X();
	}
	else
	{
		xmin = pB.Get_X();
		xmax = pA.Get_X();
	}

	if( pA.Get_Y() <= pB.Get_Y() )
	{
		ymin = pA.Get_Y();
		ymax = pB.Get_Y();
	}
	else
	{
		ymin = pB.Get_Y();
		ymax = pA.Get_Y();
	}

	if( ymin == ymax)
	{
		bHorizontal = true;
		p0.Set( xmin,ymin);
		p1.Set( xmax,ymin);
	}
	else if( xmin == xmax)
	{
		bHorizontal = false;
		p0.Set( xmin,ymin);
		p1.Set( xmin,ymax);
	}
	else
		std::cout << "cHV_Seg(cPoint2D &pA , cPoint2D &pB) : [" << pA << "-->" << pB << "] not a valid HV seg \n";
}

/*--------------------------------------------------------------------------------------------------*/
void cHV_Seg::Set( cPoint2D &pA , cPoint2D &pB )
{
	int xmin,xmax,ymin,ymax;

	if( pA.Get_X() <= pB.Get_X() )
	{
		xmin = pA.Get_X();
		xmax = pB.Get_X();
	}
	else
	{
		xmin = pB.Get_X();
		xmax = pA.Get_X();
	}

	if( pA.Get_Y() <= pB.Get_Y() )
	{
		ymin = pA.Get_Y();
		ymax = pB.Get_Y();
	}
	else
	{
		ymin = pB.Get_Y();
		ymax = pA.Get_Y();
	}

	if( ymin == ymax)
	{
		bHorizontal = true;
		p0.Set( xmin,ymin);
		p1.Set( xmax,ymin);
	}
	else if( xmin == xmax)
	{
		bHorizontal = false;
		p0.Set( xmin,ymin);
		p1.Set( xmin,ymax);
	}
	else
		std::cout << "cHV_Seg(cPoint2D &pA , cPoint2D &pB) : [" << pA << "-->" << pB << "] not a valid HV seg \n";
}


/*--------------------------------------------------------------------------------------------------*/
bool cHV_Seg::Get_Intersection( cHV_Seg& seg , cHV_Seg& result)
{
	cPoint2D pA,pB;	

	if( bHorizontal)
	{
		if( seg.Is_Horizontal() )  // les deux sont horizontaux
		{
			if( p0.Get_Y() != seg.p0.Get_Y() ) // les murs ne sont pas a la meme hauteur
				return false;
			else
			{
				if( seg.p1.Get_X() < p0.Get_X() )  // seg tt a gauche
					return false;
				if( seg.p0.Get_X() > p1.Get_X() )  // seg tt a droite
					return false;
				else
				{
					// le seg inter commence au plus grd X des deux points initiaux
					if( seg.p0.Get_X() < p0.Get_X() )
						pA = p0;
					else
						pA = seg.p0;

					// le seg inter termine au plus petit X des deux points initiaux
					if( seg.p1.Get_X() < p1.Get_X() )
						pB = seg.p1;
					else
						pB = p1;

					result.Set(pA,pB);
					return true;
				}
			}
		}
		else  // le premier mur est horizontal et le secon vertical
		{
			pA.X = seg.Get_p0().X;
			pA.Y = p0.Y;						
			result.Set( pA , pA);
			return true;
		}

	}
	else  // mur vertical
	{
		if( ! seg.Is_Horizontal() )  // les deux sont verticaux
		{
			if( seg.p1.Get_X() != p0.Get_X() )  // seg pas a la meme abcisse
					return false;
			else
			{
				if( seg.p1.Get_Y() < p0.Get_Y() )  // seg tt dessous
						return false;
				if( seg.p0.Get_Y() > p1.Get_Y() )  // seg tt au dessus
					return false;
				else
				{
					if( seg.p0.Get_Y() < p0.Get_Y() )
						pA = p0;
					else
						pA = seg.p0;

					if( seg.p1.Get_Y() < p1.Get_Y() )
						pB = seg.p1;
					else
						pB = p1;
					result.Set(pA,pB);
					return true;
				}
			}
		}
		else
		{
			pA.X = p0.X;
			pA.Y = seg.Get_p0().Y;					
			result.Set( pA , pA);
			return true;
			
		}
	}
}


bool cHV_Seg::Contient( cHV_Seg& seg )
{
	cHV_Seg inter;
	bool result;
	result = Get_Intersection( seg , inter);

	if( !result)
		return false;

	if( inter == seg)
		return true;
	else
		return false;

}

/*   seg doit etre contenu dans ce segment !!!!!!
 return :
	* 0 -> pas d'intersection
	* 1 -> egalit entre les segments ( this == seg)
	* 2 -> seg est strictement contenu ds ce segment, comp1 et comp2 contiennent les segments complementaires
	*    tq que complementaire1 soit au dessus de complementaire 2 ( abcisse infrieure, ou ord infrieure)
	*      this = comp1<->inter<->comp2
	* 3 -> complementaire1 est valide, complementaire 2 est vide
	*      this = comp1<->inter
	* 4 -> complementaire2 est valide, complementaire 1 est vide
	*      this = inter<->comp2	
	*/

int  cHV_Seg::Get_Complementaires( cHV_Seg& seg ,  cHV_Seg& complementaire1, cHV_Seg& complementaire2)
{

	assert( Contient(seg) );

	bool result;
	cHV_Seg inter;
	result = Get_Intersection( seg , inter);
	
	if( result)
	{
		if( (*this) == seg)
			return 1;
		else
		{			
			if( inter.Get_p0() != Get_p0() )  // comp1 valide
			{
				complementaire1.Set( Get_p0() , inter.Get_p0());
				if( inter.Get_p1() != Get_p1())  //comp2 valide
				{					
					complementaire2.Set( inter.Get_p1(), Get_p1()  );
					return 2;
				}
				else
				{
					return 3;
				}
			}
			else if( inter.Get_p1() != Get_p1() ) // comp2 valide
			{
				complementaire2.Set( inter.Get_p1(), Get_p1()  );
				return 4;
			}
			else
				assert(0);
		}
	}
	else
		return 0;

	return 0;	
}



/*--------------------------------------------------------------------------------------------------*/
bool cHV_Seg::Contient_Point(cPoint2D & A)
{	
	if( bHorizontal)
	{
		if( A.Get_Y() != Get_p0().Get_Y() )
			return false;
		else if( A.Get_X() < Get_p0().Get_X() || A.Get_X() > Get_p1().Get_X() ) // fonctionne car le cHV_Seg est ordonn croissant
			return false;
		return true;
	}
	else
	{
		if( A.Get_X() != Get_p0().Get_X() )
			return false;
		else if( A.Get_Y() < Get_p0().Get_Y() || A.Get_Y() > Get_p1().Get_Y() ) // fonctionne car le cHV_Seg est ordonn croissant
			return false;
		return true;
	}
}



bool cHV_Seg::On_Same_Line( cHV_Seg& seg )
{
	if( bHorizontal)
	{
		if( seg.Is_Horizontal() )
		{
			if( seg.Get_p0().Y == p0.Y)
				return true;
			else
				return false;
		}
		else
		{
			return false;
		}
	}
	else
	{
		if( seg.Is_Horizontal() )
		{
			return false;
		}
		else
		{			
			if( seg.Get_p0().X == p0.X)
				return true;
			else
				return false;			
		}
	}
}
