/***************************************************************************************************/
/***************************************************************************************************/
//! \file geo_utils.h
//! \brief Declare des classes de base en gomtrie 2D
//!    
//!	Auteur : Adrien Auclair
//!
//! Au vu des dlais limits, les classes utilisent surtout des operations simples entre des 
//! objets simples ( segments horizontaux ou verticaux)
//! 
//! TODO : developper les memes outils sur des segments quelconques
/***************************************************************************************************/
/***************************************************************************************************/
#ifndef GEO_UTILS_H
#define GEO_UTILS_H


/*------------------------------------  Prototypes  ---------------------------------------------*/
class cPoint2D;
class cRectangle;
class cPositive_Rectangle;
class cHV_Seg;
class cSeg;

/*------------------------------------   Includes   ---------------------------------------------*/
#include <iostream>

/*-------------------------------------  Globals  -----------------------------------------------*/
//! retourne la distance ente 'p1' et 'p2'
extern float Get_Distance( cPoint2D& p1 , cPoint2D& p2 );

//! retourne la distance au carr entre 'p1' et 'p2'
extern unsigned int Get_Square_Distance( cPoint2D& p1 , cPoint2D& p2 );



/*-------------------------------------   Classes   ---------------------------------------------*/


/*************************************************************************************************/
//! point 2D simple
/*************************************************************************************************/
class cPoint2D
{
	friend std::ostream &operator<<(std::ostream &os, const cPoint2D &ex) { return os << "[" << ex.X << "," <<  ex.Y << "]"; };
public:
	/// constructeur par defaut : [0,0]
	inline cPoint2D()                    
	{
		X=0;
		Y=0;
	};
	// constructeur, d'un point en x,y
	inline cPoint2D(int x,int y)         
	{ 
		X=x ; 
		Y=y; 
	};

	/// retourne l'abcisse du point
	inline Get_X() { return X;};         
	/// retourne l'ordonne du point
	inline Get_Y() { return Y;};         
	/// set les coord du point  [x,y]
	inline Set(int x, int y)             
	{
		X=x;
		Y=y;
	};
	
	/// operateur d'egalit
	inline bool operator==(cPoint2D& p)  
	{ 
		return ( (X==p.X) && (Y==p.Y) );
	};
	
	/// operateur de difference
	inline bool operator!=(cPoint2D& p)  
	{ 
		return ( (X!=p.X) || (Y!=p.Y) );
	};

	int X;                               ///< abcisse du point                
	int Y;	                             ///< ordonne du point
};



/*************************************************************************************************/
//! rectangle, dfini par deux points opposs
/*************************************************************************************************/
class cRectangle
{
	friend std::ostream &operator<<(std::ostream &os, const cRectangle &ex) { return os << "[" << ex.p0 <<"<->" <<  ex.p1 << "]"; };
public:	
	/// construit un rectangle par default : [0,0]  [0,0]
	cRectangle()                                  
	{ 
		p0.X=0; 
		p0.Y=0; 
		p1.X=0; 
		p1.Y=0;
	};
	/// construit un rectangle entre [x0,y0] et [x1,y1]
	cRectangle(int x0 , int y0 , int x1 , int y1) 
	{ 
		p0.X = x0;
		p0.Y = y0; 
		p1.X = x1; 
		p1.Y = y1; 
	};
	/// construit un rectangle entre 'pA' et 'pB'
	cRectangle(cPoint2D pA , cPoint2D pB)         
	{ 
		p0=pA ; 
		p1=pB; 
	};
	/// met ds 'p' le point au centre du rectangle	
	void Get_Centre(cPoint2D & p);                

	cPoint2D p0;                                  ///< premier point dans un angle du rectangle
	cPoint2D p1;                                  ///< point dans l'angle oppos  p0
};




/*************************************************************************************************/
//! rectangle, dfini par deux points opposs ordonns
/*! p0.X < p1.X  \n
    p0.Y < p1.Y
**************************************************************************************************/
class cPositive_Rectangle : public cRectangle
{		
public:
	/// constructeur par dedault
	cPositive_Rectangle():cRectangle() {};                
	/// construit un rectangle entre 'pA' et 'pB'	
	cPositive_Rectangle(cPoint2D &pA , cPoint2D &pB);	  

	/// met ds 'seg' le seg nord du rectangle
	void Get_Nord_Seg( cHV_Seg & seg );                   
	/// met ds 'seg' le seg ouest du rectangle
	void Get_Ouest_Seg( cHV_Seg & seg );                  
	/// met ds 'seg' le seg est du rectangle
	void Get_Est_Seg( cHV_Seg & seg );                    
	/// met ds 'seg' le seg sud du rectangle
	void Get_Sud_Seg( cHV_Seg & seg );                    

	/// retourne vrai si '[x,y]' est  l'intrieure du rectangle, frontiere comprise
	inline bool In( int x , int y )                       
	{
		return ( x>=p0.X && x <=p1.X && y>=p0.Y && y <=p1.Y );
	}
	
	/// retourne vrai si 'pA' est  l'intrieure du rectangle, frontiere comprise
	bool In( cPoint2D &pA );	
};



/*************************************************************************************************/
//! segment
/*************************************************************************************************/
class cSeg
{
	friend std::ostream &operator<<(std::ostream &os, const cSeg &ex) { return os << "[" << ex.p0 << ","  << ex.p1 << "]"; };
public:
	/// constructeur par default, seg=[0,0]->[0,0]
	cSeg()                                                
	{
		p0.Set(0,0);
		p1.Set(0,0);
	};
	
	/// construit un segment entre 'pA' et 'pB'
	cSeg(cPoint2D &pA , cPoint2D &pB)                     
	{ 
		p0=pA;
		p1=pB;
	};
	
	/// met dans 'p' le milieu du segment
	void Get_Middle(cPoint2D& p);                         
	/// retourne la distance entre 'pA' et ce segment
	float Get_Square_Distance_From_Point( cPoint2D &pA ); 
	/// retourne une reference vers le premier point du segment
	cPoint2D& Get_p0() { return p0;};                     
	/// retourne une reference vers le second point du segment	
	cPoint2D& Get_p1() { return p1;};                     
	/// retourne vrai si ce seg est egal  'seg2'	
	bool operator==(cSeg& seg2);                          
	/// retourne la longueur au carr de ce segment
	unsigned int Get_Square_Length();   
	
	virtual void Set( cPoint2D &pA , cPoint2D &pB ) { p0=pA;p1=pB; };

protected:
	cPoint2D p0;                                          ///< point de dpart du segment
	cPoint2D p1;                                          ///< point d'arrive du segment
};



/*************************************************************************************************/
//! seg horizontal ou vertical
/*! les coordonnes sont ds l'ordre croissant!
**************************************************************************************************/
class cHV_Seg : public cSeg
{
public:
	/// constructeur par default	
	cHV_Seg():cSeg() { bHorizontal=true;};               
	/// construit un segment de 'pA'  'pB'
	cHV_Seg(cPoint2D &pA , cPoint2D &pB);                
	
	// fixe le segment comme allant de 'pA'  'pB'
	void Set( cPoint2D &pA , cPoint2D &pB );             
	
	/// calcule l'intersection de ce segment avec seg et met le resultat ds cHv_Seg
	/** retourne false si intersection vide, true sinon \n
	*  si l'intersection est un point, le seg resultat est reduit a un point */
	bool Get_Intersection( cHV_Seg& seg , cHV_Seg& result);	

	/// calcule l'intersection entre ce segment et 'seg', predond : 'seg' contenu dans ce segment!!!
	/**
	* return :
	* 0 -> pas d'intersection
	* 1 -> egalit entre les segments ( this == seg)
	* 2 -> seg est strictement contenu ds ce segment, comp1 et comp2 contiennent les segments complementaires
	*    tq que complementaire1 soit au dessus de complementaire 2 ( abcisse infrieure, ou ord infrieure)
	*      this = comp1<->inter<->comp2
	* 3 -> complementaire1 est valide, complementaire 2 est vide
	*      this = comp1<->inter
	* 4 -> complementaire2 est valide, complementaire 1 est vide
	*      this = inter<->comp2
	*/
	int  Get_Complementaires( cHV_Seg& seg , cHV_Seg& complementaire1, cHV_Seg& complementaire2);	

	///bool Contient_Point(cPoint2D & A);
	bool Contient_Point(cPoint2D & A);

	///int Get_Exact_Intersection(cSeg &seg , cPoint2D & A , cHV_Seg& seg_result);
	int Get_Exact_Intersection(cSeg &seg , cPoint2D & A , cHV_Seg& seg_result);

	/// true si horizontal, false si vertical 
	inline bool Is_Horizontal() { return bHorizontal;};  

	bool cHV_Seg::Contient( cHV_Seg& seg );

	bool On_Same_Line( cHV_Seg& seg );
private:
	bool bHorizontal;                                    ///< true si le segment est horizontal, false si vertical
	
};


#endif
