#pragma warning(disable: 4786)

/*------------------------------------   Includes   ---------------------------------------------*/
#include <fstream>
#include <string>
#include <iostream>

#include "bsp.h"
#include "level.h"
#include "main.h"
#include "errrors_display.h"


using namespace std;


typedef enum { READING_AREA , READING_WALL , READING_UNDEF , READING_RED_TRAP , READING_BLUE_TRAP , READING_GREEN_TRAP ,
				READING_RED_BALL,READING_BLUE_BALL,READING_GREEN_BALL , READING_TIME , READING_PLAYER } READING_STATE;


/*------------------------------------  Prototypes  ---------------------------------------------*/
bool Get_Pair(int &x, int &y , ifstream& in);




/*-----------------------------------------------------------------------------------------*/
// lit le texte "[x,y]" dans le stream 'in' et met a jour 'x' et 'y' avec les valeurs lues
// return true s'il n'y a pas eu d'erreur, false sinon
bool Get_Pair(int &x, int &y , ifstream& in)
{
	char c;
	
	in >> c;
	if( c != '[' )
		return false;
	in >> x;
	in >> c;
	if( c != ',' )
		return false;
	in >> y;
	in >> c;
	if( c != ']' )
		return false;

	return true;
}



/****************************************************************************************************/
// class cLevel_Loader
/****************************************************************************************************/
/*-----------------------------------------------------------------------------------------*/
bool cLevel_Loader::Read_Area(ifstream& in,  cGame_Manager * game)
{	
	int x,y;
	if( !Get_Pair( x, y , in))
		return false;

	// on fixe la zone ecran	
	cGame_World::Ecran = cPositive_Rectangle( cPoint2D(0,0) , cPoint2D(x,y));
	return true;
}

/*-----------------------------------------------------------------------------------------*/
bool cLevel_Loader::Read_Player(ifstream& in,  cGame_Manager * game)
{	
	int x,y;
	if( !Get_Pair( x, y , in))
		return false;

	// on fixe la zone ecran	
	game->Player->Px = x;
	game->Player->Py = y;
	return true;
}


/*-----------------------------------------------------------------------------------------*/
bool cLevel_Loader::Read_Time(ifstream& in,  cGame_Manager * game)
{		
	int time;
	in >> time ;
	game->Nb_Seconds_For_Level = time;
	// on fixe la zone ecran	
	//cGame_World::Ecran = cPositive_Rectangle( cPoint2D(0,0) , cPoint2D(x,y));
	return true;
}

/*-----------------------------------------------------------------------------------------*/
bool cLevel_Loader::Read_Wall(ifstream& in ,  cGame_Manager * game)
{
	char c;	
	int x0,y0,x1,y1;
	
	if( !Get_Pair( x0, y0 , in))
		return false;

	// on lit la fleche entre les deux points
	in >> c;
	if( c!='-' )
		return false;
	in >> c;
	if( c!='>' )
		return false;

	if( !Get_Pair( x1, y1 , in))
		return false;	

	game->Game_World->My_BSP.Add_Divider( 	cHV_Wall(cPoint2D(x0,y0),cPoint2D(x1,y1)  )	);

	return true;
}


bool cLevel_Loader::Read_Interactive_Item(Interactive_Item_Type type,ifstream& in ,  cGame_Manager * game)
{
	int x,y;
	if( !Get_Pair( x, y , in))
		return false;

	game->Add_Bonus(type,x,y);	
	return true;
}

bool cLevel_Loader::Read_Ennemy(Ennemy_Type type,ifstream& in ,  cGame_Manager * game)
{
	int x,y;
	if( !Get_Pair( x, y , in))
		return false;

	game->Add_Ennemy(type,x,y);	
	return true;
}

/*-----------------------------------------------------------------------------------------*/
bool cLevel_Loader::Load(char *filename ,  cGame_Manager * game)
{
	ifstream in;
	int n_line=0;
	char c;
	char line[100];
	string str_line;
	READING_STATE state = READING_UNDEF;
	std::string err_message;

	in.open( filename );
	if( ! in.is_open() )
	{
		cout << "error while opening file : " <<  filename << "\n";
		err_message += "fichier ";
		err_message +=filename;
		err_message += " introuvable ";
		Display_Fatal_Error(hWnd , err_message.c_str() );
		PostQuitMessage(0);
		return false;
	}

	while( ! in.eof() )
	{
		while( in.peek() == '\n' )
		{			
			in.ignore(1);
			n_line++;
		}
		if( in.eof() )
			return true;

		c= in.peek();
		if( c == '<' )   // le caractere '<' en debut de ligne indique une balise, on change d'etat
		{
			in.getline(line,100);
			n_line++;
			str_line = string(line);
			if( str_line == "<AREA>")
				state = READING_AREA;
			else if( str_line == "<WALLS>")
				state = READING_WALL;
			else if( str_line == "<RED_TRAP>")
				state = READING_RED_TRAP;
			else if( str_line == "<BLUE_TRAP>")
				state = READING_BLUE_TRAP;
			else if( str_line == "<GREEN_TRAP>")
				state = READING_GREEN_TRAP;
			else if( str_line == "<RED_BALL>")
				state = READING_RED_BALL;
			else if( str_line == "<BLUE_BALL>")
				state = READING_BLUE_BALL;
			else if( str_line == "<GREEN_BALL>")
				state = READING_GREEN_BALL;
			else if( str_line == "<TIME>")
				state = READING_TIME;
			else if( str_line == "<PLAYER>")
				state = READING_PLAYER;
			else 
			{
				err_message.erase();
				err_message += "balise inconnue ";
				err_message += line;
				err_message += " dans le fichier ";
				err_message +=filename;
				err_message += "\n";
				Display_Fatal_Error(hWnd , err_message.c_str());				
				state=READING_UNDEF;
				PostQuitMessage(0);
				return false;
			}
		}
		
		switch( state)  // on appelle la fonction adequate selon l'etat de lecture
		{
		case READING_AREA:			
			if( !Read_Area(in,game))
			{
				Display_Error( n_line , filename);
				return false;
			}			
			break;
		case READING_WALL:			
			if( ! Read_Wall(in,game) )
			{
				Display_Error( n_line , filename);
				return false;
			}			
			break;
		case READING_RED_TRAP:
			if( ! Read_Interactive_Item(RED_TRAP,in,game) )
			{
				Display_Error( n_line , filename);
				return false;
			}			
			break;
		case READING_BLUE_TRAP:
			if( ! Read_Interactive_Item(BLUE_TRAP,in,game) )
			{
				Display_Error( n_line , filename);
				return false;
			}			
			break;
		case READING_GREEN_TRAP:
			if( ! Read_Interactive_Item(GREEN_TRAP,in,game) )
			{
				Display_Error( n_line , filename);
				return false;
			}			
			break;
		case READING_RED_BALL:
			if( ! Read_Ennemy(RED_BALL,in,game) )
			{
				Display_Error( n_line , filename);
				return false;
			}			
			break;
		case READING_BLUE_BALL:
			if( ! Read_Ennemy(BLUE_BALL,in,game) )
			{
				Display_Error( n_line , filename);
				return false;
			}			
			break;
		case READING_GREEN_BALL:
			if( ! Read_Ennemy(GREEN_BALL,in,game) )
			{
				Display_Error( n_line , filename);
				return false;
			}			
			break;
		case READING_TIME:
			if( ! Read_Time(in,game) )
			{
				Display_Error( n_line , filename);
				return false;
			}			
			break;
		case READING_PLAYER:
			if( ! Read_Player(in,game) )
			{
				Display_Error( n_line , filename);
				return false;
			}			
			break;
		default:
			cout << "case READING_UNDEF: \n";
			cout << "error while reading " << filename << "\n";
			err_message.erase();
			err_message += "balise inconnue dans le fichier ";
			err_message +=filename;
			err_message += "\n";
			Display_Fatal_Error(hWnd , err_message.c_str());
			PostQuitMessage(0);
			return false;			
		}
	}

	return true;
}


/*-----------------------------------------------------------------------------------------*/
void cLevel_Loader::Display_Error(int n_line , char * file)
{
	// modifier selon le type d'affichage d'erreur que l'on desire
	// sortie text/box message...
	cout << "ERROR while reading file : " << file << " at line " << n_line << "\n";
}