/***************************************************************************************************/
/***************************************************************************************************/
//! \file level.h
//! \brief Decrit la class 'cLevel_Loader' utilise pour charge un niveau ds un 'cGame_World'
//!    
//!	Auteur : Adrien Auclair
/***************************************************************************************************/
/***************************************************************************************************/
#ifndef LEVEL_H
#define LEVEL_H

/*------------------------------------  Prototypes  ---------------------------------------------*/
class cLevel_Loader;


/*------------------------------------   Includes   ---------------------------------------------*/
#include <fstream>

#include "game_BSP.h"
#include "game_manager.h"
#include "bonus.h"


/*-------------------------------------   Classes   ---------------------------------------------*/


/*************************************************************************************************/
//! class encapsulant l'algorithme pour charger un niveau et l'inserer dans un cGame_World
/*************************************************************************************************/
class cLevel_Loader
{
public:	
	/// charge le niveau du fichier donn dans le cGame_World pass en parametre
	/** retourne true si il n'y a pas eu d'erreur de lecture, false sinon */
	bool Load(char * filename , cGame_Manager * game);	

private:
	/// lit la taille du jeu a partir du stream 'in' et l'indique au 'world'
	/** retourne true si il n'y a pas eu d'erreur de lecture, false sinon */
	bool Read_Area(std::ifstream & in,  cGame_Manager * game);

	/// lit un mur a partir du stream 'in', et l'ajoute dans le 'world'
	/** retourne true si il n'y a pas eu d'erreur de lecture, false sinon */
	bool Read_Wall(std::ifstream & in,  cGame_Manager * game);

	/// lit un interactive_item a partir du stream 'in', et l'ajoute dans le 'world'
	/** retourne true si il n'y a pas eu d'erreur de lecture, false sinon */
	bool Read_Interactive_Item(Interactive_Item_Type type,ifstream& in ,  cGame_Manager * game);

	/// lit un ennemy a partir du stream 'in', et l'ajoute dans le 'world'
	/** retourne true si il n'y a pas eu d'erreur de lecture, false sinon */
	bool Read_Ennemy(Ennemy_Type type,ifstream& in ,  cGame_Manager * game);

	/// lit le nb de secondes alloues pour ce niveau
	/** retourne true si il n'y a pas eu d'erreur de lecture, false sinon */
	bool Read_Time(ifstream& in,  cGame_Manager * game);

	/// lit la position du joueur
	/** retourne true si il n'y a pas eu d'erreur de lecture, false sinon */
	bool Read_Player(ifstream& in,  cGame_Manager * game);


	/// affiche un message comme quoi une erreur s'est produite a la ligne 'n_line' du fichier 'file'
	void Display_Error(int n_line , char * file);
};


#endif