/***************************************************************************************************/
/***************************************************************************************************/
//! \file main.cpp
//! \brief Programme du jeu LIBELL.
//!    
//!	Auteur : Adrien Auclair
//!
//! Ce fichier contient la fonction WinMain \n
//! On trouve aussi les fonctions d'initialisation de DirectX ( draw et ddinput) \n
//! WndProc est la fonction de traitement des messages windows \n
//! elle est utilise pour traiter certains caracteres claviers \n
/***************************************************************************************************/
/***************************************************************************************************/

#pragma warning(disable: 4786)

/*----------------------------------------------------------------*/
#include <windows.h>
#include <dxerr9.h>
#include <string>
#include <dinput.h>

#include "main.h"
#include "game_manager.h"
#include "errrors_display.h"


#define BITMAP_FILE     "../graphs/graphs.bmp"
#define MESSAGES_FILE   "../graphs/messages.bmp"
#define FOND_FILE       "../graphs/fond.bmp"


//la taille de l'image bitmap a charger qui contient les sprites
#define TX_BITMAP       85
#define TY_BITMAP       64

//la taille de l'image bitmap a charger qui contient les messages
#define TX_MESSAGES_BMP 276
#define TY_MESSAGES_BMP 190




/*----------------------------------------  globals  -----------------------------------------------*/

CDisplay *g_pDisplay         = NULL;     
CSurface *g_pText            = NULL;     
CSurface *g_pLeft_Time_Text  = NULL;     
CSurface *g_pScore_Text      = NULL;     
CSurface *g_pSprites         = NULL;     
CSurface *g_pMessages        = NULL;     
CSurface *g_pFond            = NULL;     

LPDIRECTINPUT8        g_lpDI;                  
LPDIRECTINPUTDEVICE8  g_pKeyboard ;      
cGame_Manager         Game_Manager;              
HWND                  hWnd;                               
Debugger Debug_File ("output.txt");      ///< fichier vers lequel on envoie des infos de debug




/*----------------------------------------  prototypes  --------------------------------------------*/
static bool InitDD(HWND);
static void CleanUp();
static void GameLoop();
static void WINAPI DI_Term();


/*----------------------------------------   locals     --------------------------------------------*/
static bool g_bActive               = false;    ///< indique que je jeu est actif








/*--------------------------------------------------------------------------------------------------*/
/// Termine proprement les composants Direct Input
/*--------------------------------------------------------------------------------------------------*/
void WINAPI DI_Term() 
{ 		
    if (g_lpDI) 
    { 
        if (g_pKeyboard) 
        { 
        // Always unacquire device before calling Release(). 
            g_pKeyboard->Unacquire(); 
            g_pKeyboard->Release();
            g_pKeyboard = NULL; 
        } 
        g_lpDI->Release();
        g_lpDI = NULL; 
    } 
} 

/*--------------------------------------------------------------------------------------------------*/
/// Fonctions de callback de traitement des messages de la fenetre Windows
/*--------------------------------------------------------------------------------------------------*/
LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg,
                         WPARAM wParam, LPARAM lParam)
{
  switch(uMsg)
  {
  case WM_SETCURSOR:
 	SetCursor(NULL);	// unable mouse cursor
	return true;
  case WM_CREATE:	
    if( !InitDD(hWnd) )	
		PostQuitMessage(0);	
	else
		g_bActive=true;
    break;
  case WM_CLOSE:
    g_bActive=false;
    CleanUp();
	DI_Term();
    DestroyWindow(hWnd);
    break;
  case WM_DESTROY:
    PostQuitMessage(0);
    break;
  case WM_MOVE:
    g_pDisplay->UpdateBounds();
    break;
  case WM_SIZE:
    g_pDisplay->UpdateBounds();
    break;
  case WM_KEYDOWN:
    switch(wParam)
    {        
	case VK_ESCAPE:           
		PostQuitMessage(0);    
		break;
	case VK_F1 :
		Game_Manager.bDisplay_Square_Areas = ! Game_Manager.bDisplay_Square_Areas;
		break;
	case VK_F2:
		Game_Manager.bDisplay_Ennemies_Path =! Game_Manager.bDisplay_Ennemies_Path;
		break;
	case VK_F3:
		Game_Manager.bDisplay_Path_Graph =! Game_Manager.bDisplay_Path_Graph;
		break;
	case VK_SPACE:
		Game_Manager.Try_Start();		
		break;
	case VK_F5:
		Game_Manager.bGame_Paused = ! Game_Manager.bGame_Paused;				
		break;
	case VK_F11:
		if( Game_Manager.Game_State == RUNNING )
			Game_Manager.Game_State = LEVEL_FINISHED;
		break;			
	case VK_F12 :
		Game_Manager.bInvincible = !Game_Manager.bInvincible;
		break;
	}
	break;
  default:
    return DefWindowProc(hWnd,uMsg,wParam,lParam);
    break;
  }
  return 0;
}




	
/*--------------------------------------------------------------------------------------------------*/
/// ititialise les composants Direct Input
/*--------------------------------------------------------------------------------------------------*/
BOOL WINAPI DI_Init(HWND hWnd , HINSTANCE  g_hinst)
{ 
    HRESULT hr; 

    // Create the DirectInput object. 
    hr = DirectInput8Create(g_hinst, DIRECTINPUT_VERSION, 
                            IID_IDirectInput8, (void**)&g_lpDI, NULL); 

    if FAILED(hr) 
	{
		Display_Fatal_Error(hWnd , "BOOL WINAPI DI_Init(HWND hWnd , HINSTANCE  g_hinst)" , hr);		
		return FALSE;
	}

    // Retrieve a pointer to an IDirectInputDevice8 interface 
    hr = g_lpDI->CreateDevice(GUID_SysKeyboard, &g_pKeyboard, NULL); 
    if FAILED(hr) 
    {
		Display_Fatal_Error(hWnd , "BOOL WINAPI DI_Init(HWND hWnd , HINSTANCE  g_hinst)" , hr);
        DI_Term(); 		
        return FALSE; 
    }    

    // Set the data format using the predefined keyboard data     
    hr = g_pKeyboard->SetDataFormat(&c_dfDIKeyboard); 
    if FAILED(hr) 
    { 
		Display_Fatal_Error(hWnd , "BOOL WINAPI DI_Init(HWND hWnd , HINSTANCE  g_hinst)" , hr);
        DI_Term(); 
        return FALSE; 
    } 

    // Set the cooperative level 
    hr = g_pKeyboard->SetCooperativeLevel(hWnd, 
                             DISCL_FOREGROUND | DISCL_NONEXCLUSIVE); 
    if FAILED(hr) 
    { 
		Display_Fatal_Error(hWnd , "BOOL WINAPI DI_Init(HWND hWnd , HINSTANCE  g_hinst)" , hr);
        DI_Term(); 
        return FALSE; 
    }

    // Get access to the input device. 
    hr = g_pKeyboard->Acquire(); 
    if FAILED(hr) 
    { 
		Display_Fatal_Error(hWnd , "BOOL WINAPI DI_Init(HWND hWnd , HINSTANCE  g_hinst)" , hr);
        DI_Term(); 
        return FALSE; 
    } 

    return TRUE; 
}





/*--------------------------------------------------------------------------------------------------*/
/// ititialise les composants Direct Draw
/*--------------------------------------------------------------------------------------------------*/
bool InitDD(HWND hWnd)
{
  HRESULT hr;
  LPDIRECTDRAWPALETTE pDDPal = NULL;
  std::string error_message;

  //dd init code
  g_pDisplay = new CDisplay();  
  
  if(FAILED( hr = g_pDisplay->CreateFullScreenDisplay(hWnd, TX_SCREEN,TY_SCREEN,BIT_DEPTH) ) )
  {
	  Display_Fatal_Error(hWnd , "CreateFullScreenDisplay(hWnd, TX_SCREEN,TY_SCREEN,BIT_DEPTH)" , hr);	  	      
	  return false;
  }

  // Create and set the palette when in palettized color
  if( FAILED( hr = g_pDisplay->CreatePaletteFromBitmap( &pDDPal, BITMAP_FILE ) ) )
  {
	  error_message.erase();
	  error_message += "g_pDisplay->CreatePaletteFromBitmap( &pDDPal, ";
	  error_message += BITMAP_FILE ;
	  error_message += "\n";	        
	  Display_Fatal_Error(hWnd , error_message.c_str() , hr);	  
	  return false;
  }

   if( FAILED( hr = g_pDisplay->SetPalette( pDDPal ) ))
   {
	  error_message.erase();
	  error_message += "g_pDisplay->SetPalette( pDDPal )\n";
	  Display_Fatal_Error(hWnd , error_message.c_str() , hr);	  
	  return false;
   }

    SAFE_RELEASE( pDDPal );

	// Create a surface, and draw a bitmap resource on it.  
    if( FAILED( hr = g_pDisplay->CreateSurfaceFromBitmap( &g_pMessages, MESSAGES_FILE , TX_MESSAGES_BMP , TY_MESSAGES_BMP ) ) )
	{
	  error_message.erase();
	  error_message += "g_pDisplay->CreateSurfaceFromBitmap( &g_pSprites, ";  
	  error_message += MESSAGES_FILE ;
	  error_message += ",TX_MESSAGES_BMP , TX_MESSAGES_BMP  ) ) )\n";
	  Display_Fatal_Error(hWnd , error_message.c_str() , hr);
	  return false;
	}

    // Create a surface, and draw a bitmap resource on it.  
    if( FAILED( hr = g_pDisplay->CreateSurfaceFromBitmap( &g_pSprites, BITMAP_FILE , TX_BITMAP , TY_BITMAP ) ) )
	{
	  error_message.erase();
	  error_message += "g_pDisplay->CreateSurfaceFromBitmap( &g_pSprites, ";  
	  error_message += BITMAP_FILE ;
	  error_message += ", TX_BITMAP , TY_BITMAP ) ) )\n";
	  Display_Fatal_Error(hWnd , error_message.c_str() , hr);	  
      return false;
	}

	if( FAILED(	hr = g_pSprites->SetColorKey( 0 ) ))
	{
		error_message.erase();
		error_message += "g_pSprites->SetColorKey( 0 )\n";  		
		Display_Fatal_Error(hWnd , error_message.c_str() , hr);		
		return false;
	}

	// Create a surface, and draw a bitmap resource on it.  
    if( FAILED( hr = g_pDisplay->CreateSurfaceFromBitmap( &g_pFond, FOND_FILE , TX_SCREEN , TY_SCREEN ) ) )
	{
	  error_message.erase();
	  error_message += "g_pDisplay->CreateSurfaceFromBitmap( &g_pFond, ";  
	  error_message += FOND_FILE ;
	  error_message += ", TX_SCREEN , TY_SCREEN ) ) )\n";
	  Display_Fatal_Error(hWnd , error_message.c_str() , hr);	  
      return false;
	}

	// Create a surface 
    if( FAILED( hr = g_pDisplay->CreateSurface( &g_pLeft_Time_Text, 50 , 20) ) )
	{
	  error_message.erase();
	  error_message += "g_pDisplay->CreateSurface( &g_pLeft_Time_Text, 50 , 20) )";  	  
	  Display_Fatal_Error(hWnd , error_message.c_str() , hr);	  
      return false;
	}

	// Create a surface 
    if( FAILED( hr = g_pDisplay->CreateSurface( &g_pScore_Text, 50 , 20) ) )
	{
	  error_message.erase();
	  error_message += "g_pDisplay->CreateSurface( &g_pScore_Text, 50 , 20) )";  	  
	  Display_Fatal_Error(hWnd , error_message.c_str() , hr);	  
      return false;
	}

  return true;  
	  
}



/*--------------------------------------------------------------------------------------------------*/
/// Libere proprement les composants Direct Draw
/*--------------------------------------------------------------------------------------------------*/
void CleanUp()
{
  SAFE_DELETE(g_pDisplay);
  SAFE_DELETE(g_pText);
  SAFE_DELETE(g_pLeft_Time_Text);
  SAFE_DELETE(g_pScore_Text);
  SAFE_DELETE(g_pSprites);
  SAFE_DELETE(g_pMessages);
  SAFE_DELETE(g_pFond);
}




/*--------------------------------------------------------------------------------------------------*/
/// WinMain
/*--------------------------------------------------------------------------------------------------*/
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                   LPSTR lpCmdLine, int iShowCmd)
{
  WNDCLASSEX wc;  
  MSG lpMsg;
  
  wc.cbClsExtra=0;
  wc.cbSize=sizeof(WNDCLASSEX);
  wc.cbWndExtra=0;
  wc.hbrBackground=(HBRUSH)GetStockObject(BLACK_BRUSH);
  wc.hCursor=LoadCursor(NULL,IDC_ARROW);
  wc.hIcon=LoadIcon(NULL,IDI_APPLICATION);
  wc.hIconSm=LoadIcon(NULL,IDI_APPLICATION);
  wc.hInstance=hInstance;
  wc.lpfnWndProc=WndProc;
  wc.lpszClassName="wc";
  wc.lpszMenuName=0;
  wc.style=CS_HREDRAW|CS_VREDRAW;
  if(!RegisterClassEx(&wc))
  {
    MessageBox(NULL,"Couldn't Register Window Class",
      "Window Class Registration Failure",MB_OK|MB_ICONERROR);
    return 0;
  }
  

  hWnd = CreateWindowEx(NULL,"wc","Libell", WS_POPUPWINDOW,CW_USEDEFAULT,CW_USEDEFAULT,TX_SCREEN,TY_SCREEN,0,0,
	                    hInstance,0);

  if(hWnd == NULL)
  {
    MessageBox(NULL,"Failed to Create Window","Window Creation Failure",
      MB_OK|MB_ICONERROR);
    return 0;
  }  

  if( ! DI_Init(hWnd , hInstance) )
  {
	  MessageBox(NULL,"Failed to Initialize Direct Input","Window Creation Failure",
				MB_OK|MB_ICONERROR);
	  return 0;
  }
  
  ShowWindow(hWnd,SW_SHOW);
  UpdateWindow(hWnd);
  
  while(lpMsg.message != WM_QUIT)
  {
    if(PeekMessage(&lpMsg,0,0,0,PM_REMOVE))
    {
      TranslateMessage(&lpMsg);
      DispatchMessage(&lpMsg);
    }
    else if(g_bActive)
    {
      Game_Manager.Main_Loop(); 	  //la boucle principale du jeu
    }
  }
  return lpMsg.wParam;
}