/***************************************************************************************************/
/***************************************************************************************************/
//! \file main.h
//! \brief Dfini les variables globales du jeu
//!    
//!	Auteur : Adrien Auclair
//!
//! 
/***************************************************************************************************/
/***************************************************************************************************/
#ifndef MAIN_H
#define MAIN_H

#define WIN32_LEAN_AND_MEAN

#include <windows.h>
#include <dinput.h>

#include "dxutil.h"
#include "ddutil.h"
#include "mydebug.h"

extern CDisplay * g_pDisplay;              ///< l'objet DDRAW pour l'affichage
extern CSurface * g_pText ;                ///< surface de text
extern CSurface * g_pSprites;              ///< surface bitmap des sprites (avec cl de transparence);
extern CSurface * g_pMessages;             ///< surface des messages
extern CSurface * g_pFond;                 ///< surface contenant l'image de fond d'ecran
extern CSurface * g_pLeft_Time_Text;       ///< surface du text pour afficher le temps restant
extern CSurface * g_pScore_Text;           ///< surface du text pour afficher le score

extern LPDIRECTINPUTDEVICE8  g_pKeyboard ; ///< keyboard device

extern Debugger Debug_File;                ///< debugger ( fichier texte) vers lequel on envoie des infos
extern Debugger Fatal_Error;               ///< fichier texte vers lequel on envoie des infos sur les erreurs fatales

extern HWND hWnd;                          ///< instance de la fenetre

#define TX_SCREEN 800
#define TY_SCREEN 600
#define BIT_DEPTH 8




#endif


/*********************************************************************************************/
/*********************************************************************************************/
/********************************** DOXYGEN MAINPAGE *****************************************/
/*********************************************************************************************/
/*********************************************************************************************/

/*! \mainpage
\section intro Introduction
Le but de ce document est de rapidement prsenter l'architecture du programme. La documentation utilisateur 
est dans un autre fichier. J'ai utilis Doxygen pour la documentation car il permet de parcourir rapidement
le code grace  ses nombreux liens hypertextes. Visual Studio permet une approche semblable mais je prefere
une documentation indpendante, plus agrable et rapide  parcourir.

\section archi Architecture globale
Ce programme est articul autour des modules principaux suivants :

\subsection class_bsp_game La classe cGame_World
Cette classe contient la description du niveau ( ensemble de murs dans une zone). Elle se base sur
un arbre binaire de recherche construit  partir d'un ensemble de murs verticaux 
ou horizontaux. Pour cela on a d'abord dvelopp une classe cBSP_Tree gnrique qui divise tous type 
d'ensemble de donnes que l'on peut diviser deux. Cette classe est template dans notre cas pour diviser un
espace 2D ( donc des cPoint2D), avec comme diviseur des cHV_Wall ( murs horizontaux ou verticaux ). \n
On fois l'arbre construit, on le dcore en mettant  la place de chaque feuille un objet cSquare_Area qui
reprsente une piece rectangulaire. \n

Ensuite, chaque cSquare_Area construit la liste des murs qui la bordent effectivement. Elle detecte aussi les
passages entre elle et les pieces voisines. Ces passages sont reprsents par des objets cPortal. Un cPortal 
est donc qualifi par son segment porteur cPortal::Segment. Toutefois, on doit lui assigner une position exacte,
on choisira le point milieu de ce segment. Cette solution n'est pas parfaite, notament lorsqu'on a un reseau
dense de portails, mais elle convient pour ce petit jeu.\n

Puis on contruit la connectivit entre les objets cPortal. C'est  dire que chaque cPortal va avoir dans sa liste
membre cPortal::Voisinage la liste des cPortal qu'on peut rejoindre en ligne droite sans rencontrer d'obstacle. 
On prends pour cela tous les cPortal des deux pieces qui bordent chaque cPortal. \n
Il reste toutefois un problme lorsque deux objets cPortal sont colineaires dans une meme piece cSquare_Area. Dans ce cas, 
la ligne droite entre ces deux portails va intersecter un mur. On va donc cre un objet cParalell_Portal entre ces
deux portails. Il sera situ  gale distance des deux portails, mais dcal un peu dans la pice pour que le chemin
ne touche pas le mur. Ce portail en voisins que les deux cPortal paralelles qui l'ont gnrs. \n

\subsection class_a_star_search La classe cA_Star_Search
la classe cA_Star_Search permet d'appliquer un algorithme A* sur un graph de noeuds c_A_Star_Node. La cA_Star_Node est 
purement virtuelle. L'algorithme permet donc tout type de recherche bas sur l'algo A*. Dans notre cas, les noeuds du 
graph seront des objets cPortal et cA_Star_Point. On a dfini les cPortal dans la section precedente \ref class_bsp_game. 
Les cA_Star_Point sont des noeuds qu'on insere temporairement dans le graph et qui vont dfinier le point de dpart
et le point d'arrive. Lorsque l'algorithme de recherche est termin, on enleve ces noeuds du graphe.\n
La classe cGame_World prcdement dfinie contient un objet cA_Star_Search. \n

\subsection class_game_manager La class cGame_Manager
La classe cGame_Manager est le moteur haut-niveau du jeu. Le fonctionnement est celui d'une FSM dont l'tat est dfini par la
donne membre cGame_Manager::Game_State. Les valeurs possibles sont celle de enum Game_State_Type: \n
-> INITIALIZING : etat du jeu initial, mis par l'appel au constructeur \n
-> RUNNING : un niveau est en train d'etre jou\n
-> READY_TO_START : un niveau ( ou une nouvelle vie) est pret a etre jou, on attends 
                    que le joueur le lance ( press space) \n
-> LEVEL_FINISHED : le joueur vient de terminer un niveau \n
-> VICTORY : le joueur vient de terminer le dernier niveau \n

Cette classe contient les elements suivants :
\subsubsection obj1 un objet cGame_World
On le charge avec un niveau grace  cGame_Manager::Load_Level(int i). Ce chargement est effectu grace  l'objet 
membre cLevel_Loader. Cette classe encapsule l'algorithme pour lire des niveaux. \n
\subsubsection obj2 un joueur
Il s'agit d'un objet cPlayer qui hrite de cUnit ( classe des units mobiles) : cGame_manager::Player.
\subsubsection obj3 une liste des ennemis
Il s'agit d'une liste d'objet cEnnemy : cGame_Manager::Ennemies_List.
\subsubsection obj4 une liste des objets interactifs
Il s'agit d'une liste d'objet cInteractive_Item : cGame_Manager::Bonus_List.
\subsubsection obj5 un objet pour afficher des infos de debug  l'ecran
Pour vrifier la cohrence du niveau ( les cSquare_Area, leur murs et leurs portals, le reseau de cPortal, les chemins),
on ajoute  la classe un objet de type cGame_World_Debug_Displayer. Cet donne memebre cGame_Manager::World_Debug_Displayer
permet d'envoyer  l'ecran les informations voulues. On trouve dans la documentation utiisateur les touches permettant
d'afficher les infos souhaites.



\section exe Executables
Le jeu se trouve dans le repertoire bin. Libell.exe est la version Release. Libell_debug.exe est la version debug.
On trouve aussi dans bin l'xcutable bsp_test qui permet de tester le bsp construit. Il teste
les chemins construit dans un niveau, les intersections entre des segments et les murs, et enfin les intersections
entre des boules et les murs. \n


\section rmq Remarques
\subsection test Tests
Les seuls tests automatiquent sont ceux de bsp_test.exe. Ces tests sont limits. Il aurait bien sure fallut les etendre. 
Mais cela n'etait pas possible dans les temps impartis. Un test qui pourrait etre efficace serait de generer des
chemins aleatoires et de verifier ensuite que ces chemins n'intersectent pas les murs.

\subsection directx DirectX
J'utilise les classes des fichiers ddutil;h et dxutil.h. J'ai prfr blitter les boules plutot que de les dessiner, 
cela m'a parut plus rapide  coder et il fallait de toute facon du code pour blitter des surfaces pour les autres objets.
Au final, le code pour l'affichage n'est pas rs lgant, mais pour terminer dans les dlais, j'ai prfr affiner les
parties sur le BSP, le A* et le game_manager que le code DirectX. C'est pourquoi le jeu ne se fenetre pas, les surfaces
ne se restaurent pas, il n'y a qu'un mode d'affichage.... \n

\subsection opti Optimisation
Mon ide de l'optimisation est de la commencer uniquement lorsque le programme est parfaitement stable, robuste... Le code
prsent n'est donc pas optimis.

\subsection play Gameplay
Le jeu se rvle trs plaisant  jouer meme s'il faudrait dsigner mieux les niveaux. Notament, les arretes vives
sont parfois trs genantes pour diriger la balle. Il faurdait construire des niveaux qui les evitent. Les trois missions
fournies donnent un bon apercu de la jouabilit.


\section bugs Bugs
Les chemins des boules sont parfois  une distance infrieure  leur rayon d'un mur. Il arrive donc parfois qu'une boule
empiete sur un mur en le longeant. Le problme n'est pas critique sauf dans le cas ou le joueur se croyait protg 
de l'autre cot du mur, dans ce cas il peut se faire tuer.

Le BSP n'accepte pas les murs colinaires. Si l'on veut en mettre dans le jeu, il faut bien veiller  ce que un autre
mur aie precedement divis la zone dans laquelle on va ajouter les murs paralleles.

Les boules vont plus vite lorsqu'elle se dplacent en diagonale, il faudrait pour eviter ca revoir la 
classe cIncremental_Bresenham.

Les cParalell_Portal sont parfois mal positionn. Il arrive parfois que deux portails de ce type ne soit pas spars
par un mur. Dans ce cas, on ne devrait pas crer un cParalell_Portal mais seulement relier les deux cPortal.

J'ai choisi d'utilis le mode direct pour lire le clavier car cela est plus adapt pour lire le mouvement du joueur. On
a juste besoin de savoir si une touche de direction es tenfonce ou non. Mais pour poser des objets, cela n'est pas
adapt car si le joueur appuie trs rapidement sur une touche, mais au mauvais moment, le programme ne lira
pas cette touche. Il faudrait utiliser le mode buffered de Direct Input pour ces touches.



\section todo TODO
Paramtrer le jeu pour s'afficher dans plusieurs rsolution. Les samples du SDK donnent des sources pour lister les
les possibilits d'affichages, on pourrait s'en servir.

La gestion des erreurs est trs discutable. Il faudrait utiliser des interuptions pour rendre le code plus clair.

Utiliser le mot cl const. Ce n'est pas dans mes habitudes, mais c'est fournit par le C++, autant s'en servir...


J'ai tendance  rendre public la plupart des mthodes et des donnes membres. Il faudrait crire plus de fonctions
aux donnes membres et rendre ces derniers privs. Mais je trouve cela assez lourd de dclarer beaucoup de
fonctions d'acces... ( mauvaise habitude !?) Une autre solution est de dclarer des classes amies.

Il faudrait aussi rgler plus finement le gameplay des niveaux. C'est  dire reforcer la notion de pices dans
le design des niveaux, qui forcent le joueur  utiliser des teleporteurs pour ressortir de la pice, et aussi
 poser des bombes  l'entre de la pice. 

Lorsque l'on est touch par un ennemi, on redmarre la vie suivante  la meme position et l'ennemi est renvoy
 sa position initiale. Or cette position initiale peut etre trs proche voir meme  l'endroit exacte du joueur.
Dans ce cas, impossible de survivre.... Il faudrait alors dplacer un peu le joueur ou l'ennemi.

La classe de chargement des fichiers cLevel_Loader dtecte les balises inconnues ou une syntaxe mauvaise. Mais
en raison des dlais, le code n'est pas robuste  tout type d'erreur de syntaxe dans le fichier de niveau.
Il faudrait le rendre plus robuste. De plus, on ne vrifie  aucun moment la cohrence de la carte lue. Il faudrait
par exemple tester que les positions initiales des boules ne sont pas sur des murs.

Le cGame_World construit ne test  aucun moment sa cohrence. Il serait facile d'crire quelques fonctions d'autotest
pour cet arbre. Par exemple, on peut aisement tester que la somme des aires des cSquare_Area est gale  la somme totale
de l'aire de jeu. On peut aussi tester qu'une fois dcors, tous les cPortal sont situs entre deux pieces,

Le BSP empile les murs dans l'ordre auquel on les lui donne. Donc les fichiers des niveaux doivent etre construit
tels qu'ils essaient d'quilibrer l'arbre. Il faudrait qu'il soit capable de s'auto quilibrer.


*/