/*------------------------------------   Includes   ---------------------------------------------*/
#include <assert.h>
#include <iostream>

#include "bsp.h"
#include "game_bsp.h"
#include "bsp_loader.h"

using namespace std;

/*------------------------------------  Prototypes  ---------------------------------------------*/
bool Test_Path_Finder_1();
bool Test_Path_Finder_2();
bool Test_Path_Finder_3();
bool Test_Path_Finder_4();
bool Test_Path_Finder_5();

void Test_Intersections_1();
void Test_Intersections_2();


Debugger Debug_File ("output.txt");


/*------------------------------------------------------------------------*/
void Test_Intersections()
{
	Test_Intersections_1();
	Test_Intersections_2();
}


/*------------------------------------------------------------------------*/
void Test_Intersections_1()
{	
	cPoint2D p1;
	cPoint2D p2;
	cPoint2D p_inter;
	bool result;
	
	
	cGame_World * game = new cGame_World();
	cBSP_Loader loader;
	loader.Load("../test_levels/level_test_intersections.txt" , game);
	game->Decorate();
	


	p1.Set(0,0);
	p2.Set(100,0);
	cout << "TEST INTERSECTIONS 1 : ";
	result = game->Do_Segment_Intersect_Walls( cSeg(p1,p2) , p_inter );
	if( result )
		cout << "ERROR \n";
	else
		cout << "OK\n";

	p1.Set(0,0);
	p2.Set(0,100);
	cout << "TEST INTERSECTIONS 2 : ";
	result = game->Do_Segment_Intersect_Walls( cSeg(p1,p2) , p_inter );
	if( result )
		cout << "ERROR \n";
	else
		cout << "OK\n";

	p1.Set(0,0);
	p2.Set(90,10);
	cout << "TEST INTERSECTIONS 3 : ";
	result = game->Do_Segment_Intersect_Walls( cSeg(p1,p2) , p_inter );
	if( result )
		cout << "ERROR \n";
	else
		cout << "OK\n";

	p1.Set(0,0);
	p2.Set(70,15);
	cout << "TEST INTERSECTIONS 4 : ";
	result = game->Do_Segment_Intersect_Walls( cSeg(p1,p2) , p_inter );
	if( result && p_inter.Get_X() == 60  && p_inter.Get_Y() == 13 )
		cout << "OK\n";
	else
		cout << "ERROR \n";

	p1.Set(50,50);
	p2.Set(70,50);
	cout << "TEST INTERSECTIONS 5 : ";
	result = game->Do_Segment_Intersect_Walls( cSeg(p1,p2) , p_inter );
	if( result && p_inter.Get_X() == 60  && p_inter.Get_Y() == 50 )
		cout << "OK\n";
	else
		cout << "ERROR \n";

	p1.Set(50,50);
	p2.Set(50,70);
	cout << "TEST INTERSECTIONS 6 : ";
	result = game->Do_Segment_Intersect_Walls( cSeg(p1,p2) , p_inter );
	if( result && p_inter.Get_X() == 50  && p_inter.Get_Y() == 60 )
		cout << "OK\n";
	else
		cout << "ERROR \n";

	p1.Set(50,50);
	p2.Set(70,70);
	cout << "TEST INTERSECTIONS 7 : ";
	result = game->Do_Segment_Intersect_Walls( cSeg(p1,p2) , p_inter );	
	if( result && p_inter.Get_X() == 60  && p_inter.Get_Y() == 60 )
		cout << "OK\n";
	else
		cout << "ERROR \n";

	p1.Set(10,10);
	p2.Set(70,70);
	cout << "TEST INTERSECTIONS 8 : ";
	result = game->Do_Segment_Intersect_Walls( cSeg(p1,p2) , p_inter , 162);		
	if( ! result )
		cout << "OK\n";
	else
		cout << "ERROR \n";
	
	cout << "TEST INTERSECTIONS 9 : ";
	result = game->Do_Segment_Intersect_Walls( cSeg(p1,p2) , p_inter , 163);	
	if( result && p_inter.Get_X() == 20  && p_inter.Get_Y() == 20 )
		cout << "OK\n";
	else
		cout << "ERROR \n";		
		
}


/*------------------------------------------------------------------------*/
void Test_Intersections_2()
{	
	cPoint2D p1;
	cPoint2D p2;
	cPoint2D p_inter;
	bool result;
	
	
	cGame_World * game = new cGame_World();
	cBSP_Loader loader;
	loader.Load("../test_levels/level_test_intersections_2.txt" , game);
	game->Decorate();

	cout << "TEST INTERSECTIONS WITH BALLS 1 : ";
	result= game->Do_Ball_Intersect_Walls(20,20, 10);
	if( result )
		cout << "ERROR \n";
	else
		cout << "OK\n";

	cout << "TEST INTERSECTIONS WITH BALLS 2 : ";
	result= game->Do_Ball_Intersect_Walls(20,20, 10);
	if( result )
		cout << "ERROR \n";
	else
		cout << "OK\n";

	cout << "TEST INTERSECTIONS WITH BALLS 3 : ";
	result= game->Do_Ball_Intersect_Walls(350,300, 10);
	if( result )
		cout << "OK \n";
	else
		cout << "ERROR\n";

	cout << "TEST INTERSECTIONS WITH BALLS 4 : ";
	result= game->Do_Ball_Intersect_Walls(400,300, 10);
	if( result )
		cout << "OK \n";
	else
		cout << "ERROR\n";

	cout << "TEST INTERSECTIONS WITH BALLS 5 : ";
	result= game->Do_Ball_Intersect_Walls(450,300, 10);
	if( result )
		cout << "OK \n";
	else
		cout << "ERROR\n";

	cout << "TEST INTERSECTIONS WITH BALLS 6 : ";
	result= game->Do_Ball_Intersect_Walls(500,300, 10);
	if( result )
		cout << "OK \n";
	else
		cout << "ERROR\n";

	cout << "TEST INTERSECTIONS WITH BALLS 7 : ";
	result= game->Do_Ball_Intersect_Walls(550,300, 10);
	if( result )
		cout << "OK \n";
	else
		cout << "ERROR\n";

	cout << "TEST INTERSECTIONS WITH BALLS 8 : ";
	result= game->Do_Ball_Intersect_Walls(600,300, 10);
	if( result )
		cout << "OK \n";
	else
		cout << "ERROR\n";
	
}



/*------------------------------------------------------------------------*/
void Test_Path_Finder()
{		
	bool result;
	
	cout << "TEST PATH_FINDER_1 : ";
	result = Test_Path_Finder_1();	
	if( result)
		 cout << "OK\n";
	else
		cout << "ERROR \n";
	
	cout << "TEST PATH_FINDER_2 : ";
	result = Test_Path_Finder_2();	
	if( result)
		 cout << "OK\n";
	else
		cout << "ERROR \n";

	cout << "TEST PATH_FINDER_3 : ";
	result = Test_Path_Finder_3();	
	if( result)
		 cout << "OK\n";
	else
		cout << "ERROR \n";

	cout << "TEST PATH_FINDER_4 : ";
	result = Test_Path_Finder_4();	
	if( result)
		 cout << "OK\n";
	else
		cout << "ERROR \n";

	cout << "TEST PATH_FINDER_5 : ";
	result = Test_Path_Finder_5();	
	if( result)
		 cout << "OK\n";
	else
		cout << "ERROR \n";
}

/*------------------------------------------------------------------------*/
// verifie que le chemin calcul soit bien le chemin attendu
// interet : le chemin est assez long, mais la carte est simple
bool Test_Path_Finder_1()
{
	cPoint2D path[10];

	cGame_World * game = new cGame_World();
	cBSP_Loader loader;
	loader.Load("../test_levels/level_test_pathfinder_1.txt" , game);
	game->Decorate();

	
	std::vector<cPoint2D>  Path;
	std::vector<cPoint2D>::iterator  Path_itr;
	game->Find_Path( cPoint2D(5,5) , cPoint2D(95,95) , Path);
	
	path[0].X = 10;
	path[0].Y = 95;

	path[1].X = 20;
	path[1].Y = 5;

	path[2].X = 30;
	path[2].Y = 95;

	path[3].X = 40;
	path[3].Y = 5;

	path[4].X = 50;
	path[4].Y = 95;

	path[5].X = 60;
	path[5].Y = 5;

	path[6].X = 70;
	path[6].Y = 95;

	path[7].X = 80;
	path[7].Y = 5;

	path[8].X = 90;
	path[8].Y = 95;

	path[9].X = 95;
	path[9].Y = 95;

	if( Path.size() != 10 ) return false;
	
	int n=0;
	cPoint2D p;
	for( Path_itr=Path.begin() ; Path_itr != Path.end() ; Path_itr++)
	{
		p = (*Path_itr) ;
		if( !(  p.X == path[n].X &&  p.Y == path[n].Y)) return false;
		n++;
	}	

	return true;
}

/*------------------------------------------------------------------------*/
// calcul un chemin qui ne peut pas etre trouv
// verifie que l'on a bien effectivement pas trouv de chemin
// interet : verifier le cas ou l'on ne doit pas trouver de chemin
bool Test_Path_Finder_2()
{	
	cGame_World * game = new cGame_World();
	cBSP_Loader loader;
	loader.Load("../test_levels/level_test_pathfinder_2.txt" , game);
	game->Decorate();
	
	
	
	bool result;

	std::vector<cPoint2D>  Path;	
	result = game->Find_Path( cPoint2D(5,5) , cPoint2D(95,95) , Path); 	
	
	return !result;
}


// calcul un chemin et verifie qu'il est correcte
// calcul un chemin qui est impossible, verifie qu'on ne trouve rien
// calcul un autre chemin et verifie qu'il est correcte
// interet : reprends les deux tests precedents mais reprends le meme World a chaque fois, cela
//           permet de verifier que l'arbre n'est pas altr par les recherches
bool Test_Path_Finder_3()
{
	cPoint2D path[10];	
	bool chemin_trouve;
	int n=0;
	cPoint2D p;
	cGame_World * game = new cGame_World();
	cBSP_Loader loader;
	loader.Load("../test_levels/level_test_pathfinder_3.txt" , game);
	game->Decorate();
	

	std::vector<cPoint2D>  Path;
	std::vector<cPoint2D>::iterator  Path_itr;
	

	// -----------         test 1 : premier chemin a verifier ! ----------------------
	// path contient le chemin correct
	path[0].X = 5;
	path[0].Y = 20;

	path[1].X = 5;
	path[1].Y = 60;

	path[2].X = 20;
	path[2].Y = 85;

	path[3].X = 58;
	path[3].Y = 62;

	game->Find_Path( cPoint2D(18,18) , cPoint2D(58,62) , Path);

	
	if(  Path.size() != 4 ) 
	{
		cout << "(001)";
		return false;
	}
	for( Path_itr=Path.begin() ; Path_itr != Path.end() ; Path_itr++)
	{
		p = (*Path_itr) ;		
		if( !(  p.X == path[n].X &&  p.Y == path[n].Y)) 
		{
			
			cout << "(002)";
			return false;
		}
		n++;
	}


	// -----------         test 2 : chemin impossible  ----------------------	
	chemin_trouve = game->Find_Path( cPoint2D(18,18) , cPoint2D(58,58) , Path);
	if( chemin_trouve )
	{
		cout<<  "(003)";
		return false;
	}

	// -----------         test 3 : on recalcule le meme chemin que ds test1  ----------------------	
	// on verifie que les points sont les memes que ds path[]
	n=0;
	Path.clear();
	game->Find_Path( cPoint2D(18,18) , cPoint2D(58,62) , Path);
	if(  Path.size() != 4 )
	{
		cout << "(005)";
		return false;
	}
	for( Path_itr=Path.begin() ; Path_itr != Path.end() ; Path_itr++)
	{
		p = (*Path_itr) ;		
		if( !(  p.X == path[n].X &&  p.Y == path[n].Y)) 
		{
			
			cout << "(006)";
			return false;
		}
		n++;
	}

	// -----------         test 4 : on calcule un autre chemin ----------------------
	// path contient le chemin correct
	path[0].X = 20;
	path[0].Y = 5;

	path[1].X = 60;
	path[1].Y = 5;

	path[2].X = 85;
	path[2].Y = 20;

	path[3].X = 62;
	path[3].Y = 22;

	n=0;
	Path.clear();
	game->Find_Path( cPoint2D(18,18) , cPoint2D(62,22) , Path);
	if(  Path.size() != 4 )
	{
		cout << "(007)";
		return false;
	}
	for( Path_itr=Path.begin() ; Path_itr != Path.end() ; Path_itr++)
	{
		p = (*Path_itr) ;		
		if( !(  p.X == path[n].X &&  p.Y == path[n].Y)) 
		{
			cout << "(008)";
			return false;		
		}
		n++;
	}


	return true;
	
}


/*------------------------------------------------------------------------*/
// verifie que le chemin calcul soit bien le chemin attendu
// interet : le chemin est assez long, et la carte faite de longs murs qui s'entrecoupent
// donc qui font souvent appel  DIVIDER_CROSS lors de la construction du bsp
bool Test_Path_Finder_4()
{
	cPoint2D path[20];

	cGame_World * game = new cGame_World();
	cBSP_Loader loader;
	loader.Load("../test_levels/level_test_pathfinder_4.txt" , game);
	game->Decorate();
	
	

	std::vector<cPoint2D>  Path;
	std::vector<cPoint2D>::iterator  Path_itr;
	game->Find_Path( cPoint2D(55,82) , cPoint2D(63,83) , Path);
	
	path[0].X = 50;
	path[0].Y = 95;

	path[1].X = 30;
	path[1].Y = 90;

	path[2].X = 10;
	path[2].Y = 80;

	path[3].X = 5;
	path[3].Y = 50;

	path[4].X = 30;
	path[4].Y = 5;

	path[5].X = 50;
	path[5].Y = 10;

	path[6].X = 60;
	path[6].Y = 40;

	path[7].X = 90;
	path[7].Y = 5;

	path[8].X = 95;
	path[8].Y = 50;

	path[9].X = 95;
	path[9].Y = 80;

	path[10].X = 90;
	path[10].Y = 95;

	path[11].X = 63;
	path[11].Y = 83;




	if( Path.size() != 12 ) return false;
	
	int n=0;
	cPoint2D p;
	for( Path_itr=Path.begin() ; Path_itr != Path.end() ; Path_itr++)
	{
		p = (*Path_itr) ;		
		if( !(  p.X == path[n].X &&  p.Y == path[n].Y)) return false;
		n++;
	}	

	return true;
}



/*------------------------------------------------------------------------*/
// verifie que le chemin calcul soit bien le chemin attendu
// interet : les points de depart et d'arrive sont sur des portals !!
bool Test_Path_Finder_5()
{
	cPoint2D path[10];

	cGame_World * game = new cGame_World();
	cBSP_Loader loader;
	loader.Load("../test_levels/level_test_pathfinder_5.txt" , game);
	game->Decorate();
	

	std::vector<cPoint2D>  Path;
	std::vector<cPoint2D>::iterator  Path_itr;
	Path.clear();
	game->Find_Path( cPoint2D(3,40) , cPoint2D(85,80) , Path);
	
	path[0].X = 10;
	path[0].Y = 40;

	path[1].X = 30;
	path[1].Y = 92;

	path[2].X = 50;
	path[2].Y = 95;

	path[3].X = 85;
	path[3].Y = 80;


	if( Path.size() != 4 ) return false;
	
	int n=0;
	cPoint2D p;
	for( Path_itr=Path.begin() ; Path_itr != Path.end() ; Path_itr++)
	{		
		p = (*Path_itr) ;		
		//cout <<p << "\n";
		if( !(  p.X == path[n].X &&  p.Y == path[n].Y)) return false;
		n++;
	}	


	return true;
}