#pragma warning(disable: 4786)

/*------------------------------------   Includes   ---------------------------------------------*/
#include "unit.h"
#include "HV_bsp.h"
#include "game_manager.h"
#include "draw_line.h"




/****************************************************************************************************/
// class cUnit
/****************************************************************************************************/
cUnit::cUnit(int x, int y, cSprite_Display_Def * display_def , int s, int r) 
{ 
	Px           = x; 
	Py           = y; 
	pDisplay_Def = display_def;
	Speed        = s;
	Rayon        = r;
	bOn_The_Road = false;
	Current_Path.clear();	
	
	Time_Stamp_Last_Move = DXUtil_Timer( TIMER_GETAPPTIME )-FREQUENCY_MOVE;
};


/*--------------------------------------------------------------------------------------------------*/
void cUnit::React_To_Bonus(cInteractive_Item * bonus)
{
	switch( bonus->Type)
	{
	case TELEPORTEUR:
		React_To_Teleporteur( (cTeleporteur*) bonus );
		return;
	case BOMB:
		React_To_Bomb( (cBomb*) bonus );
		return;
	case HOLOGRAMME:
		React_To_Hologramme( (cHologramme*) bonus );
		return;
	case CADEAU:
		React_To_Cadeau( (cCadeau*) bonus );
		return;
	case RED_TRAP:		
		React_To_Red_Trap( (cRed_Trap*) bonus );
		return;
	case BLUE_TRAP:
		React_To_Blue_Trap( (cBlue_Trap*) bonus );
		return;
	case GREEN_TRAP:
		React_To_Green_Trap( (cGreen_Trap*) bonus );
		return;		
	}
}

/*--------------------------------------------------------------------------------------------------*/
void cUnit::Add_Node_To_Path(cPoint2D& p)
{		
	Current_Path.push_back(p);
}

/*--------------------------------------------------------------------------------------------------*/
bool cUnit::Follow_Path()
{
	cPoint2D node;
	cPoint2D new_pos;
	bool arrived;
	
	if(  Current_Path.empty() )   // si pas de chemin, on retourne 'arriv'
	{		
		return true;
	}
	else if( !bOn_The_Road ) // on est pas encore en route, mais il y a une route a suivre, on demarre
	{		
		bOn_The_Road = true;
		node = Current_Path.front();		
		Bresenham_Algo.Reset( cPoint2D(Px,Py) , node );
		Time_Stamp_Last_Move = DXUtil_Timer( TIMER_GETAPPTIME ) -FREQUENCY_MOVE ;
	}	
	
	// si le temps depuis le dernier pas est suffisant, on fait un pas
	if( DXUtil_Timer( TIMER_GETAPPTIME ) - Time_Stamp_Last_Move >= FREQUENCY_MOVE )
	{
		arrived = Bresenham_Algo.Step( Speed, new_pos) ;	
		Px=new_pos.X;
		Py=new_pos.Y;


		if( Px-Rayon-1 < 0)
		{
			Px = Rayon+1;
			arrived = true;   // comme on change la position manuellement, on stoppe ici car
			                  // cela fausserait l'algo de bresenham
		}
		if( Px+Rayon+1 >=  TX_SCREEN)
		{
			Px = TX_SCREEN - Rayon-1;
			arrived = true;
		}

		if( Py-Rayon-1 < 0)
		{
			Py = Rayon+1;
			arrived = true;
		}
		if( Py+Rayon+1 >=  TY_SCREEN)
		{
			Py = TY_SCREEN - Rayon-1;
			arrived = true;
		}


		assert( Px-Rayon >= 0);
		assert( Py-Rayon >= 0);		
		assert( Px+Rayon < TX_SCREEN);	
		assert( Py+Rayon < TY_SCREEN);

		// si on est arriv au premier noeud du chemin
		// on enleve ce noeud et on s'arrete, le prochain appel a update path reprendra le chemin vers le 
		// prochain noeud s'il existe
		if( arrived )
		{		
			bOn_The_Road = false;
			Current_Path.pop_front();
		}
		Time_Stamp_Last_Move += FREQUENCY_MOVE;
	}

	return false;
}



/*--------------------------------------------------------------------------------------------------*/
void cUnit::Display() 
{ 	
	assert( Px-Rayon >= 0);
	assert( Py-Rayon >= 0);	
	assert( Px+Rayon < TX_SCREEN );
	assert( Py+Rayon < TY_SCREEN);

	// on demande  la classe d'affichage de cette unit de s'afficher
	pDisplay_Def->Display( Px - Rayon , Py - Rayon);
}

/*--------------------------------------------------------------------------------------------------*/
void cUnit::Move(int x_move, int y_move)
{	
	int real_delta_x=0;   // le mvt dsir
	int real_delta_y=0;

	int possible_delta_x=0;
	int possible_delta_y=0;
	
	int final_move_x = 0;
	int final_move_y = 0;
	
	if( DXUtil_Timer( TIMER_GETAPPTIME ) - Time_Stamp_Last_Move >= FREQUENCY_MOVE )
	{	
		if( x_move!=0 && y_move!=0)
		{
			real_delta_x = ((float) x_move)/1.414;
			real_delta_y = ((float) y_move)/1.414;
		}
		else			
		{
			real_delta_x = x_move;
			real_delta_y = y_move;
		}		
		
		// on met ds possible_delta_x et possible_delta_y
		// le deplacement qu'on peut vraiment faire sur chaque axe
		// car le mvt ordonn peut sortir de l'aire de jeu
		if( Px-Rayon+real_delta_x < cGame_World::Ecran.p0.Get_X())		
			possible_delta_x = cGame_World::Ecran.p0.Get_X() + Rayon - Px ;		
		else if( Px+Rayon+real_delta_x >= cGame_World::Ecran.p1.Get_X())		
			possible_delta_x = cGame_World::Ecran.p1.Get_X() - Rayon - Px ;					
		else
			possible_delta_x = real_delta_x;

		if( Py-Rayon+real_delta_y < cGame_World::Ecran.p0.Get_Y())		
			possible_delta_y = cGame_World::Ecran.p0.Get_Y() + Rayon - Py;		
		else if( Py+Rayon+real_delta_y >= cGame_World::Ecran.p1.Get_Y())		
			possible_delta_y = cGame_World::Ecran.p1.Get_Y() - Rayon - Py ;		
		else
			possible_delta_y = real_delta_y;
		
		// on met ds final_move le mouvement qu'il faut essayer d'effectuer
		if( possible_delta_x == 0 && possible_delta_y ==0) // si on ne peut pas bouger, on retourne
			return;
		else if( possible_delta_x == 0) // si on ne peut pas bouger sur x, on bouge de tte la vitesse sur Y
		{
			final_move_x = 0;
			final_move_y = y_move;
		}
		else if( possible_delta_y == 0) // si on ne peut pas bouger sur y, on met toute la vitesse sur x
		{
			final_move_x = x_move;
			final_move_y = 0;
		}
		else  // on peut bouger ds les deux directions
		{		
			final_move_x = x_move;
			final_move_y = y_move;
		}

		
		// on effectue le mouvement
		Px+= final_move_x;
		Py+= final_move_y;

		// on regarde si en effectuant ce mouvement, on est pas sorti de l'ecran
		if( Px-Rayon-1 < cGame_World::Ecran.p0.Get_X())		
			Px = Rayon+1;		
		else if( Px+Rayon >= cGame_World::Ecran.p1.Get_X())		
			Px = cGame_World::Ecran.p1.Get_X() - Rayon -1;		

		if( Py-Rayon-1 < cGame_World::Ecran.p0.Get_Y())		
			Py = Rayon+1;		
		else if( Py+Rayon >= cGame_World::Ecran.p1.Get_Y())				
			Py = cGame_World::Ecran.p1.Get_Y() - Rayon -1;		

		// on verifie qu'on ne rentre pas ds un mur
		if( Game_Manager.Game_World->Do_Ball_Intersect_Walls(Px , Py , Rayon ))
		{
			// si c le cas, on regarde si on peut juste faire le mvt voulue suivant Y
			Px -= final_move_x;  // on essaie le mvt just sur Y
			if( ! Game_Manager.Game_World->Do_Ball_Intersect_Walls(Px , Py , Rayon ))
			{
				Time_Stamp_Last_Move += FREQUENCY_MOVE;
				return;
			}			
			
			// sinon, on regarde si on peut juste faire le mvt voulue suivant X
			Px += final_move_x;  // mvt juste sur X
			Py -= final_move_y;
			if( ! Game_Manager.Game_World->Do_Ball_Intersect_Walls(Px , Py , Rayon ))
			{
				Time_Stamp_Last_Move += FREQUENCY_MOVE;
				return;
			}	
			
			// on revient au point de depart, l'unit ne bouge pas
			Px -= final_move_x;
			Time_Stamp_Last_Move += FREQUENCY_MOVE;
			return ;
		}		

		Time_Stamp_Last_Move += FREQUENCY_MOVE;
	}	
	
}	


/*--------------------------------------------------------------------------------------------------*/
void cUnit::Display_Path()
{
	cPoint2D last_p(this->Px,this->Py);

	// on parcours tous les noeuds en tracant une ligne entre eux
	std::list<cPoint2D>::iterator  chemin_itr;
	for(chemin_itr = Current_Path.begin() ; chemin_itr != Current_Path.end() ;chemin_itr++)
	{
		Draw_Line(g_pDisplay,last_p,(*chemin_itr),0x15);
		last_p = (*chemin_itr);		
	}
}