/***************************************************************************************************/
/***************************************************************************************************/
//! \file unit.h
//! \brief Dfini la class cUnit, classe gnrique des units mobiles circulaires
//!    
//!	Auteur : Adrien Auclair
//!
//! 
/***************************************************************************************************/
/***************************************************************************************************/
#ifndef UNIT_H
#define UNIT_H

/*------------------------------------  Prototypes  ---------------------------------------------*/
class cUnit;

/*------------------------------------   Includes   ---------------------------------------------*/
#include <windows.h>
#include <assert.h>
#include <list>

#include "main.h"
#include "geo_utils.h"
#include "bresenham.h"
#include "bonus.h"

#define FREQUENCY_MOVE (float)0.01    // periodicite des mouvements ( en secs);

/*-------------------------------------   Classes   ---------------------------------------------*/


/*************************************************************************************************/
//! classe gnrique des units mobiles du jeu
/*! La class cUnit est une classe gnrique pour reprsenter toute unit mobile du jeu \n
//! Pour ce jeu, toutes les units sont dfinis par un point 2D (x,y) qui reprsente leur centre \n
//! Elles ont aussi un rayon, qui dfini leur emplacement au sol ( pour les collisions ) \n
//! Chaque elt a un pointeur vers un objet cAbtract_Display_Def qui dfini sa mthode d'affichage \n
**************************************************************************************************/
class cUnit
{
	friend class cGame_Manager;
public :
	/// constructeur 
	/**
	* x,y     : position de l'unit  rajouter \n
	* speed   : vitesse de l'unit \n
	* rayon   : rayon de l'unit \n
	* display_def : pointeur sur le modele d'affichage a utiliser pour cette unit
	*/
	cUnit(int x, int y, cSprite_Display_Def * display_def , int speed, int rayon);
	
	inline cSprite_Display_Def * Get_Display_Def() {return pDisplay_Def;};
	inline Get_Speed() { return Speed;};
	inline Get_Rayon() { return Rayon;};	
	
	/// affiche l'unit ( via la classe d'affichage )
	void Display();	
	
	/// bouge la unit selon le vecteur (x_move, y_move)
	void Move(int x_move, int y_move);

	/// ajoute un noeud au chemin en cours
	void Add_Node_To_Path(cPoint2D& p);

	/// avance d'un pas dans le chemin en cours, retourne vrai si arriv, false sinon
	bool Follow_Path();

	/// reagit au contact du 'bonus'
	void React_To_Bonus(cInteractive_Item * bonus);
	
	// affiche les chemins des units, utile pour le debug
	void Display_Path();
	
	int Px,Py;	                             ///< position de l'unit	
	std::list<cPoint2D>  Current_Path;       ///< chemin  suivre par l'unit
	bool bOn_The_Road;                       ///< indique que l'unit est en train de suivre un chemin

protected:	
	cSprite_Display_Def * pDisplay_Def;      ///< classe de definition de l'affichage	
	int Speed;                               ///< vitesse de l'unit
	int Rayon;                               ///< rayon de l'emplacement au sol
	cIncremental_Bresenham Bresenham_Algo;   ///< algorithme de bresenham, utile pour aller d'un point  un autre
	float Time_Stamp_Last_Move;              ///< moment auquel on a effectu le dernier mouvement
	

	
	/*------------------------- fonctions d'IA ----------------------------------*/
	// notes : il serait plus propre de faire un tableau de fonctions
	// indexes par le type de bonus auquel reagir
	// mais il semble qu'on ne puisse pas faire de tableau de fonctions membres
	// cela oblige donc  dclarer des fonctions C...
	// je prefere donc dclarer des fonctions qui ne font rien ( en inline )
	inline virtual void React_To_Teleporteur(cTeleporteur * teleporteur){}
	inline virtual void React_To_Bomb(cBomb * bomb){};
	inline virtual void React_To_Hologramme(cHologramme * hologramme){};
	inline virtual void React_To_Cadeau(cCadeau * cadeau){};
	inline virtual void React_To_Red_Trap(cRed_Trap * red_trap){};
	inline virtual void React_To_Blue_Trap(cBlue_Trap * blue_trap){};
	inline virtual void React_To_Green_Trap(cGreen_Trap * green_trap){};	
};




#endif